; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION BACK_EVAL_BASIS,  BEG_T, END_T

; Michael S. Briggs, 17 May 1993, UAH / MSFC ES-66.
; This function can only be run after calling BACK_LFIT, which places info
; into the common block.

; Evaluates the basis functions of the background model.   The basis functions
; are the various polymonials (1, t, t^2 ...) averaged over the data time
; intervals.
; Revised 19 November 1993 by MSB: converted calculation to double precision--
; returned argument is still single precision.

; Input arguments:
; BEG_T -- begining times of the intervals (which might be the data intervals),
;       -- floating vector (time interval).
; END_T -- ending times of the intervals (which might be the data itervals),
;       -- floating vector (time interval).

; Other inputs: via common: see BACK_LFIT for descriptions:
COMMON BACK_COM,  BACK_ORDER, NUM_FIT_CHAN, COEFF, COVAR, BACK_CHISQ,     $
                  BEG_BCHAN, END_BCHAN

; Output: via function return:
; AFUNC -- the various basis func. values for each of the data time intervals,
;       -- floating array (time interval, coefficient).


; ******************************************************************************

NUM_TIMES = N_ELEMENTS (BEG_T)
AFUNC = FLTARR (NUM_TIMES, BACK_ORDER+1)                 
                            
DBEG_T = DOUBLE (BEG_T)
DEND_T = DOUBLE (END_T)

FOR JCOEFF=0, BACK_ORDER DO BEGIN
   AFUNC (*, JCOEFF) =                                    $
     (DEND_T^(JCOEFF+1) - DBEG_T^(JCOEFF+1)) / ((JCOEFF+1) * (DEND_T - DBEG_T))
ENDFOR

RETURN, AFUNC
END



; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION BACK_EVAL_MODEL,   BEG_T, END_T

; Michael S. Briggs, 17 May 1993, UAH / MSFC ES-66.
; This function can only be run after calling BACK_LFIT, which places info
; into the common block.
; Revised 18 Nov 1993 by MSB: add warning message for negative model.
; Revised 19 November 1993 by MSB: converted calculation to double precision--
; returned argument is still single precision.

; Evaluates the fitting funtion for each of the NUM_TIMES time intervals.
; The fitting function is the mean over each interval of a polynomial.
; The polynomial order and coefficients are obtained from common.

; Input arguments:
; BEG_T -- begining times of the intervals (which might be the data intervals),
;       -- floating vector (time interval).
; END_T -- ending times of the intervals (which might be the data intervals),
;       -- floating vector (time interval).

; Other inputs: via common: see BACK_LFIT for descriptions:
COMMON BACK_COM,  BACK_ORDER, NUM_FIT_CHAN, COEFF, COVAR, BACK_CHISQ,      $
                  BEG_BCHAN, END_BCHAN

; Output: via function return:
; BACK_FIT -- the fitting function = mean over each interval of polynomial,
;          -- floating array (time interval, energy channel).

;*******************************************************************************
           
NUM_TIMES = N_ELEMENTS (BEG_T)
BACK_FIT = FLTARR (NUM_TIMES, NUM_FIT_CHAN)
                            
DBEG_T = DOUBLE (BEG_T)
DEND_T = DOUBLE (END_T)              
                              
;FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
;   FOR JCOEFF=0, BACK_ORDER DO BEGIN
;      BACK_FIT (*,KCHAN) = BACK_FIT (*,KCHAN) +     $
;          COEFF (JCOEFF,KCHAN) * (DEND_T^(JCOEFF+1) - DBEG_T^(JCOEFF+1)) /   $
;                       ((JCOEFF+1) * (DEND_T - DBEG_T))
;   ENDFOR
;ENDFOR

FOR JCOEFF=0, BACK_ORDER DO BEGIN
    BACK_FIT (*, BEG_BCHAN:END_BCHAN) = BACK_FIT (*, BEG_BCHAN:END_BCHAN) + $
                (DEND_T(*)^(JCOEFF+1) - DBEG_T(*)^(JCOEFF+1)) /   $
                ((JCOEFF+1) * (DEND_T(*) - DBEG_T(*))) # $
                COEFF (JCOEFF, BEG_BCHAN:END_BCHAN)
ENDFOR

;IF MIN (BACK_FIT) LT 0.0 THEN BEGIN
;   PRINT, ' '
;   PRINT, '*** BACK_FIT WARNING: Background model is NEGATIVE somewhere!'
;   PRINT, ' '
;ENDIF

RETURN, BACK_FIT
END


                      
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION BACK_EVAL_DIFF,   TIMES

; Michael S. Briggs, 17 May 1993, UAH / MSFC ES-66.
; This function can only be run after calling BACK_LFIT, which places info
; into the common block.
; Revised 19 November 1993 by MSB: converted calculation to double precision--
; returned argument is still single precision.

; Evaluates the differential funtion for the requested times.
; The "differential" function is a polynomial.
; The fitting function is the mean over each interval of the polynomial.
; The polynomial order and coefficients are obtained from common.

; Input argument:
; TIMES --times to evaluate polynomial for.Intended to be relative to trig time,
;       -- floating vector ("continous" time index).

; Other inputs: via common: see BACK_LFIT for descriptions:
COMMON BACK_COM,  BACK_ORDER, NUM_FIT_CHAN, COEFF, COVAR, BACK_CHISQ,     $
                  BEG_BCHAN, END_BCHAN

; Output: via function return:
; BACK_DIFF -- the polynomial values,
;           -- floating array ("continuous" time index, energy channel).

;*******************************************************************************
           
BACK_DIFF = FLTARR (N_ELEMENTS (TIMES), NUM_FIT_CHAN)

FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
   BACK_DIFF (*, KCHAN) = POLY (DOUBLE(TIMES), COEFF (*, KCHAN))
ENDFOR


RETURN, BACK_DIFF
END



; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION BACK_EVAL_MSIG,  BEG_T, END_T

; Michael S. Briggs, 17 May 1993, UAH / MSFC ES-66.
; This function can only be run after calling BACK_LFIT, which places info
; into the common block.

; figure the uncertainty in the model predicted rates for the data time
; intervals, based upon the uncertainty in the model coefficients.   See my
; (MSB) memo of 21 March 1993, "Ideas on Background Models, ...".   It is
; particularly easy for this model since it is linear in its parameters.

; Input arguments:
; BEG_T -- begining times of the intervals (which might be the data intervals),
;       -- floating vector (time interval).
; END_T -- ending times of the intervals (which might be the data intervals),
;       -- floating vector (time interval).

; Other inputs: via common: see BACK_LFIT for descriptions:
COMMON BACK_COM,  BACK_ORDER, NUM_FIT_CHAN, COEFF, COVAR, BACK_CHISQ,      $
                  BEG_BCHAN, END_BCHAN

; Output: via function return:
; MODEL_SIG -- the uncertainty in the model predicted rates, based upon the
;              uncertainties of the model parameters/coefficients in COVAR,
;           -- floating array (time interval, energy channel).

; ******************************************************************************

;get the values of the basis functions for each time interval:
AFUNC = BACK_EVAL_BASIS (BEG_T, END_T)
     
NUM_TIMES = N_ELEMENTS (BEG_T)
MODEL_SIGMA = FLTARR (NUM_TIMES, NUM_FIT_CHAN)

; formal propagation of errors of fit coefficients to errors of model rates:
;FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
;   FOR ITIME=0, NUM_TIMES-1 DO BEGIN
;      MODEL_SIGMA (ITIME,KCHAN) =   $
;      SQRT (AFUNC (ITIME,*) # (COVAR (*,*,KCHAN) # TRANSPOSE (AFUNC (ITIME,*))))
;   ENDFOR
;ENDFOR

; Leftmost dimension first for large arrays!
FOR ITIME=0, NUM_TIMES-1 DO BEGIN
    FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
        MODEL_SIGMA (ITIME,KCHAN) =   $
        AFUNC (ITIME,*) # (COVAR (*,*,KCHAN) # TRANSPOSE (AFUNC (ITIME,*)))
    ENDFOR
ENDFOR
MODEL_SIGMA = SQRT(MODEL_SIGMA)

; If the fit is systematically bad, as indicated by reduced chisq > 1, increase
; the model rate errors to reflect the inadequate fit:
FACTOR = SQRT (BACK_CHISQ > 1)
FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
   MODEL_SIGMA (*,KCHAN) = MODEL_SIGMA (*,KCHAN) * FACTOR (KCHAN)
ENDFOR

;MODEL_SIGMA (*, BEG_BCHAN:END_BCHAN) = MODEL_SIGMA (*, BEG_BCHAN:END_BCHAN) # $
;                                      TRANSPOSE(FACTOR (BEG_BCHAN:END_BCHAN))


RETURN, MODEL_SIGMA
END



; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO BACK_LFIT,  REQ_ORDER, BEG_T, END_T, RATES, LIVE_TIME,      $
                FIRST_BACK_CHAN, LAST_BACK_CHAN, RET_BACK_CHISQ, BACK_FIT

; Michael S. Briggs, 17 May 1993, UAH / MSFC ES-66.

; Adapted from Numerical Recipes subroutine LFIT in order do linear fitting
; to the background data.   The background data are the mean rates for the 
; various data intervals.  The "differential" fitting function is a polynomial
; of order REQ_ORDER.   The actual fitting function is the mean of the 
; differential fitting function over the data intervals.
; The subroutine LFIT has been considerably simplified by removing the option
; of fixing some of the parameters--in this version you can only choose the
; order of the polynomial.  It has also been hardcoded to this particular
; application.  Model variances are used for chisq via two fitting passes.

; Revisions 15 June 1993 by MSB:
;   1) allow REQ_ORDER from 0 to 5, 2) slight special case to handle order 0,
;   3) declare all common arrays at their maximum size in order so that 1st
;      call doesn't determine maximum size for succeding calls.
; Revisions 16 & 17 June 1993 by MSB:
;   1) fix so that it doesn't fail for energy channels with zero rates for
;      all time intervals,
;   2) fix wrong chisq calculation.
; Revisions 6 August 1993 by MSB:
;   add use of WHERE (VARIANCE) to avoid divide by 0 for channels with 0 rates.
; Revisions 17 & 18 November 1993 by MSB:
;   1) change to WHERE (VARIANCE GT 0.0) to avoid negative variances.  Such
;      are impossible & illegal but arise (esp. on 2nd pass) from negative
;      model rates, which are also unphysical.   This is a bit of a cheat--
;      what is really needed is a model which must be nonnegative ala Loredo.
;   2) lower largest allowed order REQ_ORDER to 4.
;   3) added arguments FIRST_BACK_CHAN & LAST_BACK_CHAN (which are copied to 
;      common as BEG_BCHAN & END_BCHAN).   These specify the channel range
;      (as array index starting with zero) for which the user wants the
;      background model.   This saves run time by skipping channels not of
;      interest.
; Revisions 19 November 1993 by MSB: converted calculation to double precision--
; returned argument is still single precision.


; Input arguments:
; IMPORTANT!: FOR NUMERICAL STABILITY THE TIMES SHOULD BE RELATIVE TO THE 
; TRIGGER TIME.
; REQ_ORDER -- the requested order of the polynomial,
;           -- integer scaler.
; BEG_T -- beginging times of the data intervals,
;       -- floating vector (time interval).
; END_T -- ending times of the data intervals,
;       -- floating vector (time interval).
; RATES -- the data count rates, UNITS MUST BE COUNTS / SEC, not counts/s-keV !!
;          These units are assumed in the calculation of variances using
;          Poission statistics.
;       -- floating array (time interval, energy channel).
; LIVE_TIME -- the live time of each accumulation interval,
;           -- floating vector (time interval).
; FIRST_BACK_CHAN -- first channel for which user wants background calculated,
;                    this is an array index starting at 0,
;                 -- integer scaler.
; LAST_BACK_CHAN -- first channel for which user wants background calculated,
;                   this is an array index w. max allowed value = # of chan - 1,
;                -- integer scaler.
                                                          
; Output arguments:
; RET_BACK_CHISQ -- reduced chisq of the fit,
;                -- floating vector (energy channel).
; BACK_FIT -- the model predicted rates for each of the data intervals--in a
;             perfect world these would equal the data rates.
;          -- floating array (time interval, energy channel)


; Internal "Output": in common: stored for access by other BACK_xxx subroutines:
COMMON BACK_COM,  BACK_ORDER, NUM_FIT_CHAN, COEFF, COVAR, BACK_CHISQ,      $
                  BEG_BCHAN, END_BCHAN

; BACK_ORDER -- polynomial order of the differential model,
;            -- integer scalar.
; NUM_FIT_CHAN -- number of energy channels for which fit was done,
;              -- integer scalar.
; COEFF -- the polynomial coefficient of the differential model,
;       -- double vector (energy channel).
; COVAR -- the covariance matrix of the fits to each energy channel,
;       -- double array (coefficient, coefficient, energy channel)
; BACK_CHISQ -- reduced chisq of the fit,
;            -- floating vector (energy channel).
; BEG_BCHAN -- first channel (i.e., index) for which background model is wanted,
;           -- integer scalar.
; END_BCHAN -- last channel (i.e., index) for which background model is wanted,
;           -- integer scalar.
; ******************************************************************************


NUM_TIMES = N_ELEMENTS (BEG_T)                            ; # of time intervals

; test legality of order:
IF REQ_ORDER LT 0 OR REQ_ORDER GE NUM_TIMES - 1 OR REQ_ORDER GT 4 THEN BEGIN
   PRINT, "## ERROR: BACK_LFIT: illegal requested order:", REQ_ORDER
   BACK_ORDER = 0
   RETURN
ENDIF
IF MIN (LIVE_TIME) LE 0.0 THEN     $
   PRINT, "## ERROR: BACK_LFIT: NON-POSITIVE LIVETIME"


NUM_FIT_CHAN = N_ELEMENTS (RATES) / NUM_TIMES    ; copy # channels fitted to com
BACK_ORDER = REQ_ORDER                               ; copy order to common
BEG_BCHAN = FIRST_BACK_CHAN                          ; copy to common
END_BCHAN = LAST_BACK_CHAN                           ; copy to common


; test legality of channel range:
IF FIRST_BACK_CHAN LT 0 OR FIRST_BACK_CHAN GT LAST_BACK_CHAN OR       $
   LAST_BACK_CHAN GT NUM_FIT_CHAN - 1 THEN BEGIN
   PRINT, '## ERROR: BACK_LFIT: illegal requested channel range:'
   PRINT, FIRST_CHAN_CHAN, LAST_BACK_CHAN, ' vs', 0, NUM_FIT_CHAN - 1
   RETURN
ENDIF


; reserve common variables at maximum size based upon maximum allowed order of
; four so that first call doesn't restrict latter calls as to allowed order.
; however, latter calls still cannot increase number of fit channels.
COVAR = DBLARR (5, 5, NUM_FIT_CHAN)
COEFF = DBLARR (5, NUM_FIT_CHAN)

; setup arrays:
COVAR = DBLARR (BACK_ORDER+1, BACK_ORDER+1, NUM_FIT_CHAN)
COEFF = DBLARR (BACK_ORDER+1, NUM_FIT_CHAN)
BACK_CHISQ = FLTARR (NUM_FIT_CHAN)
WT = DBLARR (BACK_ORDER+1, NUM_TIMES)   

; get the values of the basis functions for each time interval:
AFUNC = BACK_EVAL_BASIS (BEG_T, END_T)


FOR LPASS=1, 2 DO BEGIN

   FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
       IF MAX (RATES (*,KCHAN)) GT 0.0 THEN BEGIN                   ; have data?
 
         ; set up weight matrix used to create covariance matrix:
         ; how to calculate variance: 1) rate * livetime = counts, 2) Poission
         ; statistics ==> variance of counts = counts, 3) variance of rate = 
         ; variance of counts / livetime^2:
         IF LPASS EQ 1 THEN BEGIN
            ; pass 1 ==> data variances calculated from data rates and livetime:
            VARIANCE = RATES (*,KCHAN) / LIVE_TIME 
            INDEX = WHERE (VARIANCE GT 0.0)
            FOR JCOEFF=0, BACK_ORDER DO BEGIN
               WT (JCOEFF, INDEX) = AFUNC (INDEX, JCOEFF) / VARIANCE (INDEX)
            ENDFOR
         ENDIF ELSE BEGIN
            ; pass 2 ==> data variances calc. from model rates (1st pass) & 
            ; livetime:
            VARIANCE = BACK_FIT (*,KCHAN) / LIVE_TIME
            INDEX = WHERE (VARIANCE GT 0.0)
            IF INDEX (0) NE -1 THEN BEGIN
               FOR JCOEFF=0, BACK_ORDER DO BEGIN
                  WT (JCOEFF, INDEX) = AFUNC (INDEX, JCOEFF) / VARIANCE (INDEX)
               ENDFOR
            ENDIF ELSE BEGIN
               PRINT, ' '
               PRINT, '*** BACK_FIT: SEVERE ERROR: Background model is Negative'
               PRINT, '    for all time intervals of energy index', KCHAN
               PRINT, ' '
               FOR JCOEFF=0, BACK_ORDER DO BEGIN
                  WT (JCOEFF, *) = 1.                     ; arbitary
               ENDFOR
            ENDELSE
         ENDELSE

         ; set up covariance matrix:
         COVAR (*,*,KCHAN) = WT # AFUNC          

         ; set up coefficient matrix:
         COEFF (*,KCHAN) = WT # RATES (*,KCHAN)

         ; get the matrix solution: need both inverse of COVAR & inverse
         ; times COEFF:
         IF REQ_ORDER GE 1 THEN BEGIN
            COVAR (*,*,KCHAN) = INVERT (COVAR (*,*,KCHAN))
         ENDIF ELSE BEGIN
            COVAR (0, 0, KCHAN) = 1. / COVAR (0, 0, KCHAN)
         ENDELSE
         COEFF (*,KCHAN) = COVAR (*,*,KCHAN) # COEFF (*,KCHAN)

      ENDIF
   ENDFOR                                                           ; kchan

   ; get the model rates:
   BACK_FIT = BACK_EVAL_MODEL (BEG_T, END_T)
ENDFOR                                                              ; lpass

; figure reduced chisq:
;FOR KCHAN=BEG_BCHAN, END_BCHAN DO BEGIN
;   IF MAX (RATES (*,KCHAN)) GT 0.0 THEN BEGIN                   ; have data?
;      VARIANCE = BACK_FIT (*,KCHAN) / LIVE_TIME
;      DUMMY = (RATES (*,KCHAN) - BACK_FIT (*,KCHAN)) ^2
;      BACK_CHISQ (KCHAN) = TOTAL (DUMMY / VARIANCE)
;   ENDIF
;ENDFOR
IF MAX (RATES (*, BEG_BCHAN:END_BCHAN)) GT 0.0 THEN BEGIN          ; have data?
   VARIANCE = BACK_FIT (*, BEG_BCHAN:END_BCHAN) / LIVE_TIME
   DUMMY = (RATES (*, BEG_BCHAN:END_BCHAN) - BACK_FIT (*, BEG_BCHAN:END_BCHAN)) ^2
   BACK_CHISQ (BEG_BCHAN:END_BCHAN) = TOTAL (DUMMY / VARIANCE)
ENDIF
DOF = NUM_TIMES - (BACK_ORDER + 1)
BACK_CHISQ = BACK_CHISQ / DOF
RET_BACK_CHISQ = BACK_CHISQ                     ; copy from common to argument

RETURN
END
