; These FITS routines were copied from the IDL Astronomy User's Library
;  available from the Goddard Space Flight Center:
;
;  Anonymous FTP: idlastro.gsfc.nasa.gov
;  WWW:           http://idlastro.gsfc.nasa.gov/homepage.html
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function since_version, release
;+
; NAME:
;	SINCE_VERSION
;
; PURPOSE:
;	Determine if the current release of IDL (as given in the 
;	!VERSION.RELEASE system variable) comes after the user specified 
;	release.   
;
; CALLING SEQUENCE:
;	test = SINCE_VERSION( release )
;
; INPUT:
;	release - scalar string, must be formatted exactly like the 
;		!VERSION.RELEASE system variable (e.g. '3.0.0') 
;
; OUTPUT:
;	test - 1 if current release is identical or later than the specified 
;              'release' else 0
;
; EXAMPLE:
;	Use the /FTOXDR keyword to the BYTEORDER procedure if the current 
;	release of IDL is 2.2.2 or later
;
;	IDL> if since_version('2.2.2') then byteorder, a, /FTOXDR
;
; REVISION HISTORY:
;	Written   Wayne Landsman         Hughes/STX        January, 1992
;	Corrected algorithm     W. Landsman                April, 1992
;-
 On_error,2

 if N_params() EQ 0 then begin
    print, 'Syntax -  test = SINCE_VERSION( release )
    return, -1
 endif

 release = strtrim( release, 2)
 v1 = strmid( release, 0 ,1)
 v2 = strmid( release, 2, 1)
 v3 = strmid( release, 4, 1)

 c1 = strmid( !VERSION.RELEASE, 0, 1)
 c2 = strmid( !VERSION.RELEASE, 2, 1)
 c3 = strmid( !VERSION.RELEASE, 4, 1)

 if c1 EQ v1 then begin

       if c2 EQ v2 then return, (c3 GE v3) else return, (c2 GT v2)

 endif else return, (c1 GT v1)

 end
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	FUNCTION FXBFINDLUN, UNIT
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBFINDLUN()
; Purpose     : 
;	Find logical unit number UNIT in FXBINTABLE common block.
; Explanation : 
;	Finds the proper index to use for getting information about the logical
;	unit number UNIT in the arrays stored in the FXBINTABLE common block.
;	Called from FXBCREATE and FXBOPEN.
; Use         : 
;	Result = FXBFINDLUN( UNIT )
; Inputs      : 
;	UNIT	= Logical unit number.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	The result of the function is an index into the FXBINTABLE common
;	block.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	None.
; Calls       : 
;	None.
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	None.
; Side effects: 
;	If UNIT is not found in the common block, then it is added to the
;	common block.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	William Thompson, Feb. 1992.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
;	Version 2, William Thompson, GSFC, 21 July 1993.
;		Added DHEAP variable to fix bug with variable length arrays.
; Version     : 
;	Version 2, 21 July 1993.
;-
;
@fxbintbl.pro

	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE,	$
		'Syntax:  ILUN = FXBFINDLUN( UNIT )'
;
;  If the common block hasn't been initialized yet, then initialize it.
;
	IF N_ELEMENTS(LUN) EQ 0 THEN BEGIN
		LUN     = UNIT
		STATE	= 0
		HEAD	= ''
		MHEADER	= 0L
		NHEADER = 0L
		NAXIS1  = 0L
		NAXIS2  = 0L
		TFIELDS = 0L
		HEAP	= 0L
		DHEAP	= 0L
		BYTOFF  = 0L
		TTYPE	= ''
		FORMAT	= ''
		IDLTYPE	= 0
		N_ELEM	= 0L
		TSCAL	= 1.
		TZERO	= 0.
		MAXVAL	= 0L
		N_DIMS	= INTARR(9,2)
		ILUN = 0
;
;  Otherwise, find the logical unit number in the common block.  If not found,
;  then add it.
;
	END ELSE BEGIN
		ILUN = WHERE(LUN EQ UNIT,NLUN)
		ILUN = ILUN(0)
		IF NLUN EQ 0 THEN BEGIN
			LUN     = [LUN,UNIT]
			STATE	= [STATE,  0]
			BOOST_ARRAY,HEAD,''
			MHEADER = [MHEADER,0]
			NHEADER = [NHEADER,0]
			NAXIS1  = [NAXIS1, 0]
			NAXIS2  = [NAXIS2, 0]
			TFIELDS = [TFIELDS,0]
			HEAP	= [HEAP,   0]
			DHEAP	= [DHEAP,  0]
			BOOST_ARRAY,BYTOFF,0
			BOOST_ARRAY,TTYPE,''
			BOOST_ARRAY,FORMAT,''
			BOOST_ARRAY,IDLTYPE,0
			BOOST_ARRAY,N_ELEM,0
			BOOST_ARRAY,TSCAL,1.
			BOOST_ARRAY,TZERO,0.
			BOOST_ARRAY,MAXVAL,0
			BOOST_ARRAY,N_DIMS,INTARR(9,2)
			ILUN = N_ELEMENTS(LUN)-1
		ENDIF
	ENDELSE
;
;  Return the index into the common block arrays.
;
	RETURN,ILUN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function gettok,st,char
;+
; NAME:
;	GETTOK                                    
; PURPOSE:
;	Function to retrieve the first part of the string
;	until the character char is encountered.
;
; CALLING SEQUENCE:
;	token = gettok( st, char )
;
; INPUT:
;	char - character separating tokens, scalar string
;
; INPUT-OUTPUT:
;	st - (scalar) string to get token from (on output token is removed)
;
; OUTPUT:
;	token - scalar string value is returned 
;
; EXAMPLE:
;	If ST is 'abc=999' then gettok(ST,'=') would return
;	'abc' and ST would be left as '999' 
;
; HISTORY
;	version 1  by D. Lindler APR,86
;	Remove leading blanks    W. Landsman (from JKF)    Aug. 1991
;-
;----------------------------------------------------------------------
  On_error,2                           ;Return to caller

; if char is a blank treat tabs as blanks

  tab = string(9b)
  while strpos(st,tab) GE 0 do begin    ;Search for tabs
	pos = strpos(st,tab)
	strput,st,' ',pos
  endwhile

  st = strtrim(st,1)              ;Remove leading blanks

; find character in string

  pos = strpos(st,char)
  if pos EQ -1 then begin         ;char not found?
	token = st
 	st = ''
 	return, token
  endif

; extract token

 token = strmid(st,0,pos)
 len = strlen(st)
 if pos EQ (len-1) then st = '' else st = strmid(st,pos+1,len-pos-1)

;  Return the result.

 return,token
 end
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function strnumber, st, val
;+
; NAME:
;	STRNUMBER
; PURPOSE:
;	Function to determine if a string is a valid numeric value.
;
; CALLING SEQUENCE:
;	result = strnumber( st, [val] )
;
; INPUTS:
;	st - any IDL scalar string
;
; OUTPUTS:
;	1 is returned as the function value if the string st has a
;	valid numeric value, otherwise, 0 is returned.
;
; OPTIONAL OUTPUT:
;	val - (optional) value of the string.  real*8
;
; WARNING:
;	(1)   In V2.2.2 there was a bug in the IDL ON_IOERROR procedure that
;	      will cause the following statement to hang up IDL
;
;	      IDL> print,'' + string( strnumber('xxx') )
;	      This bug was fixed in V2.3.0
;	(2)   In V2.3.2, an IDL bug is seen in the following statements 
;	      IDL> st = 'E'
;	      IDL> q = strnumber(st)  & print,st
;	      The variable 'st' gets modified to an empty string.   This problem
;	      is related to the ambiguity of whether 'E' is a number or not 
;	      (could be = 0.0E).    This bug was fixed in V3.0.0
;	(3)   STRNUMBER was modified in February 1993 to include a special 
;	      test for empty or null strings, which now returns a 0 (not a 
;	      number).     Without this special test, it was found that a
;	      empty string (' ') could corrupt the stack.
; HISTORY:
;	version 1  By D. Lindler Aug. 1987
;       test for empty string, W. Landsman          February, 1993
;-
 if N_params() EQ 0 then begin
      print,'Syntax - result = strnumber( st, [val] )
      return, 0
 endif

 newstr = strtrim( st )

 if ( newstr EQ '' ) then return, 0    ;Empty string is a special case

 On_IOerror, L1			;Go to L1 if conversion error occurs

 val = double( newstr )
 return, 1			;No conversion error

 L1: return, 0			;Conversion error occured

 end
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	FUNCTION FXPAR, HDR, NAME, ABORT, COUNT=MATCHES, COMMENT=COMMENTS
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXPAR()
; Purpose     : 
;	Obtain the value of a parameter in a FITS header.
; Explanation : 
;	The first 8 chacters of each element of HDR are searched for a match to
;	NAME.  If the keyword is one of those allowed to take multiple values
;	("HISTORY", "COMMENT", or "        " (blank)), then the value is taken
;	as the next 72 characters.  Otherwise, it is assumed that the next
;	character is "=", and the value (and optional comment) is then parsed
;	from the last 71 characters.  An error occurs if there is no parameter
;	with the given name.
;
;	Complex numbers are recognized as two numbers separated by one or more
;	space characters.
;
;	If a numeric value has no decimal point (or E or D) it is returned as
;	type LONG.  If it contains more than 8 numerals, or contains the
;	character 'D', then it is returned as type DOUBLE.  Otherwise it is
;	returned as type FLOAT
; Use         : 
;	Result = FXPAR( HDR, NAME  [, ABORT ] )
;
;	Result = FXPAR(HEADER,'DATE')		;Finds the value of DATE
;	Result = FXPAR(HEADER,'NAXIS*')		;Returns array dimensions as
;						;vector
; Inputs      : 
;	HDR	= FITS header string array (e.g. as returned by FXREAD).  Each
;		  element should have a length of 80 characters
;	NAME	= String name of the parameter to return.  If NAME is of the
;		  form 'keyword*' then an array is returned containing values
;		  of keywordN where N is an integer.  The value of keywordN
;		  will be placed in RESULT(N-1).  The data type of RESULT will
;		  be the type of the first valid match of keywordN found.
; Opt. Inputs : 
;	ABORT	= String specifying that FXPAR should do a RETALL if a
;		  parameter is not found.  ABORT should contain a string to be
;		  printed if the keyword parameter is not found.  If not
;		  supplied, FXPAR will return with a negative !err if a keyword
;		  is not found.
; Outputs     : 
;	The returned value of the function is the value(s) associated with the
;	requested keyword in the header array.
;
;	If the parameter is complex, double precision, floating point, long or
;	string, then the result is of that type.  Apostrophes are stripped from
;	strings.  If the parameter is logical, 1 is returned for T, and 0 is
;	returned for F.
;
;	If NAME was of form 'keyword*' then a vector of values are returned.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	COUNT	= Optional keyword to return a value equal to the number of
;		  parameters found by FXPAR.
;	COMMENTS= Array of comments associated with the returned values.
; Calls       : 
;	GETTOK, STRNUMBER
; Common      : 
;	None.
; Restrictions: 
;	None.
; Side effects: 
;	Keyword COUNT returns the number of parameters found.
;
;	The system variable !err is set to -1 if parameter not found, 0 for a
;	scalar value returned.  If a vector is returned it is set to the number
;	of keyword matches found.
;
;	If a keyword occurs more than once in a header, a warning is given,
;	and the first occurence is used.  However, if the keyword is "HISTORY",
;	"COMMENT", or "        " (blank), then multiple values are returned.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	DMS, May, 1983, Written.
;	D. Lindler Jan 90 added ABORT input parameter
;	J. Isensee Jul,90 added COUNT keyword
;	W. Thompson, Feb. 1992, added support for FITS complex values.
;	W. Thompson, Oct. 1992, rewrote to change strategy for extracting
;		values to allow for non-standard formats and renamed to FXPAR.
;		Added COMMENT keyword.
; Written     : 
;	David M. Stern, RSI, May 1983.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;------------------------------------------------------------------------------
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 2 THEN BEGIN
	    PRINT,'Syntax:  result =  FXPAR( HDR, NAME  [, ABORT ])'
	    RETURN, -1
	ENDIF
;
;  Determine the abort condition.
;
	VALUE = 0
	IF N_PARAMS() LE 2 THEN BEGIN
	    ABORT_RETURN = 0
	    ABORT = 'FITS Header'
	END ELSE ABORT_RETURN = 1
	IF ABORT_RETURN THEN ON_ERROR,1 ELSE ON_ERROR,2
;
;  Check for valid header.  Check header for proper attributes.
;
	S = SIZE(HDR)
	IF ( S(0) NE 1 ) OR ( S(2) NE 7 ) THEN $
	    MESSAGE,'FITS Header (first parameter) must be a string array'
;
;  Convert the selected keyword NAME to uppercase.
;
	NAM = STRTRIM( STRUPCASE(NAME) )
;
;  Determine if NAME is of form 'keyword*'.  If so, then strip off the '*', and
;  set the VECTOR flag.
;
	IF STRPOS( NAM, '*' ) EQ STRLEN( NAM ) - 1 THEN BEGIN    
	    NAM = STRMID( NAM, 0, STRLEN( NAM ) - 1)  
	    VECTOR = 1				;Flag for vector output  
	    NAME_LENGTH = STRLEN(NAM)  		;Length of name 
	    NUM_LENGTH = 8 - NAME_LENGTH 	;Max length of number portion  
	    IF NUM_LENGTH LE 0 THEN MESSAGE,	$
		'Keyword length must be 8 characters or less'
;
;  Otherwise, extend NAME with blanks to eight characters.
;
	ENDIF ELSE BEGIN
	    WHILE STRLEN(NAM) LT 8 DO NAM = NAM + ' '
	    VECTOR = 0
	ENDELSE
;
;  If of the form 'keyword*', then find all instances of 'keyword' followed by
;  a number.  Store the positions of the located keywords in NFOUND, and the
;  value of the number field in NUMBER.
;
	KEYWORD = STRMID( HDR, 0, 8)
	IF VECTOR THEN BEGIN
	    NFOUND = WHERE(STRPOS(KEYWORD,NAM) GE 0, MATCHES)
	    IF ( MATCHES GT 0 ) THEN BEGIN
		NUMST= STRMID(HDR(NFOUND), NAME_LENGTH, NUM_LENGTH)
		NUMBER = INTARR(MATCHES)-1
		FOR I = 0, MATCHES-1 DO		$
		    IF STRNUMBER( NUMST(I), NUM) THEN NUMBER(I) = NUM
		IGOOD = WHERE(NUMBER GE 0, MATCHES)
		IF MATCHES GT 0 THEN BEGIN
		    NFOUND = NFOUND(IGOOD)
		    NUMBER = NUMBER(IGOOD)
		ENDIF
	    ENDIF
;
;  Otherwise, find all the instances of the requested keyword.  If more than
;  one is found, and NAME is not one of the special cases, then print an error
;  message.
;
	ENDIF ELSE BEGIN
	    NFOUND = WHERE(KEYWORD EQ NAM, MATCHES)
	    IF (MATCHES GT 1) AND (NAM NE 'HISTORY ') AND		$
		(NAM NE 'COMMENT ') AND (NAM NE '        ') THEN	$
		MESSAGE,/INFORMATIONAL, 'WARNING- Keyword ' +	$
		NAM + 'located more than once in ' + ABORT
	ENDELSE
;
;  Extract the parameter field from the specified header lines.  If one of the
;  special cases, then done.
;
	IF MATCHES GT 0 THEN BEGIN
	    LINE = HDR(NFOUND)
	    SVALUE = STRTRIM( STRMID(LINE,9,71),2)
	    IF (NAM EQ 'HISTORY ') OR (NAM EQ 'COMMENT ') OR	$
		    (NAM EQ '        ') THEN BEGIN
		VALUE = STRTRIM( STRMID(LINE,8,72),2)
		COMMENTS = STRARR(N_ELEMENTS(VALUE))
;
;  Otherwise, test to see if the parameter contains a string, signalled by
;  beginning with a single quote character (') (apostrophe).
;
	    END ELSE FOR I = 0,MATCHES-1 DO BEGIN
		IF ( STRMID(SVALUE(I),0,1) EQ "'" ) THEN BEGIN
		    TEST = STRMID( SVALUE(I),1,STRLEN( SVALUE(I) )-1)
		    NEXT_CHAR = 0
		    VALUE = ''
;
;  Find the next apostrophe.
;
NEXT_APOST:
		    ENDAP = STRPOS(TEST, "'", NEXT_CHAR)
		    IF ENDAP LT 0 THEN MESSAGE,		$
			'Value of '+NAME+' invalid in '+ABORT
		    VALUE = VALUE + STRMID( TEST, NEXT_CHAR, ENDAP-NEXT_CHAR )
;
;  Test to see if the next character is also an apostrophe.  If so, then the
;  string isn't completed yet.  Apostrophes in the text string are signalled as
;  two apostrophes in a row.
;
		    IF STRMID( TEST, ENDAP+1, 1) EQ "'" THEN BEGIN    
	     		VALUE = VALUE + "'"
	      		NEXT_CHAR = ENDAP+2	 
			GOTO, NEXT_APOST
	  	    ENDIF
;
;  Extract the comment, if any.
;
		    SLASH = STRPOS(TEST, "/", ENDAP)
		    IF SLASH LT 0 THEN COMMENT = '' ELSE	$
			COMMENT = STRMID(TEST, SLASH+1, STRLEN(TEST)-SLASH-1)
;
;  If not a string, then separate the parameter field from the comment field.
;
                ENDIF ELSE BEGIN
		    TEST = SVALUE(I)
		    SLASH = STRPOS(TEST, "/")
		    IF SLASH GT 0 THEN BEGIN
			COMMENT = STRMID(TEST, SLASH+1, STRLEN(TEST)-SLASH-1)
			TEST = STRMID(TEST, 0, SLASH)
		    END ELSE COMMENT = ''
;
;  Find the first word in TEST.  Is it a logical value ('T' or 'F')?
;
		    TEST2 = TEST
		    VALUE = GETTOK(TEST2,' ')
		    TEST2 = STRTRIM(TEST2,2)
		    IF ( VALUE EQ 'T' ) THEN BEGIN
			VALUE = 1
		    END ELSE IF ( VALUE EQ 'F' ) THEN BEGIN
			VALUE = 0
		    END ELSE BEGIN
;
;  Test to see if a complex number.  It's a complex number if the value and the
;  next word, if any, both are valid numbers.
;
			IF STRLEN(TEST2) EQ 0 THEN GOTO, NOT_COMPLEX
			VALUE2 = GETTOK(TEST2,' ')
			IF STRNUMBER(VALUE,VAL1) AND STRNUMBER(VALUE2,VAL2) $
				THEN BEGIN
			    VALUE = COMPLEX(VAL1,VAL2)
			    GOTO, GOT_VALUE
			ENDIF
;
;  Not a complex number.  Decide if it is a floating point, double precision,
;  or integer number.  If an error occurs, then a string value is returned.
;
NOT_COMPLEX:
			ON_IOERROR, GOT_VALUE
			VALUE = TEST
			IF (STRPOS(VALUE,'.') GE 0) OR (STRPOS(VALUE,'E') $
				GE 0) OR (STRPOS(VALUE,'D') GE 0) THEN BEGIN
			    IF ( STRPOS(VALUE,'D') GT 0 ) OR $
				    ( STRLEN(VALUE) GE 8 ) THEN BEGIN
				VALUE = DOUBLE(VALUE)
			    END ELSE VALUE = FLOAT(VALUE)
			END ELSE VALUE = LONG(VALUE)
;
GOT_VALUE:
			ON_IOERROR, NULL
		    ENDELSE
		ENDELSE		; if string
;
;  Add to vector if required.
;
		IF VECTOR THEN BEGIN
		    MAXNUM = MAX(NUMBER)
		    IF ( I EQ 0 ) THEN BEGIN
			SZ_VALUE = SIZE(VALUE)
			RESULT = MAKE_ARRAY( MAXNUM, TYPE=SZ_VALUE(1))
			COMMENTS = STRARR(MAXNUM)
		    ENDIF 
		    RESULT(   NUMBER(I)-1 ) =  VALUE
		    COMMENTS( NUMBER(I)-1 ) =  COMMENT
		ENDIF ELSE BEGIN
		    COMMENTS = COMMENT
		ENDELSE
	    ENDFOR
;
;  Set the value of !ERR for the number of matches for vectors, or simply 0
;  otherwise.
;
	    IF VECTOR THEN BEGIN
		!ERR = MATCHES
		RETURN, RESULT
	    ENDIF ELSE !ERR = 0
;
;  Error point for keyword not found.
;
	ENDIF ELSE BEGIN
	    IF ABORT_RETURN THEN MESSAGE,'Keyword '+NAM+' not found in '+ABORT
	    !ERR = -1
	ENDELSE
;
	RETURN, VALUE
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBTFORM,HEADER,TBCOL,IDLTYPE,FORMAT,NUMVAL,MAXVAL
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBTFORM
; Purpose     : 
;	Returns information about FITS binary table columns.
; Explanation : 
;	Procedure to return information about the format of the various columns
;	in a FITS binary table.
; Use         : 
;	FXBTFORM,HEADER,TBCOL,IDLTYPE,FORMAT,NUMVAL,MAXVAL
; Inputs      : 
;	HEADER	= Fits binary table header.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	TBCOL	= Array of starting column positions in bytes.
;	IDLTYPE	= IDL data types of columns.
;	FORMAT	= Character code defining the data types of the columns.
;	NUMVAL	= Number of elements of the data arrays in the columns.
;	MAXVAL	= Maximum number of elements for columns containing variable
;		  length arrays, or zero otherwise.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	None.
; Calls       : 
;	FXPAR
; Common      : 
;	None.
; Restrictions: 
;	None.
; Side effects: 
;	None.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	W. Thompson, Feb. 1992, from TBINFO by D. Lindler.
;	W. Thompson, Jan. 1993, renamed to be compatible with DOS limitations.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
	ON_ERROR,2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 1 THEN MESSAGE,	$
		'Syntax:  FXBTFORM,HEADER,TBCOL,IDLTYPE,FORMAT,' + $
		'NUMVAL,MAXVAL'
;
;  Get the number of fields.
;
	TFIELDS = FXPAR(HEADER,'TFIELDS')
	IF !ERR LT 0 THEN MESSAGE,	$
		'Invalid FITS header -- keyword TFIELDS is missing'
	IF TFIELDS EQ 0 THEN MESSAGE,	$
		'FIT binary table has no columns'
;
;  Initialize the arrays.
;
	WIDTH	= INTARR(TFIELDS)
	IDLTYPE	= INTARR(TFIELDS)
	TBCOL	= LONARR(TFIELDS)
	FORMAT	= STRARR(TFIELDS)
	NUMVAL	= LONARR(TFIELDS)
	MAXVAL	= LONARR(TFIELDS)
;
;  Get the column formats.
;
	TFORM = FXPAR(HEADER,'TFORM*')
	IF !ERR LT 0 THEN MESSAGE,	$
		'Invalid FITS table header -- keyword TFORM not present'
	TFORM =  STRUPCASE(STRTRIM(TFORM,2))
;
;  Parse the values of the TFORM keywords.
;
	LEN = STRLEN(TFORM)
	FOR I = 0,N_ELEMENTS(TFORM)-1 DO BEGIN
;
;  Step through each character in the format, until a non-numerical character
;  is encountered.
;
		ICHAR = 0
NEXT_CHAR:
		IF ICHAR GE LEN(I) THEN MESSAGE,	$
			'Invalid format specification for keyword TFORM ' + $
			STRTRIM(I+1)
		CHAR = STRUPCASE(STRMID(TFORM(I),ICHAR,1))
		IF ((CHAR GE '0') AND (CHAR LE '9')) THEN BEGIN
			ICHAR = ICHAR + 1
			GOTO, NEXT_CHAR
		ENDIF
;
;  Get the number of elements.
;
		IF ICHAR EQ 0 THEN NUMVAL(I) = 1 ELSE	$
			NUMVAL(I) = LONG(STRMID(TFORM(I),0,ICHAR))
;
;  If the character is "P" then the next character is the actual data type,
;  followed by the maximum number of elements surrounded by quotes.
;
		IF CHAR EQ "P" THEN BEGIN
			CHAR = STRUPCASE(STRMID(TFORM(I),ICHAR+1,1))
			MAXVAL(I) = LONG(STRMID(TFORM(I),ICHAR+3,	$
				LEN(I)-ICHAR-4))
		ENDIF
;
;  Get the IDL data type, and the size of an element.
;
		FORMAT(I) = CHAR
		CASE CHAR OF
			'L':  BEGIN & IDLTYPE(I) = 1 & WIDTH(I) = 1 & END
			'A':  BEGIN & IDLTYPE(I) = 7 & WIDTH(I) = 1 & END
			'B':  BEGIN & IDLTYPE(I) = 1 & WIDTH(I) = 1 & END
			'I':  BEGIN & IDLTYPE(I) = 2 & WIDTH(I) = 2 & END
			'J':  BEGIN & IDLTYPE(I) = 3 & WIDTH(I) = 4 & END
			'E':  BEGIN & IDLTYPE(I) = 4 & WIDTH(I) = 4 & END
			'D':  BEGIN & IDLTYPE(I) = 5 & WIDTH(I) = 8 & END
			'C':  BEGIN & IDLTYPE(I) = 6 & WIDTH(I) = 8 & END
;
;  Treat bit arrays as byte arrays with 1/8 the number of elements.
;
			'X':  BEGIN
				IDLTYPE(I) = 1
				WIDTH(I) = 1
				IF MAXVAL(I) GT 0 THEN BEGIN
					MAXVAL(I) = LONG((MAXVAL(I)+7)/8)
				END ELSE BEGIN
					NUMVAL(I) = LONG((NUMVAL(I)+7)/8)
				ENDELSE
				END

;
;  Treat double-complex arrays as double-precision arrays with twice the number
;  of elements.
;
			'M':  BEGIN
				IDLTYPE(I) = 5
				IF MAXVAL(I) GT 0 THEN MAXVAL(I)=MAXVAL(I)*2 $
						  ELSE NUMVAL(I)=NUMVAL(I)*2
				WIDTH(I) = 8
				END
;
			ELSE:  MESSAGE,'Invalid format specification ' + $
					'for keyword TFORM' + STRTRIM(I+1,2)
		ENDCASE
;
;  Variable length array pointers always take up eight bytes.
;
		IF MAXVAL(I) GT 0 THEN WIDTH(I) = 8
;
;  Calculate the starting byte for each column.
;
		IF I GE 1 THEN TBCOL(I) = TBCOL(I-1) + WIDTH(I-1)*NUMVAL(I-1)
	ENDFOR
;
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
function SXPAR, hdr, name, abort, COUNT=matches   
;+
; NAME:
;	SXPAR
; PURPOSE:
;	Obtain the value of a parameter in a FITS or STSDAS header
;
; CALLING SEQUENCE:
;	result = SXPAR( Hdr, Name, [ Abort, COUNT= ])   
;
; INPUTS:
;	Hdr =  FITS or STSDAS header array, (e.g. as returned by READFITS or 
;		SXOPEN) string array, each element should have a length of 80
;		characters	
;
;	Name = String name of the parameter to return.   If Name is of 
;		the form 'keyword*' then an array is returned containing 
;		values of keywordN where N is an integer.  The value
;		of keywordN will be placed in RESULT(N-1).  The data type 
;		of RESULT will be the type of the first valid match of keywordN 
;		found.
;
; OPTIONAL INPUTS:
;	ABORT - string specifying that SXPAR should do a RETALL
;		if a parameter is not found.  ABORT should contain
;		a string to be printed if the keyword parameter is not found.
;		If not supplied SXPAR will return with a negative
;		!err if a keyword is not found.
;
; OPTIONAL KEYWORD OUTPUT:
;	COUNT - Optional keyword to return a value equal to the number of 
;		parameters found by sxpar, integer scalar
;
; OUTPUTS:
;	Function value = value of parameter in header.
;		If parameter is double precision, floating, long or string,
;		the result is of that type.  Apostrophes are stripped
;		from strings.  If the parameter is logical, 1 is
;		returned for T, and 0 is returned for F.
;		If Name was of form 'keyword*' then a vector of values
;		are returned.
;
; SIDE EFFECTS:
;	!ERR is set to -1 if parameter not found, 0 for a scalar
;	value returned.  If a vector is returned it is set to the
;	number of keyword matches found.
;
;	If a keyword occurs more than once in a header, a warning is given,
;	and the first occurence is used.
;
; EXAMPLES:
;	Given a FITS header, h, return the values of all the NAXISi values
;	into a vector.    Then place the history records into a string vector.
;
;	IDL> naxisi = sxpar( h ,'NAXIS*')         ; Extract NAXISi value
;	IDL> history = sxpar( h, 'HISTORY' )      ; Extract HISTORY records
;
; PROCEDURE:
;	The first 8 chacters of each element of Hdr are searched for a 
;	match to Name.  The value from the last 20 characters is returned.  
;	An error occurs if there is no parameter with the given name.
;       
;	If a numeric value has no decimal point it is returned as type
;	LONG.   If it contains more than 8 numerals, or contains the 
;	character 'D', then it is returned as type DOUBLE.  Otherwise
;	it is returned as type FLOAT
;
; MODIFICATION HISTORY:
;	DMS, May, 1983, Written.
;	D. Lindler Jan 90 added ABORT input parameter
;	J. Isensee Jul,90 added COUNT keyword
;	W. Thompson, Feb. 1992, added support for FITS complex values.
;	W. Thompson, May 1992, corrected problem with HISTORY/COMMENT/blank
;		keywords, and complex value error correction.
;-
;----------------------------------------------------------------------
 if N_params() LT 2 then begin
     print,'Syntax -     result =  sxpar( hdr, name, [abort, COUNT = ])
     return, -1
 endif 

 VALUE = 0
 if N_params() LE 2 then begin
      abort_return = 0
      abort = 'FITS Header'
 end else abort_return = 1
 if abort_return then On_error,1 else On_error,2

;       Check for valid header

  s = size(hdr)		;Check header for proper attributes.
  if ( s(0) NE 1 ) or ( s(2) NE 7 ) then $
	   message,'FITS Header (first parameter) must be a string array'

  nam = strtrim( strupcase(name) )	;Copy name, make upper case     

; Determine if name is of form 'keyword*'

   if strpos( nam, '*' ) EQ strlen( nam ) - 1 then begin    
	    nam = strmid( nam, 0, strlen( nam ) - 1)  
	    vector = 1	   		;Flag for vector output  
	    name_length = strlen(nam)  	;Length of name 
	    num_length = 8 - name_length 	;Max length of number portion  
	    if num_length LE 0 then  $ 
		  message, 'Keyword length must be 8 characters or less'
    endif else begin  
	 	while strlen(nam) LT 8 do nam = nam + ' ' ;Make 8 chars long
		vector = 0      
    endelse
;
; Loop on lines of the header 
;
 keyword = strmid( hdr, 0, 8)
 histnam = (nam eq 'HISTORY ') or (nam eq 'COMMENT ') or (nam eq '        ') 
 if vector then begin
           nfound = where(strpos(keyword,nam) GE 0, matches)
	   if ( matches GT 0 ) then begin
                   numst= strmid(hdr(nfound), name_length, num_length)
                   number = intarr(matches)-1
                   for i = 0, matches-1 do $
	            if strnumber( numst(i), num) then number(i) = num
                   igood = where(number GE 0, matches)
                   if matches GT 0 then begin
                        nfound = nfound(igood) & number = number(igood)
                   endif
           endif

 endif else begin

       nfound = where(keyword EQ nam, matches)
       if not histnam then if matches GT 1 then message,$
         'WARNING- Keyword '+NAM +'located more than once in '+abort,/inform
 endelse   

; Process string parameter 

 if matches GT 0 then begin
  line = hdr(nfound)
  svalue = strtrim( strmid(line,9,70),2)
  if histnam then $
	value = strtrim(strmid(line,8,71),2) else for i = 0,matches-1 do begin
      if ( strmid(svalue(i),0,1) EQ "'" ) then begin   ;Is it a string?
	          test = strmid( svalue(i),1,strlen( svalue(i) )-1)
		  next_char = 0
		  value = '' 
          NEXT_APOST:
		  endap = strpos(test, "'", next_char)      ;Ending apostrophe  
	     	  if endap LT 0 then $ 
			    MESSAGE,'Value of '+name+' invalid in '+abort
	  	  value = value + strmid( test, next_char, endap-next_char )  
;
;  Test to see if the next character is also an apostrophe.  If so, then the
;  string isn't completed yet.  Apostrophes in the text string are signalled as
;  two apostrophes in a row.
;
          	 if strmid( test, endap+1, 1) EQ "'" then begin    
	     	    value = value + "'"
	      	    next_char = endap+2         
		    goto, NEXT_APOST
          	 endif      
;
; Process non-string value  
;
          endif else begin
	       value = strtrim( strmid(line(i), 10, 20), 2)   ;Extract value    
	       if ( value EQ 'T' ) then value = 1 else $
	       if ( value EQ 'F' ) then value = 0 else begin
;
;  Test to see if a complex number.  It's not a complex number of columns 31-50
;  are blank, or if a slash character occurs in columns 11-50.  Otherwise, try
;  to interpret columns 31-50 as a number.
;
	    	value2 = strtrim( strmid(line(i),30,20), 2) ;Imaginary part
                if value2 EQ '' then goto, NOT_COMPLEX
		if strpos( strmid( line(i),10,40), '/') GE 0 then $
                       goto,NOT_COMPLEX
		On_ioerror, NOT_COMPLEX
	   	value2 = float(value2)
		value = complex(value,value2)
		goto, GOT_VALUE
;
;  Not a complex number.  Decide if it is a floating point, double precision,
;  or integer number.
;
NOT_COMPLEX:
		On_ioerror, GOT_VALUE
		  if strpos(value,'.') GE 0 then begin      
		      if ( strpos(value,'D') GT 0 ) or $
			 ( strlen(value) GE 8 ) then value = double(value) $
						else value = float(value)
		       END ELSE value = long(value)
;
GOT_VALUE:
		On_IOerror, NULL
		endelse
	     endelse; if c eq apost
;
;  Add to vector if required
;
	 if vector then begin
               maxnum = max(number)
	       if ( i EQ 0 ) then begin
                     sz_value = size(value)
                     result = make_array( maxnum, type=sz_value(1), /NOZERO)
	       endif 
               result( number(i)-1 ) =  value
	  endif
  endfor

  if vector then begin
         !ERR = matches     
         return, result
  endif else !ERR = 0

endif  else  begin    
     if abort_return then message,'Keyword '+nam+' not found in '+abort
     !ERR = -1
endelse     

return, value       

END                 
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; This version of HEADFITS in the FITS library does not handle 
;  IO errors, and has been modified for the WINGSPAN/MFIT
;  package to trap any IO errors.  The new version is in the file
;  W_UTIL.PRO, in the WINGSPAN/MFIT package.
;
function OLD_HEADFITS, filename, EXTEN = exten
;+
; NAME:
;	HEADFITS
; CALLING SEQUENCE:
;	Result = headfits( filename ,[ EXTEN = ])
;
; PURPOSE:
;	Read a FITS file header record      
;
; INPUTS:
;	FILENAME = String containing the name of the FITS file to be read.
;
; OPTIONAL INPUT KEYWORD:
;	EXTEN  = integer scalar, specifying which FITS extension to read.
;		For example, to read the header of the first extension set
;		EXTEN = 1.   Default is to read the primary FITS header 
;		(EXTEN = 0).
;
; OUTPUTS:
;	Result of function = FITS header, string array
;
; EXAMPLE:
;	Read the FITS header of a file 'test.fits' into a string variable, h
;
;	IDL>  h = headfits( 'test.fits')
;
; MODIFICATION HISTORY:
;	adapted by Frank Varosi from READFITS by Jim Wofford, January, 24 1989
;	Keyword EXTEN added, K.Venkatakrishna, May 1992
;	Make sure first 8 characters are 'SIMPLE'  W. Landsman October 1993
;-
 On_error,2

 if N_params() LT 1 then begin
     print,'Syntax - header = headfits( filename, [ EXTEN = ])
     return, -1
 end
 
; Open file and read header information
       	openr,unit,filename, /GET_LUN, /BLOCK
	file = fstat(unit)
	y = indgen(36*8)
	y2 = y - 8*(y/8) + 80*(y/8)
        offset = 0
        extn = 0
START: 	r = 0
        hdr = assoc(unit, bytarr(80,36), offset)
; Read header one record at a time

	nbytesleft = file.size - offset
	
        if nbytesleft LT 2880 then $
		message,' No such extension, End of file reached'

LOOP:
	x = hdr(r)
	nbytesleft = nbytesleft - 2880
	name = string( x(y2) )		;Get first 8 char of each line
        if (r EQ 0) and (extn EQ 0) then $
 		if strmid(name,0,8) NE 'SIMPLE  ' then begin 
		free_lun, unit              ;Added Mar 94
		message, $
	   'ERROR - FITS header missing required "SIMPLE" in first 8 chars'
		endif
	
        pos = strpos( name, 'END     ' )
        if r EQ 0 then header = string(x) else header = [header,string(x)]
	if (pos lt 0) then begin
		r = r + 1
		goto, LOOP 
	endif 
    	lastline = 36*r + pos / 8
	header = header(0:lastline)
;                                        IF extension, get the size of the
;                                        data. Find no of records to skip 
        If keyword_set(EXTEN) then begin
        bitpix = sxpar( header, 'BITPIX')
        naxis = sxpar( header, 'NAXIS')
        Nax = sxpar( header, 'NAXIS*' )                  ; Read NAXES
        nbytes = nax(0) * abs( bitpix )/ 8
        if naxis GT 1 then for i = 2, naxis do nbytes = nbytes*nax(i-1) $
                      else nbytes = 0
        nrec = nbytes /2880
        if nbytes GT nrec*2880L then nrec = long( nrec + 1 ) else $
        nrec = long(nrec)                
        point_lun, -unit, pointlun
        pointlun = pointlun + nrec*2880L
        point_lun,unit,pointlun
        offset = pointlun
        extn = extn + 1
        if (extn LE EXTEN) then goto, START 
        endif
	free_lun, unit

return, header

end
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXHREAD,UNIT,HEADER,STATUS
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXHREAD
; Purpose     : 
;       Reads a FITS header from an opened disk file.
; Explanation : 
;       Reads a FITS header from an opened disk file.
; Use         : 
;	FXHREAD, UNIT, HEADER  [, STATUS ]
; Inputs      : 
;	UNIT	= Logical unit number.
; Opt. Inputs : 
;
; Outputs     : 
;	HEADER	= String array containing the FITS header.
; Opt. Outputs: 
;	STATUS	= Condition code giving the status of the read.  Normally, this
;		  is zero, but is set to !ERR if an error occurs, or if the
;		  first byte of the header is zero (ASCII null).
; Keywords    : 
;	None.
; Calls       : 
;	None.
; Common      : 
;	None.
; Restrictions: 
;	The file must already be positioned at the start of the header.  It
;	must be a proper FITS file.
; Side effects: 
;	The file ends by being positioned at the end of the FITS header, unless
;	an error occurs.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	W. Thompson, Feb 1992, from READFITS by J. Woffard and W. Landsman.
;	W. Thompson, Aug 1992, added test for SIMPLE keyword.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
;
	ON_ERROR,2			;Return to caller
	STATUS = 0
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 2 THEN MESSAGE,	$
		'Calling sequence:  FXHREAD, UNIT, HEADER  [, STATUS ]'
;
;  Find out whether one is at the beginning of the file (POSITION=0) or not.
;
	POINT_LUN,-UNIT,POSITION
;
;  Read in the first 2880 byte FITS logical block as a series of 36 card images
;  of 80 bytes each.
;
	HDR = BYTARR( 80, 36, /NOZERO )
	ON_IOERROR, RETURN_STATUS
	READU, UNIT, HDR
;
;  If not the primary header, then the first eight bytes should decode to
;  XTENSION.  If not, then set status to -1, and return.
;
	IF POSITION NE 0 THEN BEGIN
		FIRST = STRING(HDR(0:7))
		IF FIRST NE 'XTENSION' THEN BEGIN
			MESSAGE,'XTENSION keyword not found',/CONTINUE
			STATUS = -1
			GOTO, DONE
		ENDIF
	ENDIF
;
;  Interpret the header as a string, and check to see if the END line has been
;  reached.
;
	HEADER = STRING( HDR > 32B )
	ENDLINE = WHERE( STRMID(HEADER,0,8) EQ 'END     ', NEND)
	IF NEND GT 0 THEN HEADER = HEADER( 0:ENDLINE(0) ) 
;
;  If the primary header (POSITION=0) and the SIMPLE keyword can't be found in
;  the first record, then this can't be a FITS file.
;
	IF POSITION EQ 0 THEN BEGIN
		SIMPLE_LINE = WHERE(STRMID(HEADER,0,8) EQ 'SIMPLE  ',N_SIMPLE)
		IF N_SIMPLE EQ 0 THEN BEGIN
			MESSAGE,'SIMPLE keyword not found',/CONTINUE
			STATUS = -1
			GOTO, DONE
		ENDIF
	ENDIF
;
;  Keep reading until the END line is reached.
;
	WHILE NEND EQ 0 DO BEGIN
		READU, UNIT, HDR
		HDR1 = STRING( HDR > 32B )
		ENDLINE = WHERE( STRMID(HDR1,0,8) EQ 'END     ', NEND)
		IF NEND GT 0 THEN HDR1 = HDR1( 0:ENDLINE(0) ) 
		HEADER = [HEADER, HDR1 ]
	ENDWHILE
	GOTO, DONE	
;
;  Error encounter.  Store the error code in status.
;
RETURN_STATUS:
	STATUS = !ERR
;
;  Reset the ON_IOERROR condition.
;
DONE:
	ON_IOERROR,NULL
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBPARSE, ILUN, HEADER, NO_TDIM=NO_TDIM
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBPARSE
; Purpose     : 
;	Parse the binary table extension header.
; Explanation : 
;	Parses the binary table extension header, and store the information
;	about the format of the binary table in the FXBINTABLE common
;	block--called from FXBCREATE and FXBOPEN.
; Use         : 
;	FXBPARSE, ILUN, UNIT, HEADER
; Inputs      : 
;	ILUN	= Index into the arrays in the FXBINTABLE common block.
;	HEADER	= FITS binary table extension header.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	None.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	NO_TDIM	  = If set, then any TDIMn keywords found in the header are
;		    ignored.
; Calls       : 
;	FXBFIND, FXBTDIM, FXBTFORM, FXPAR
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	None.
; Side effects: 
;	Any TDIMn keywords found for bit arrays (format 'X') are ignored, since
;	the dimensions would refer to bits, not bytes.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	William Thompson, Feb. 1992.
;	William Thompson, Jan. 1993, modified for renamed FXBTFORM and FXBTDIM.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
@fxbintbl.pro

	ON_ERROR,2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 2 THEN MESSAGE,	$
		'Syntax:  FXBPARSE, ILUN, HEADER'
;
;  Gather the necessary information, and store it in the common block.
;
	FXBTFORM,HEADER,BYTOFF0,IDLTYPE0,FORMAT0,N_ELEM0,MAXVAL0
	FXBFIND,HEADER,'TTYPE',COLUMNS,TTYPE0,N_FOUND,''
	FXBFIND,HEADER,'TSCAL',COLUMNS,TSCAL0,N_FOUND,1.
	FXBFIND,HEADER,'TZERO',COLUMNS,TZERO0,N_FOUND,0.
	POINT_LUN,-LUN(ILUN),NHEAD0
;
;  Get the information from the required keywords.
;
	STORE_ARRAY,HEAD,HEADER,ILUN
	NHEADER(ILUN) = NHEAD0
	NAXIS1(ILUN)  = FXPAR(HEADER,'NAXIS1')
	NAXIS2(ILUN)  = FXPAR(HEADER,'NAXIS2')
	TFIELDS(ILUN) = FXPAR(HEADER,'TFIELDS')
;
;  If THEAP is not present, then set it equal to the size of the table.
;
	THEAP = FXPAR(HEADER,'THEAP')
	IF !ERR LT 0 THEN THEAP = NAXIS1(ILUN)*NAXIS2(ILUN)
	HEAP(ILUN) = THEAP
;
;  Store the information about the columns.
;
	STORE_ARRAY,BYTOFF,BYTOFF0,ILUN
	STORE_ARRAY,TTYPE,STRUPCASE(STRTRIM(TTYPE0,2)),ILUN
	STORE_ARRAY,IDLTYPE,IDLTYPE0,ILUN
	STORE_ARRAY,FORMAT,FORMAT0,ILUN
	STORE_ARRAY,N_ELEM,N_ELEM0,ILUN
	STORE_ARRAY,TSCAL,TSCAL0,ILUN
	STORE_ARRAY,TZERO,TZERO0,ILUN
	STORE_ARRAY,MAXVAL,MAXVAL0,ILUN
	STORE_ARRAY,N_DIMS,INTARR(9,N_ELEMENTS(N_ELEM0)),ILUN
;
;  If not a variable length array, then get the dimensions associated with each
;  column from the TDIMn keywords.  If not found, then assume to be the number
;  of elements.
;
	FOR ICOL = 0,TFIELDS(ILUN)-1 DO IF MAXVAL(ICOL,ILUN) EQ 0 THEN BEGIN
		TDIM = FXPAR(HEADER,'TDIM'+STRTRIM(ICOL+1,2))
		TDIM_USED = (!ERR GE 0) AND (NOT KEYWORD_SET(NO_TDIM))
		IF TDIM_USED THEN DIMS = FIX(FXBTDIM(TDIM))	$
			     ELSE DIMS = N_ELEM(ICOL,ILUN)
		DIMS = [N_ELEMENTS(DIMS),DIMS]
;
;  If the datatype is a double complex array, then the array is treated as a
;  double precision array with an extra dimension of two.
;
		IF FORMAT(ICOL,ILUN) EQ 'M' THEN IF TDIM_USED THEN	$
			DIMS = [DIMS(0)+1,2,DIMS(1:*)]	  ELSE		$
			DIMS = [2,2,N_ELEM(ICOL,ILUN)/2]
;
;  If the datatype is a bit array, then no dimensions are applied to the data.
;
		IF FORMAT(ICOL,ILUN) EQ 'X' THEN DIMS = [1,N_ELEM(ICOL,ILUN)]
		N_DIMS(0,ICOL,ILUN) = DIMS
;
;  For those columns which are character strings, then the number of
;  characters, N_CHAR, is the first dimension, and the number of elements is
;  actually N_ELEM/N_CHAR.
;
		IF IDLTYPE(ICOL,ILUN) EQ 7 THEN		$
			N_ELEM(ICOL,ILUN) = N_ELEM(ICOL,ILUN) / DIMS(1)
	ENDIF
;
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBOPEN, UNIT, FILENAME, EXTENSION, HEADER, NO_TDIM=NO_TDIM
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBOPEN
; Purpose     : 
;	Open binary table extension in a disk FITS file for reading.
; Explanation : 
;	Opens a binary table extension in a disk FITS file for reading.  The
;	columns are then read using FXBREAD, and the file is closed when done
;	with FXBCLOSE.
; Use         : 
;	FXBOPEN, UNIT, FILENAME, EXTENSION  [, HEADER ]
; Inputs      : 
;	FILENAME  = Name of FITS file to be opened.
;	EXTENSION = Either the number of the FITS extension, starting with the
;		    first extension after the primary data unit being one; or a
;		    character string containing the value of EXTNAME to search
;		    for.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	UNIT	  = Logical unit number of the opened file.
; Opt. Outputs: 
;	HEADER	  = String array containing the FITS binary table extension
;		    header.
; Keywords    : 
;	NO_TDIM	  = If set, then any TDIMn keywords found in the header are
;		    ignored.
; Calls       : 
;	FXBFINDLUN, FXBPARSE, FXHREAD, FXPAR
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	The file must be a valid FITS file.
; Side effects: 
;	None.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	W. Thompson, Feb 1992, based on READFITS by J. Woffard and W. Landsman.
;	W. Thompson, Feb 1992, changed from function to procedure.
;	W. Thompson, June 1992, fixed up error handling.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
@fxbintbl.pro
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN MESSAGE, $
		'Syntax:  FXBOPEN, UNIT, FILENAME, EXTENSION  [, HEADER ]'
;
;  Check the type of the EXTENSION parameter.
;
	IF N_ELEMENTS(EXTENSION) NE 1 THEN MESSAGE,	$
		'EXTENSION must be a scalar'
	SZ = SIZE(EXTENSION)
	ETYPE = SZ(SZ(0)+1)
	IF ETYPE EQ 8 THEN MESSAGE,'EXTENSION must not be a structure'
;
;  If EXTENSION is of type string, then search for the proper extension by
;  name.  Otherwise, search by number.
;
	IF ETYPE EQ 7 THEN BEGIN
		S_EXTENSION = STRTRIM(STRUPCASE(EXTENSION),2)
	END ELSE BEGIN
		I_EXTENSION = FIX(EXTENSION)
		IF I_EXTENSION LT 1 THEN MESSAGE,	$
			'EXTENSION must be greater than zero'
	ENDELSE
;
;  Get a logical unit number, and open the file.
;
	ON_IOERROR, NO_SUCH_FILE
       	OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN
	ON_IOERROR, NULL
;
;  Store the UNIT number in the common block, and leave space for the other
;  parameters.  Initialize the common block if need be.  ILUN is an index into
;  the arrays.
;
	ILUN = FXBFINDLUN(UNIT)
;
;  Mark the file as open for read.
;
	STATE(ILUN) = 1
;
;  Read the primary header.
;
	FXHREAD,UNIT,HEADER,STATUS
	IF STATUS NE 0 THEN BEGIN
		FREE_LUN,UNIT
		MESSAGE,'Unable to read primary FITS header'
	ENDIF
	I_EXT = 0
;
;  Make sure that the file does contain extensions.
;
	IF NOT FXPAR(HEADER,'EXTEND') THEN BEGIN
		FREE_LUN, UNIT
		MESSAGE,'File ' + FILENAME + ' does not contain extensions'
	ENDIF
;
;  Get the number of bytes taken up by the data.
;
NEXT_EXT:
	BITPIX = FXPAR(HEADER,'BITPIX')
	NAXIS  = FXPAR(HEADER,'NAXIS')
	GCOUNT = FXPAR(HEADER,'GCOUNT')  &  IF GCOUNT EQ 0 THEN GCOUNT = 1
	PCOUNT = FXPAR(HEADER,'PCOUNT')
	IF NAXIS GT 0 THEN BEGIN 
		DIMS = FXPAR(HEADER,'NAXIS*')		;Read dimensions
		NDATA = DIMS(0)
		IF NAXIS GT 1 THEN FOR I=2,NAXIS DO NDATA = NDATA*DIMS(I-1)
	ENDIF ELSE NDATA = 0
	NBYTES = (ABS(BITPIX) / 8) * GCOUNT * (PCOUNT + NDATA)
;
;  Read the next extension header in the file.
;
	NREC = LONG((NBYTES + 2879) / 2880)
	POINT_LUN, -UNIT, POINTLUN			;Current position
	MHEAD0 = POINTLUN + NREC*2880L
	POINT_LUN, UNIT, MHEAD0				;Next FITS extension
	FXHREAD,UNIT,HEADER,STATUS
	IF STATUS NE 0 THEN BEGIN
		FREE_LUN,UNIT
		MESSAGE,'Requested extension not found'
	ENDIF
	I_EXT = I_EXT + 1
;
;  Check to see if the current extension is the one desired.
;
	IF ETYPE EQ 7 THEN BEGIN
		EXTNAME = STRTRIM(STRUPCASE(FXPAR(HEADER,'EXTNAME')),2)
		IF EXTNAME EQ S_EXTENSION THEN GOTO, DONE
	END ELSE IF I_EXT EQ I_EXTENSION THEN GOTO, DONE
	GOTO, NEXT_EXT
;
;  Check to see if the extension type is BINTABLE or A3DTABLE.
;
DONE:
	XTENSION = STRTRIM(STRUPCASE(FXPAR(HEADER,'XTENSION')),2)
	IF (XTENSION NE 'BINTABLE') AND (XTENSION NE 'A3DTABLE') THEN BEGIN
		IF ETYPE EQ 7 THEN EXT = S_EXTENSION ELSE EXT = I_EXTENSION
		FREE_LUN,UNIT
		MESSAGE,'Extension ' + STRTRIM(EXT,2) +		$
			' is not a binary table'
	ENDIF
;
;  Get the rest of the information, and store it in the common block.
;
	MHEADER(ILUN) = MHEAD0
	FXBPARSE,ILUN,HEADER,NO_TDIM=NO_TDIM
	RETURN
;
;  Error point for not being able to open the file
;
NO_SUCH_FILE:
	MESSAGE,'Unable to open file ' + STRTRIM(FILENAME,2)
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBFIND,P1,KEYWORD,COLUMNS,VALUES,N_FOUND,DEFAULT
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBFIND
; Purpose     : 
;	Find column keywords in a FITS binary table header.
; Explanation : 
;	Finds the value of a column keyword for all the columns in the binary
;	table for which it is set.  For example,
;
;		FXBFIND, UNIT, 'TTYPE', COLUMNS, VALUES, N_FOUND
;
;	Would find all instances of the keywords TTYPE1, TTYPE2, etc.  The
;	array COLUMNS would contain the column numbers for which a TTYPEn
;	keyword was found, and VALUES would contain the values.  N_FOUND would
;	contain the total number of instances found.
;
; Use         : 
;	FXBFIND, [UNIT or HEADER], KEYWORD, COLUMNS, VALUES, N_FOUND
;		[, DEFAULT ]
; Inputs      : 
;	Either UNIT or HEADER must be passed.
;
;	UNIT	= Logical unit number of file opened by FXBOPEN.
;	HEADER	= FITS binary table header.
;	KEYWORD	= Prefix to a series of FITS binary table column keywords.  The
;		  keywords to be searched for are formed by combining this
;		  prefix with the numbers 1 through the value of TFIELDS in the
;		  header.
; Opt. Inputs : 
;	DEFAULT	= Default value to use for any column keywords that aren't
;		  found.  If passed, then COLUMNS and VALUES will contain
;		  entries for every column.  Otherwise, COLUMNS and VALUES only
;		  contain entries for columns where values were found.
; Outputs     : 
;	COLUMNS	= Array containing the column numbers for which values of the
;		  requested keyword series were found.
;	VALUES	= Array containing the found values.
;	N_FOUND	= Number of values found.  The value of this parameter is
;		  unaffected by whether or not DEFAULT is passed.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	None.
; Calls       : 
;	FXBFINDLUN, FXPAR
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	If UNIT is passed, then the file must have been opened with FXBOPEN.
;	If HEADER is passed, then it must be a legal FITS binary table header.
;
;	The type of DEFAULT must be consistent with the values of the requested
;	keywords, i.e. both most be either of string or numerical type.
;
;	The KEYWORD prefix must not have more than five characters to leave
;	room for the three digits allowed for the column numbers.
;
; Side effects: 
;	None.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	William Thompson, Feb. 1992.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
@fxbintbl.pro
	ON_ERROR,2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 5 THEN MESSAGE,	$
		'Syntax:  FXBFIND,[UNIT/HEADER],KEYWORD,COLUMNS,VALUES,' + $
		'N_FOUND [,DEFAULT]'
;
;  Get the header.
;
	IF N_ELEMENTS(P1) EQ 1 THEN BEGIN
		ILUN = FXBFINDLUN(P1)
		HEADER = HEAD(*,ILUN)
	END ELSE HEADER = P1
;
;  Get the value of TFIELDS from HEADER.
;
	TFIELDS0 = FXPAR(HEADER,'TFIELDS')
	IF TFIELDS0 EQ 0 THEN MESSAGE,'No columns found in HEADER'
;
;  Step through the columns one by one.
;
	N_COLUMNS = 0
	N_FOUND = 0
	FOR I = 1,TFIELDS0 DO BEGIN
		VALUE = FXPAR(HEADER,STRTRIM(KEYWORD,2)+STRTRIM(I,2))
		IF !ERR GE 0 THEN BEGIN
			N_FOUND = N_FOUND + 1
			ADD_COL = 1
		END ELSE IF N_PARAMS() EQ 6 THEN BEGIN
			VALUE = DEFAULT
			ADD_COL = 1
		END ELSE BEGIN
			ADD_COL = 0
		ENDELSE
;
;  Append the found (or default) data to the arrays.
;
		IF ADD_COL THEN BEGIN
			IF N_COLUMNS EQ 0 THEN BEGIN
				COLUMNS = I
				VALUES = VALUE
			END ELSE BEGIN
				COLUMNS = [COLUMNS,I]
				VALUES = [VALUES,VALUE]
			ENDELSE
			N_COLUMNS = N_COLUMNS + 1
		ENDIF
	ENDFOR
;
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO STORE_ARRAY, DESTINATION, INSERT, INDEX
;+
; NAME:
;	STORE_ARRAY
; PURPOSE:
;	Insert array INSERT into array DESTINATION, allowing the dimensions of
;	DESTINATION to adjust to accomodate it.
; CATEGOBY:
;	Utility
; CALLING SEQUENCE:
;	STORE_ARRAY, DESTINATION, INSERT, INDEX
; INPUT:
;	DESTINATION	= Array to be expanded.
;	INSERT		= Array to insert into DESTINATION.
;	INDEX		= Index of the final dimension of DESTINATION to insert
;			  INSERT into.
; OUTPUTS:
;	DESTINATION	= Expanded output array.  If both input arrays have the
;			  same number of dimensions, then the DESTINATION will
;			  be replaced with INSERT.
; RESTRICTIONS:
;	DESTINATION and INSERT have to be either both of type string or both of
;	numerical types.
;
;	INSERT must not have more dimensions than DESTINATION.
;
; MODIFICATION HISTOBY:
;	William Thompson, Feb. 1992, from BOOST_ARRAY by D. Zarro and P. Hick.
;-
;
	ON_ERROR, 2			;On error, return to caller
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 3 THEN MESSAGE,	$
		'Syntax:  STORE_ARRAY, DESTINATION, INSERT, INDEX'
;
;  Make sure everything is defined.
;
	IF N_ELEMENTS(INSERT) EQ 0 THEN MESSAGE,'INSERT not defined'
	IF N_ELEMENTS(INDEX) EQ 0 THEN MESSAGE,'INDEX not defined'
;
;  If DESTINATION is not defined, then set it equal to INSERT.
;
	IF N_ELEMENTS(DESTINATION) EQ 0 THEN BEGIN
		DESTINATION = INSERT
		RETURN
	ENDIF
;
;  Get the array types and dimensions of DESTINATION and INSERT.
;
	SD = SIZE(DESTINATION)
	SA = SIZE(INSERT)
	D_NDIM = SD(0)
	A_NDIM = SA(0)
	IF D_NDIM EQ 0 THEN D_DIM = 1 ELSE D_DIM = SD(1:D_NDIM)
	IF A_NDIM EQ 0 THEN A_DIM = 1 ELSE A_DIM = SA(1:A_NDIM)
	D_TYPE = SD(N_ELEMENTS(SD)-2)
	A_TYPE = SA(N_ELEMENTS(SA)-2)
;
;  Treat scalars as one-dimensional arrays.
;
	D_NDIM = D_NDIM > 1
	A_NDIM = A_NDIM > 1
; 
;  Check to see if both arrays are of type string or numeric.
;
	IF D_TYPE EQ 7 THEN D_STRING = 1  ELSE D_STRING = 0
	IF A_TYPE EQ 7 THEN A_STRING = 1  ELSE A_STRING = 0
	IF D_STRING NE A_STRING THEN MESSAGE,	$
		'Data arrays should be either both string or both non-string'
;
;  If both arrays have the same number of elements, then replace DESTINATION
;  with INSERT.
;
	IF D_NDIM EQ A_NDIM THEN BEGIN
		DESTINATION = INSERT
		RETURN
;
;  Otherwise, make sure that INSERT has fewer dimensions than DESTINATION.
;
	END ELSE IF D_NDIM LT A_NDIM THEN MESSAGE,	$
		'INSERT has more dimensions than DESTINATION'
;
;  Check INDEX
;
	LAST = D_DIM(D_NDIM-1) - 1
	IF (INDEX LT 0) OR (INDEX GT LAST) THEN MESSAGE,	$
		'INDEX must be between 0 and ' + STRTRIM(LAST,2)
;
;  Merge the dimensions of DESTINATION and INSERT.
;
	R_DIM = D_DIM
	FOR I = 0,A_NDIM-1 DO R_DIM(I) = D_DIM(I) > A_DIM(I)
;
;  Create the output array with the correct number of elements, and the greater
;  of the types of DESTINATION and INSERT.
;
	OUTPUT = MAKE_ARRAY(DIMENSION=R_DIM, TYPE=(D_TYPE > A_TYPE))
	R_NDIM = N_ELEMENTS(R_DIM)
;
;  If INDEX is not zero, then store the first part of DESTINATION in the output
;  array.
;
	IF INDEX NE 0 THEN BEGIN
	    K = INDEX - 1
	    CASE R_NDIM OF
	    	2:  OUTPUT(0,0) = DESTINATION(*,0:K)
	    	3:  OUTPUT(0,0,0) = DESTINATION(*,*,0:K)
	    	4:  OUTPUT(0,0,0,0) = DESTINATION(*,*,*,0:K)
	    	5:  OUTPUT(0,0,0,0,0) = DESTINATION(*,*,*,*,0:K)
	    	6:  OUTPUT(0,0,0,0,0,0) = DESTINATION(*,*,*,*,*,0:K)
	    	7:  OUTPUT(0,0,0,0,0,0,0) = DESTINATION(*,*,*,*,*,*,0:K)
	    ENDCASE
	ENDIF
;
;  Add INSERT.
;
	CASE R_NDIM OF
		2:  OUTPUT(0,INDEX) = INSERT
		3:  OUTPUT(0,0,INDEX) = INSERT
		4:  OUTPUT(0,0,0,INDEX) = INSERT
		5:  OUTPUT(0,0,0,0,INDEX) = INSERT
		6:  OUTPUT(0,0,0,0,0,INDEX) = INSERT
		7:  OUTPUT(0,0,0,0,0,0,INDEX) = INSERT
	ENDCASE
;
;  Store the remainder of DESTINATION, if any, in the output array.
;
	IF INDEX NE LAST THEN BEGIN
	    K = INDEX + 1
	    CASE R_NDIM OF
	    	2:  OUTPUT(0,K) = DESTINATION(*,K:*)
	    	3:  OUTPUT(0,0,K) = DESTINATION(*,*,K:*)
	    	4:  OUTPUT(0,0,0,K) = DESTINATION(*,*,*,K:*)
	    	5:  OUTPUT(0,0,0,0,K) = DESTINATION(*,*,*,*,K:*)
	    	6:  OUTPUT(0,0,0,0,0,K) = DESTINATION(*,*,*,*,*,K:*)
	    	7:  OUTPUT(0,0,0,0,0,0,K) = DESTINATION(*,*,*,*,*,*,K:*)
	    ENDCASE
	ENDIF
;
;  Replace DESTINATION with OUTPUT, and return.
;
	DESTINATION = OUTPUT
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBREAD, UNIT, DATA, COL, ROW, NOSCALE=NOSCALE, VIRTUAL=VIR, $
		DIMENSIONS=DIMENSIONS, NANVALUE=NANVALUE
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBREAD
; Purpose     : 
;	Read a data array from a disk FITS binary table file.
; Explanation : 
;	Each call to FXBREAD will read the data from one column and one row
;	from the FITS data file, which should already have been opened by
;	FXBOPEN.  One needs to call this routine for every column and every row
;	in the binary table.  FXBCLOSE will then close the FITS data file.
; Use         : 
;	FXBREAD, UNIT, DATA, COL  [, ROW ]
; Inputs      : 
;	UNIT	= Logical unit number corresponding to the file containing the
;		  binary table.
;	COL	= Column in the binary table to read data from, either as a
;		  character string containing a column label (TTYPE), or as a
;		  numerical column index starting from column one.
; Opt. Inputs : 
;	ROW	= Either row number in the binary table to read data from,
;		  starting from row one, or a two element array containing a
;		  range of row numbers to read.  If not passed, then the entire
;		  column is read in.
;
;		  Row must be passed for variable length arrays.
;
; Outputs     : 
;	DATA	= IDL data array to be read from the file.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	NOSCALE	= If set, then the ouput data will not be scaled using the
;		  optional TSCAL and TZERO keywords in the FITS header.
;		  Default is to scale.
;	VIRTUAL	= If set, and COL is passed as a name rather than a number,
;		  then if the program can't find a column with that name, it
;		  will then look for a keyword with that name in the header.
;		  Such a keyword would then act as a "virtual column", with the
;		  same value for every row.
;	DIMENSIONS = Vector array containing the dimensions to be used to read
;		  in the data.  Bypasses any dimensioning information stored in
;		  the header.  Ignored for bit arrays.  If the data type is
;		  double-precision complex, then an extra dimension of 2 is
;		  prepended to the dimensions passed by the user.
;	NANVALUE= Value signalling data dropout.  All points corresponding to
;		  IEEE NaN (not-a-number) are converted to this number.
;		  Ignored unless DATA is of type float, double-precision or
;		  complex.
; Calls       : 
;	IEEE_TO_HOST, FXPAR, WHERENAN
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	The binary table file must have been opened with FXBOPEN.
;
;	The data must be consistent with the column definition in the binary
;	table header.
;
;	The row number must be consistent with the number of rows stored in the
;	binary table header.
;
;	The number of elements implied by the dimensions keyword must not
;	exceed the number of elements stored in the file.
;
; Side effects: 
;	If the DIMENSIONS keyword is used, then the number of data points read
;	in may be less than the number of points stored in the table.
;
;	If there are no elements to read in (the number of elements is zero),
;	then the program sets !ERR to -1, and DATA is unmodified.
;
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	W. Thompson, Jan 1992.
;	W. Thompson, Feb 1992, modified to support variable length arrays.
;	W. Thompson, Jun 1992, modified way that row ranges are read in.  No
;			       longer works reiteratively.
;	W. Thompson, Jun 1992, fixed bug where NANVALUE would be modified by
;			       TSCAL and TZERO keywords.
;	W. Thompson, Jun 1992, fixed bug when reading character strings.
;			       Treats dimensions better when reading multiple
;			       rows.
; Written     : 
;	William Thompson, GSFC, January 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
;	Version 2, William Thompson, GSFC, 30 June 1993.
;		Added overwrite keyword to REFORM call to speed up.
;	Version 3, William Thompson, GSFC, 21 July 1993.
;		Fixed bug with variable length arrays.
;	Version 4, William Thompson, GSFC, 29 October 1993.
;		Added error message for not finding column by name.
; Version     : 
;	Version 4, 29 October 1993.
;-
;
@fxbintbl.pro
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 3 THEN MESSAGE, $
		'Syntax:  FXBREAD, UNIT, DATA, COL  [, ROW ]'
;
;  Find the logical unit number in the FXBINTABLE common block.
;
	ILUN = WHERE(LUN EQ UNIT,NLUN)
	ILUN = ILUN(0)
	IF NLUN EQ 0 THEN MESSAGE,'Unit ' + STRTRIM(UNIT,2) +	$
		' not opened properly'
;
;  If COL is of type string, then search for a column with that label.
;
	SC = SIZE(COL)
	VIRTUAL = 0
	IF SC(SC(0)+1) EQ 7 THEN BEGIN
		SCOL = STRUPCASE(STRTRIM(COL,2))
		ICOL = WHERE(TTYPE(*,ILUN) EQ SCOL, NCOL)
		ICOL = ICOL(0)
		IF ICOL LT 0 THEN MESSAGE,'Column "' + SCOL + '" not found'
;
;  If the column was not found, and VIRTUAL was set, then search for a keyword
;  by that name.
;
		IF NCOL EQ 0 THEN BEGIN
			IF KEYWORD_SET(VIR) THEN BEGIN
				HEADER = HEAD(*,ILUN)
				VALUE = FXPAR(HEADER,SCOL)
				IF !ERR GE 0 THEN BEGIN
					DATA = VALUE
					VIRTUAL = 1
					GOTO, CHECK_ROW
				ENDIF
			ENDIF
			MESSAGE,'Column "' + SCOL + '" not found'
		ENDIF
;
;  Otherwise, a numerical column was passed.  Check its value.
;
	END ELSE ICOL = LONG(COL) - 1
	IF (ICOL LT 0) OR (ICOL GE TFIELDS(ILUN)) THEN MESSAGE,		$
		'COL must be between 1 and ' + STRTRIM(TFIELDS(ILUN),2)
;
;  If there are no elements in the array, then set !ERR to -1.
;
	IF N_ELEM(ICOL,ILUN) EQ 0 THEN BEGIN
		MESSAGE,'Number of elements to read in is zero',/INFORMATIONAL
		!ERR = -1
		RETURN
	ENDIF
;
;  If ROW was not passed, then set it equal to the entire range.  Otherwise,
;  extract the range.
;
CHECK_ROW:
	IF N_PARAMS() EQ 3 THEN ROW = [1,NAXIS2(ILUN)]
	CASE N_ELEMENTS(ROW) OF
		1:  ROW2 = LONG(ROW(0))
		2:  ROW2 = LONG(ROW(1))
		ELSE:  MESSAGE, 'ROW must have one or two elements'
	ENDCASE
	ROW1 = LONG(ROW(0))
;
;  If ROW represents a range, then make sure that the row range is legal, and
;  that reading row ranges is allowed (i.e., the column is not variable length.
;
	IF ROW1 NE ROW2 THEN BEGIN
		MAXROW = NAXIS2(ILUN)
		IF (ROW1 LT 1) OR (ROW1 GT MAXROW) THEN MESSAGE, $
			'ROW(0) must be between 1 and ' + STRTRIM(MAXROW,2)
		IF (ROW2 LT ROW1) OR (ROW2 GT MAXROW) THEN MESSAGE, $
			'ROW(1) must be between ' + STRTRIM(ROW1,2) + $
			' and ' + STRTRIM(MAXROW,2)
		IF NOT VIRTUAL THEN IF MAXVAL(ICOL,ILUN) GT 0 THEN MESSAGE, $
			'Row ranges not allowed for variable-length columns'
;
;  Otherwise, if ROW is a single number, then just make sure it's valid.
;
	END ELSE BEGIN
		IF (ROW1 LT 1) OR (ROW1 GT NAXIS2(ILUN)) THEN MESSAGE,	$
			'ROW must be between 1 and ' + STRTRIM(NAXIS2(ILUN),2)
	ENDELSE
;
;  If a virtual column, then simply return the value.  If necessary, then
;  replicate the value the correct number of times.
;
	IF VIRTUAL THEN BEGIN
		IF ROW1 EQ ROW2 THEN DATA = VALUE ELSE	$
			DATA = REPLICATE(VALUE,ROW2-ROW1+1)
		RETURN
	ENDIF
;
;  Find the position of the first byte of the data array in the file.
;
	OFFSET = NHEADER(ILUN) + NAXIS1(ILUN)*(ROW1-1) + BYTOFF(ICOL,ILUN)
	POINT_LUN,UNIT,OFFSET
;
;  If a variable length array, then read in the number of elements, and the
;  pointer to the variable length array.  Change the pointing.
;
	IF MAXVAL(ICOL,ILUN) GT 0 THEN BEGIN
		POINTER = LONARR(2)
		READU,UNIT,POINTER
		IEEE_TO_HOST,POINTER
		DIMS = POINTER(0)
		POINT_LUN,UNIT,NHEADER(ILUN) + HEAP(ILUN) + POINTER(1)
;
;  If there are no elements in the array, then set !ERR to -1.
;
		IF DIMS EQ 0 THEN BEGIN
			MESSAGE,'Number of elements to read in is zero', $
				/INFORMATIONAL
			!ERR = -1
			RETURN
		ENDIF
;
;  If the datatype is a double complex array, then the array is treated as a
;  double precision array with an extra dimension of two.
;
		IF FORMAT(ICOL,ILUN) EQ 'M' THEN DIMS = [2,DIMS]
;
;  If the datatype is a bit array, then the array is treated as a byte array
;  with 1/8 the number of elements.
;
		IF FORMAT(ICOL,ILUN) EQ 'X' THEN DIMS = LONG((DIMS+7)/8)
;
;  If fixed length, then get the dimensions of the output array.
;
	END ELSE BEGIN
		DIMS = N_DIMS(*,ICOL,ILUN)
		DIMS = DIMS(1:DIMS(0))
	ENDELSE
;
;  If the DIMENSIONS keyword has been passed, then use that instead of the
;  dimensions already determined.
;
	IF (N_ELEMENTS(DIMENSIONS) GT 0) AND (FORMAT(ICOL,ILUN) NE 'X')	$
			THEN BEGIN
		IF FORMAT(ICOL,ILUN) EQ 'M' THEN	$
			DIMS0 = [2,DIMENSIONS] ELSE DIMS0 = DIMENSIONS
		IF PRODUCT(DIMS0) GT PRODUCT(DIMS) THEN MESSAGE,	$
			'Requested dimensions exceeds the number of elements'
		DIMS = DIMS0
	ENDIF
;
;  Read in the data.  If a character string array, then read in a byte array.
;
	DATATYPE = IDLTYPE(ICOL,ILUN)
	IF DATATYPE EQ 7 THEN DATATYPE = 1
;
;  If only reading in a single row, then the pointer should already be set.
;  Otherwise, the pointer needs to be set for each row.
;
	IF ROW1 EQ ROW2 THEN BEGIN
		DATA = MAKE_ARRAY(TYPE=DATATYPE,DIMENSION=DIMS)
		READU,UNIT,DATA
	END ELSE BEGIN
		DATA = MAKE_ARRAY(TYPE=DATATYPE,DIMENSION=[DIMS,ROW2-ROW1+1])
		TEMPDATA = MAKE_ARRAY(TYPE=DATATYPE,DIMENSION=DIMS)
		NTEMP = N_ELEMENTS(TEMPDATA)
		FOR IROW = ROW1,ROW2 DO BEGIN
			OFFSET = NHEADER(ILUN) + BYTOFF(ICOL,ILUN)
			POINT_LUN,UNIT,OFFSET + NAXIS1(ILUN)*(IROW-1)
			READU,UNIT,TEMPDATA
			DATA((IROW-ROW1)*NTEMP) = TEMPDATA(*)
		ENDFOR
	ENDELSE
;
;  If a character string array, then convert to type string.
;
	IF IDLTYPE(ICOL,ILUN) EQ 7 THEN BEGIN
		DATA = STRING(DATA)
		COUNT = 0
;
;  Otherwise, if necessary, then convert the data to the native format of the
;  host machine.  Also, if NANVALUE is passed, then keep track of any IEEE NaN
;  values.
;
	END ELSE IF IDLTYPE(ICOL,ILUN) NE 1 THEN BEGIN
		IF (N_ELEMENTS(NANVALUE) EQ 1) AND (IDLTYPE(ICOL,ILUN) GE 4) $
			AND (IDLTYPE(ICOL,ILUN) LE 6) THEN	$
			W = WHERENAN(DATA,COUNT) ELSE COUNT = 0
		IEEE_TO_HOST,DATA
	END ELSE COUNT = 0
;
;  If the number of elements of DATA is one, then convert it to a scalar.
;  Otherwise, if DIMS is simply the number 1, but multiple rows were read in, then
;  convert DATA to a simple vector.
;
	IF N_ELEMENTS(DATA) EQ 1 THEN BEGIN
		DATA = DATA(0)
	END ELSE IF (N_ELEMENTS(DIMS) EQ 1) AND (DIMS(0) EQ 1) THEN BEGIN
		DATA = REFORM(DATA,ROW2-ROW1+1,/OVERWRITE)
	ENDIF
;
;  If the parameters TZERO and TSCAL are non-trivial, then adjust the array by
;  these values.
;
	IF NOT KEYWORD_SET(NOSCALE) THEN BEGIN
		BZERO  = TZERO(ICOL,ILUN)
		BSCALE = TSCAL(ICOL,ILUN)
		IF (BSCALE NE 0) AND (BSCALE NE 1) THEN DATA = BSCALE*DATA
		IF BZERO NE 0 THEN DATA = DATA + BZERO
	ENDIF
;
;  Store NANVALUE everywhere where the data corresponded to IEE NaN.
;
	IF COUNT GT 0 THEN DATA(W) = NANVALUE
;
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
pro ieee_to_host, data, IDLTYPE = idltype
;+
; NAME:
;	IEEE_TO_HOST
; PURPOSE:
;	To translate an IDL variable in IEEE-754 representation (as used, for
;	example, in FITS data ), into the host machine architecture.
;
; CALLING SEQUENCE:
;	IEEE_TO_HOST, data, [ IDLTYPE = , ]
;
; INPUT-OUTPUT PARAMETERS:
;	data - any IDL variable, scalar or vector.   It will be modified by
;		IEEE_TO_HOST to convert from IEEE to host representation.  Byte 
;		and string variables are returned by IEEE_TO_HOST unchanged
;
; OPTIONAL KEYWORD INPUTS:
;	IDLTYPE - scalar integer (1-7) specifying the IDL datatype according
;		to the code given by the SIZE function.     This keyword
;		is usually when DATA is a byte array to be interpreted as
;		another datatype (e.g. FLOAT).
;
; EXAMPLE:
;	A 2880 byte array (named FITARR) from a FITS record is to be 
;	interpreted as floating and converted to the host representaton:
;
;	IDL> IEEE_TO_HOST, bytarr, IDLTYPE = 4     
;
; METHOD:
;	The BYTEORDER procedure is called with the appropriate keyword
;
; RESTRICTION:
;	Will run *much* faster for floating or double precision if the IDL 
;	version is since 2.2.2 when the /XDRTOF keyword  became available to 
;	BYTEORDER.   However, IEEE_TO_HOST should still work in earlier 
;	versions of IDL.   Note that V3.0.0 has a bug in the /XDRTOD keyword 
;	on DecStations so IEEE_TO_HOST has a workaround.
;
; MODIFICATION HISTORY:
;	Written, W. Landsman   Hughes/STX   May, 1992
;	Fixed error Case statement for float and double   September 1992
;	Workaround to /XDRTOD problem on DecStations January 1993 
;-
 On_error,2 

 if N_params() EQ 0 then begin
    print,'Syntax - IEEE_TO_HOST, data, [ IDLTYPE = ]
    return
 endif  

 npts = N_elements( data )
 if npts EQ 0 then $
     message,'ERROR - IDL data variable (first parameter) not defined'

 sz = size(data)
 if not keyword_set( idltype) then idltype = sz( sz(0)+1)

 case idltype of

      1: return                             ;byte

      2: byteorder, data, /NTOHS            ;integer

      3: byteorder, data, /NTOHL            ;long

      4: begin                              ;float
         if since_version('2.2.2') then byteorder, data, /XDRTOF $         
         else begin
            case STRUPCASE(!VERSION.OS) of
            'VMS': conv_unix_vax, data
            'MIPSEL': byteorder, data, /LSWAP  
            '386I': byteorder, data, /LSWAP 
             '386': byteorder, data, /LSWAP 
            else:
            endcase
         endelse
         end

      5: BEGIN                              ;double
       
            if !VERSION.ARCH NE 'mipsel' then begin

                if since_version('2.2.2') then $ 
                          byteorder, data, /XDRTOD  else $
                          conv_unix_vax, data

            endif else begin
                
                    if !VERSION.RELEASE NE '3.0.0' then begin
 
                          byteorder, data, /XDRTOD

                    endif else begin

                    dtype = sz( sz(0) + 1)
                    if ( dtype EQ 5 ) then data = byte(data, 0, npts*8) $
                                      else npts = npts/8
                    data = reform( data, 8 , npts ,/OVER)
                    data = rotate( data, 5)
                    if ( dtype EQ 5 ) then data = double(data, 0, npts)
                    data = reform( data, sz(1:sz(0)), /OVER )

                 endelse

             endelse
         end
     
      6: BEGIN                              ;complex
           fdata = float(data)
           byteorder, fdata, /XDRTOF
           idata = imaginary( temporary(data) )
           byteorder, idata, /XDRTOF
           data = complex( fdata, idata )
         END

      7: return                             ;string

       8: BEGIN				    ;structure

	Ntag = N_tags( data )

	for t=0,Ntag-1 do  begin
          temp = data.(t)
          ieee_to_host, temp
          data.(t) = temp
        endfor 

       END
 ENDCASE

 return
 end 
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	PRO FXBCLOSE, UNIT
;+
; Project     : SOHO - CDS
;
; Name        : 
;	FXBCLOSE
; Purpose     : 
;	Close a FITS binary table extension opened for read.
; Explanation : 
;	Closes a FITS binary table extension that had been opened for read by
;	FXBOPEN.
; Use         : 
;	FXBCLOSE, UNIT
; Inputs      : 
;	UNIT	= Logical unit number of the file.
; Opt. Inputs : 
;	None.
; Outputs     : 
;	None.
; Opt. Outputs: 
;	None.
; Keywords    : 
;	None.
; Calls       : 
;	None.
; Common      : 
;	Uses common block FXBINTABLE--see "fxbintable.pro" for more
;	information.
; Restrictions: 
;	The file must have been opened with FXBOPEN.
; Side effects: 
;	None.
; Category    : 
;	Data Handling, I/O, FITS, Generic.
; Prev. Hist. : 
;	W. Thompson, Feb. 1992.
; Written     : 
;	William Thompson, GSFC, February 1992.
; Modified    : 
;	Version 1, William Thompson, GSFC, 12 April 1993.
;		Incorporated into CDS library.
; Version     : 
;	Version 1, 12 April 1993.
;-
;
@fxbintbl.pro
	ON_ERROR, 2
;
;  Check the number of parameters.
;
	IF N_PARAMS() NE 1 THEN MESSAGE, 'Syntax:  FXBCLOSE, UNIT'
;
;  Find the index of the file.
;
	ILUN = WHERE(LUN EQ UNIT,NLUN)
	ILUN = ILUN(0)
	IF NLUN EQ 0 THEN MESSAGE,'Unit ' + STRTRIM(UNIT,2) +	$
		' not opened properly'
;
;  Make sure the file was opened for read access.
;
	IF STATE(ILUN) NE 1 THEN MESSAGE,'Unit ' + STRTRIM(UNIT,2) +	$
		' not opened for read access'
;
;  Close the file, and mark it as closed.
;
	FREE_LUN,UNIT
	STATE(ILUN) = 0
;
	RETURN
	END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
