; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
pro synth_bfits, in_file, out_file, num_out, out_times, out_rates, out_errs
;
; Procedure to write a synthetic BFITS file 'out_file' with num_out records,
;based upon 'in_file'. Some minimal error checking is done, but the caller
;should check to see that the 'in_file' is actually present, and that the
;number of energy channels in the out_rates and out_errs arrays match that of
;the template file.
;INPUTS:
;	STRING   in_file: Input BFITS filename; file is used as the template.
;	STRING  out_file: Output BFITS filename; file is based upon the 
;                         in_file template with num_out rows of data.
;	INTEGER  num_out: Number of output rows, to be inserted in NAXIS2 of
;                         the output data table extension.
;	REAL   out_times: 2 X num_out array of times to be output in out_file.
;	REAL   out_rates: nchan X num_out array of rates to be output.
;	REAL    out_errs: nchan X num_out array of errors in rates to be output.
;
header = headfits(in_file)
filetype = sxpar(header, 'FILETYPE')
IF filetype NE 'BATSE BURST SPECTRA' THEN BEGIN
   PRINT, '*** Wrong type of FITS file: ', filetype, ' ***'
   RETURN
   ENDIF
file_temp = STRUPCASE(out_file)
brack_pos = STRPOS(file_temp, ']')
IF brack_pos NE 0 THEN BEGIN
   file_temp = STRMID(file_temp, brack_pos+1, 999)
   ENDIF
dot_pos = STRPOS(file_temp, '.')
IF dot_pos NE 0 THEN BEGIN
   file_temp = STRMID(file_temp, 0, dot_pos)
   ENDIF
fxaddpar, header, 'OBJECT', file_temp
old_trig = sxpar(header, 'BATSE_TR')
fxaddpar, header, 'BATSE_TR', 100, 'Synthetic burst only: NOT a trigger'
fxaddpar, header, 'FILE-ID', out_file, 'Name of FITS file'
get_date, today
fxaddpar, header, 'DATE', today, 'FITS file creation date (dd/mm/yy)'
fxaddpar, header, 'MNEMONIC', 'SYNTH_BFITS 1.0', 'Program creating this file'
fxaddpar, header, 'COMMENT', 'This file consists of synthetic time-sequenced spectral data'
fxwrite, out_file, header

; Calibration table remains unmodified...
fxbopen, unit_in, in_file, 1, ch
n_dets = sxpar(ch, 'NAXIS2')
n_cols = sxpar(ch, 'TFIELDS')
fxbcreate, unit_out, out_file, ch
FOR outer = 1, n_cols DO BEGIN
   FOR inner = 1, n_dets DO BEGIN
      fxbread, unit_in, dummy, outer, inner
      fxbwrite, unit_out, dummy, outer, inner
      ENDFOR
   ENDFOR
fxbfinish, unit_out
fxbclose, unit_in

fxbopen, unit_in, in_file, 2, dh
n_bins = sxpar(dh, 'NAXIS2')
n_cols = sxpar(dh, 'TFIELDS')
fxaddpar, dh, 'NOTE', 'Synthetic Trigger: ' + STRTRIM(STRING(old_trig), 2)
IF n_elements(num_out) NE 0 THEN BEGIN    ; We are writing synthetic data
   fxaddpar, dh, 'NAXIS2', num_out
   fxbcreate, unit_out, out_file, dh
   FOR inner = 1, num_out DO BEGIN
      fxbwrite, unit_out, out_times(*, inner-1), 1, inner
      fxbwrite, unit_out, out_rates(*, inner-1), 2, inner
      fxbwrite, unit_out, out_errs(*, inner-1), 3, inner
      ENDFOR
   ENDIF $
ELSE BEGIN                              ; For testing, need alternate path
   fxbcreate, unit_out, out_file, dh
   FOR outer = 1, n_cols DO BEGIN
      fxbread, unit_in, dummy, outer
      FOR inner = 1, n_bins DO BEGIN
         fxbwrite, unit_out, dummy(*, inner-1), outer, inner
         ENDFOR
      ENDFOR
   ENDELSE
fxbfinish, unit_out
fxbclose, unit_in

end; pro synth_bfits
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO CREATE_SYNTH, DATA_TITL, THE_PP

@wgsp_cb.pro


NCHAN = NUM_CHAN(CDET)

WIDGET_CONTROL, WIDS.TOP_BASE, SHOW=0
READ, 'Input Background and Source Livetimes: ', BACK_LIVE, OBS_LIVE
READ, 'Input Number of Source Spectra and Random integer: ', $
       NUM_OBS, RAN_SEED
WIDGET_CONTROL, WIDS.TOP_BASE, SHOW=1

; Get synthetic background rates
RAN_SEED = LONG(RAN_SEED)
SYNTH_FILE = 'SYNTH' + STRTRIM(STRING(RAN_SEED), 2) + '.FITS'
NUM_TIMES = LONG(1)
BACK_LIVE = FLOAT(BACK_LIVE)
BACK_SPECS = FLTARR(NUM_TIMES, NCHAN)

READ, 'Input two time boundaries of interval to calculate background for', $
       bck1_beg, bck1_end

STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MODEL', NUM_TIMES, $
                       bck1_beg, bck1_end, BACK_SPECS)
;STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MODEL', NUM_TIMES, $
;                       TIME_SPAN(0,0), TIME_SPAN(0,1), BACK_SPECS)

T_BEG = - BACK_LIVE
T_END = 0.0
BACK_SPECS = TRANSPOSE(BACK_SPECS) * BACK_LIVE
BACKG = FLTARR(NUM_TIMES, NCHAN)
NUM_E = LONG(NCHAN)
RAN_SEED = - ABS(RAN_SEED)
STATUS = CALL_EXTERNAL('MFIT_SHARE', 'POIDEV_WS', NUM_E, $
                       NUM_TIMES, RAN_SEED, BACK_SPECS, BACKG)

; Get synthetic observed rates
OBS_LIVE = FLOAT(OBS_LIVE)

DATA_COMM = '0.0: ' + STRING(OBS_LIVE) + ' of ' + DATA_TITL 
RUN_MODE = STRING(FORMAT='(A1)', 'I')  ; Put in interactive mode
PASS_COMMAND = STRING(FORMAT='(A2)', 'S ')       ; Synthetic fit
NEW_DATA = LONG(1)                     ; Passing in new data to MFIT
COUNT_SPAN, COMBINED_TIMES, TIME_SPAN, T_IND

WRITE_SPEC, T_IND(0), DATA_TITL, DATA_COMM, THE_PP, IO_ERR, $
           'CREATE SYNTHETIC BURST'

MFIT_INFORM, 'Creating synthetic data.'
D_E = (THRESHOLDS(1, *) - THRESHOLDS(0, *))
MODEL_CNTS = FLTARR(MAX_CHAN)
MODEL_CNTS(0: NCHAN-1) = MODEL_CNT_RATE(0: NCHAN-1, 0) * TRANSPOSE(D_E)
OBS_CNTS = (BACK_SPECS / BACK_LIVE + MODEL_CNTS) * OBS_LIVE
NUM_OBS = LONG(NUM_OBS)
OBS_RATE = FLTARR(NUM_OBS, NCHAN)
STATUS = CALL_EXTERNAL('MFIT_SHARE', 'POIDEV_WS', NUM_E, $
                       NUM_OBS, RAN_SEED, OBS_CNTS, OBS_RATE)
OBS_RATE = TRANSPOSE(OBS_RATE) 

; Set up arrays for the synthetic FITS file
MFIT_INFORM, 'Creating synthetic data file ' + SYNTH_FILE 

SYNTH_TIMES = FLTARR(2, NUM_OBS + 1)
SYNTH_TIMES(0, 0) = T_BEG
SYNTH_TIMES(1, 0) = T_END

BACKG = BACKG(0: NCHAN - 1) 
NON_NEG = WHERE(BACKG GE 0)
SYNTH_RATES = FLTARR(NCHAN, NUM_OBS + 1)
SYNTH_RATES(*, 0) = BACKG / BACK_LIVE
SYNTH_SIG = FLTARR(NCHAN, NUM_OBS + 1)
SYNTH_SIG(NON_NEG, 0) = SQRT(BACKG(NON_NEG)) / BACK_LIVE
TIME_VAR = T_END
FOR K = 1, NUM_OBS DO BEGIN
    SYNTH_TIMES(0, K) = TIME_VAR
    TIME_VAR = TIME_VAR + OBS_LIVE
    SYNTH_TIMES(1, K) = TIME_VAR
    OBS_TEMP = OBS_RATE(0: NCHAN - 1, K - 1) 
    NON_NEG = WHERE(OBS_TEMP GE 0)
    SYNTH_RATES(*, K) = OBS_TEMP / OBS_LIVE
    SYNTH_SIG(NON_NEG, K) = SQRT(OBS_TEMP(NON_NEG)) / OBS_LIVE
ENDFOR

BFITS_FILE = READ_MENU(6)
SYNTH_BFITS, BFITS_FILE, SYNTH_FILE, NUM_OBS + 1, SYNTH_TIMES, $
             SYNTH_RATES, SYNTH_SIG



END; PRO CREATE_SYNTH
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
