; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO COMBINE, IN_SPEC, IN_ERRS, IN_THRESHOLDS, LOOKUP, OUT_SPEC, OUT_ERRS, $
             OUT_THRESHOLDS

; Rebin an array according to the LOOKUP table. The inputs can either be an
;energy spectrum, with its errors and energy bin thresholds, or a burst time
;history, with its error history and time bin thresholds.
;NB: This routine converts the units by multiplying by 1/Width(thresholds).

;INPUTS:
;   IN_SPEC: Input array
;   IN_ERRS: Input error array
;   IN_THRESHOLDS: Bin thresholds array: (2, number_of_bins + 1)
;   LOOKUP: Index array (integer), missing integers in the sequence indicate
;       which bins are to be combined into one bin in the output. For ease of
;       calculation, always has the last element equal to the number of bins + 1

;OUTPUTS:
;   OUT_SPEC, OUT_ERRS: Binned arrays with dimension N_ELEMENTS(LOOKUP) - 1.
;   OUT_THRESHOLDS: Bin thresholds of the OUT_ arrays.


N_BINS = N_ELEMENTS(LOOKUP) - 1
DTHRESHOLDS = IN_THRESHOLDS(1, *) - IN_THRESHOLDS(0, *)
OUT_SPEC = FLTARR(N_BINS) 
OUT_ERRS = FLTARR(N_BINS)
DT = FLTARR(N_BINS)
OUT_THRESHOLDS = FLTARR(2, N_BINS)
FOR I = 0, N_BINS - 1 DO BEGIN
   FIRST_IND = LOOKUP(I)
   LAST_IND = LOOKUP(I + 1) - 1 
   OUT_SPEC(I) = TOTAL(IN_SPEC(FIRST_IND:LAST_IND))
   ERRS = IN_ERRS(FIRST_IND:LAST_IND)
   OUT_ERRS(I) = SQRT(TOTAL(ERRS*ERRS))
   DT(I) = TOTAL(DTHRESHOLDS(FIRST_IND:LAST_IND))
   IF DT(I) LE 0.0 THEN BEGIN         ;Eliminate zero-width bins
      DT(I) = 0.1
      OUT_SPEC(I) = 0.0
      OUT_ERRS(I) = 0.0
      ENDIF
   OUT_THRESHOLDS(0, I) = IN_THRESHOLDS(0, FIRST_IND)
   OUT_THRESHOLDS(1, I) = IN_THRESHOLDS(1, LAST_IND)
   ENDFOR
OUT_SPEC = OUT_SPEC/DT
OUT_ERRS = OUT_ERRS/DT
END ; FUNCTION COMBINE
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO INTEGRATE_SPECTRUM

COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON ERROR_C, SPECTRA_ERRORS, R_SPECTRA_ERR, SPECTRA_E_ERR, SPECTRA_T_ERR, $
                SPECTRUM_ERR, HISTORY_ERR

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

; Integrates the energy spectrum over the observed selected time bins.
RANGE = INTARR(2)
NN = SIZE(SPECTRA)
SPECTRA_T = FLTARR(NN(1))
SPECTRA_T_ERR = FLTARR(NN(1))
TOTAL_TIME = 0

N_REGIONS = SIZE(TIME_SPAN)
IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   LOWER = 1
   UPPER = N_REGIONS(1) - 1
ENDIF ELSE BEGIN
   LOWER = 0
   UPPER = 0
   ENDELSE
FOR HH = LOWER, UPPER DO BEGIN
   ER = NEAREST_EDGE(COMBINED_TIMES, TIME_SPAN(HH, *), RANGE)
   RANGE(0) = TIME_LOOKUP(RANGE(0))
   RANGE(1) = TIME_LOOKUP(RANGE(1) + 1) - 1

   FOR I = RANGE(0), RANGE(1) DO BEGIN
      DT = (TIMES(1, I) - TIMES(0, I))
      TOTAL_TIME = TOTAL_TIME + DT
      SPECTRA_T = SPECTRA_T + SPECTRA(*, I)*DT
      ERR = SPECTRA_ERRORS(*, I)*DT
      SPECTRA_T_ERR = SPECTRA_T_ERR + ERR*ERR
      ENDFOR
   ENDFOR
SPECTRA_T_ERR = SQRT(SPECTRA_T_ERR)/TOTAL_TIME
SPECTRA_T = SPECTRA_T/TOTAL_TIME

END ; INTEGRATE_SPECTRUM
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO INTEGRATE_HISTORY

COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON ERROR_C, SPECTRA_ERRORS, R_SPECTRA_ERR, SPECTRA_E_ERR, SPECTRA_T_ERR, $
                SPECTRUM_ERR, HISTORY_ERR

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

; Integrate the spectral history over the selected energy bins.
ER = INTARR(2)
NN = SIZE(SPECTRA)
SPECTRA_E = FLTARR(NN(2))
SPECTRA_E_ERR = FLTARR(NN(2))
TOTAL_ENERGY = 0

N_REGIONS = SIZE(ENERGY_SPAN)
IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   LOWER = 1
   UPPER = N_REGIONS(1) - 1
ENDIF ELSE BEGIN
   LOWER = 0
   UPPER = 0
   ENDELSE
FOR HH = LOWER, UPPER DO BEGIN
   TR = NEAREST_EDGE(COMBINED_ENERGIES, ENERGY_SPAN(HH, *), ER)
   ER(0) = ENERGY_LOOKUP(ER(0))
   ER(1) = ENERGY_LOOKUP(ER(1) + 1) - 1

   FOR I = ER(0), ER(1) DO BEGIN
      DT = (THRESHOLDS(1, I) - THRESHOLDS(0, I))
      TOTAL_ENERGY = TOTAL_ENERGY + DT
      ERR = SPECTRA_ERRORS(I, *)
      SPECTRA_E_ERR = SPECTRA_E_ERR + ERR*ERR
      SPECTRA_E = SPECTRA_E + SPECTRA(I, *)
      ENDFOR
   ENDFOR
DT = TIMES(1, *) - TIMES(0, *)
   SPECTRA_E = SPECTRA_E*DT
   SPECTRA_E_ERR = SQRT(SPECTRA_E_ERR)*DT

IF (CV_RATE_MODE EQ 'E') THEN BEGIN       ; Divide by bin width to get c/s-keV
   SPECTRA_E = SPECTRA_E / TOTAL_ENERGY
; 10/12/94 RDP Removed a SQRT from SPECTRA_E_ERR in the next line:
   SPECTRA_E_ERR = SPECTRA_E_ERR / TOTAL_ENERGY
   ENDIF
END ; INTEGRATE_HISTORY
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO INTEGRATE_BACKGROUND


COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT

; Integrate the background history over the selected energy bins.
; OUTPUTS (passed by COMMON BACK_C)
;    BACK_TIMES: Large array of times for plotting history of integrated bkgnd.
;    BACK_HISTORY: Array containing background model integrated over energy.

; We start by creating the BACK_TIMES array to be 4 times the resolution of the
; TIMES array (but no greater than 1000 elements total).
ER = INTARR(2)
NN = SIZE(SPECTRA)
BACK_TIMES = TRANSPOSE((TIMES(1, *) + TIMES(0, *)) / 2.)
N_BACK = LONG(N_ELEMENTS(BACK_TIMES))
IF N_BACK LT 250 THEN BEGIN         ;Don't make too many background bins!
   ; Sometimes we get duplicate times, want to eliminate those.
   ANT_IND = INTARR(N_BACK)
   ANT_IND(INDGEN(N_BACK)) = 1
   DUP_IND = WHERE(FIX(TIMES(0, 1: *)) EQ FIX(TIMES(1, 0: N_BACK - 1)))
   ANT_IND(DUP_IND) = 0
   B_IND = WHERE(ANT_IND, BCNT)
   IF BCNT NE 0 THEN BEGIN
      NON_DUP = TRANSPOSE(TIMES(1, WHERE(ANT_IND)))
      BACK_TIMES = [BACK_TIMES, TRANSPOSE(TIMES(0, *)), NON_DUP]
      ENDIF $
   ELSE $
      BACK_TIMES = [BACK_TIMES, TRANSPOSE(TIMES(0, *)), TIMES(1, N_BACK)]
   BACK_TIMES = BACK_TIMES(SORT(BACK_TIMES))
   N_BACK = N_ELEMENTS(BACK_TIMES)

   ; Make the array bigger; find midpoint between each array element.
   BACK_TIMES = [BACK_TIMES, (BACK_TIMES(1: *) + $
                              BACK_TIMES(0: N_ELEMENTS(BACK_TIMES) - 1)) / 2.]
   BACK_TIMES = BACK_TIMES(SORT(BACK_TIMES))
   N_BACK = LONG(N_ELEMENTS(BACK_TIMES))
   ENDIF

IF WGSP_NO_MFIT THEN BEGIN
   BACK_SPECTRA = TRANSPOSE(BACK_EVAL_DIFF(BACK_TIMES))  
ENDIF ELSE BEGIN
   BACK_SPECTRA = FLTARR(N_BACK, NN(1))    ; Holds spectra until we integrate.

   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_DIFF', N_BACK, BACK_TIMES, $
                        BACK_SPECTRA)
   BACK_SPECTRA = TRANSPOSE(BACK_SPECTRA)  ; Returns background history.
ENDELSE

BACK_HISTORY = FLTARR(N_BACK)           ; History of background model integrated
TOTAL_ENERGY = 0                        ;over selected energy range.

; Standard proceedure for picking bins spanned by the selection:
N_REGIONS = SIZE(ENERGY_SPAN)           ; Number of selected energy regions.
IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   LOWER = 1
   UPPER = N_REGIONS(1) - 1
ENDIF ELSE BEGIN                        ; Only one region, use the first column
   LOWER = 0                            ;in the ENERGY_SPAN array.
   UPPER = 0
   ENDELSE

; Loop over regions in the selection.
FOR HH = LOWER, UPPER DO BEGIN      
   TR = NEAREST_EDGE(COMBINED_ENERGIES, ENERGY_SPAN(HH, *), ER)
   ER(0) = ENERGY_LOOKUP(ER(0))
   ER(1) = ENERGY_LOOKUP(ER(1) + 1) - 1

   ; Finally, do the integral over energy in each region spanned by selection.
   FOR I = ER(0), ER(1) DO BEGIN
      DT = (THRESHOLDS(1, I) - THRESHOLDS(0, I))
      TOTAL_ENERGY = TOTAL_ENERGY + DT
      BACK_HISTORY = BACK_HISTORY + BACK_SPECTRA(I, *)
      ENDFOR
   ENDFOR

; Normalize the result to get the proper units:
IF (CV_RATE_MODE EQ 'E') THEN BEGIN       ; Divide by bin width to get c/s-keV
   BACK_HISTORY = BACK_HISTORY / TOTAL_ENERGY
   ENDIF

BIN_BACK

END ; INTEGRATE_BACKGROUND
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO BIN_BACK

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON CNT_C, NUM_CHAN

COMMON IN_ARGS_C, TOPLABEL, NUM_DET, DET_NAMES, $
          NEW_DRM, NEW_DATA, DATA_FILE_NAME, COMMENT_DATA, $
          CHAN_OFFSET, RES_FRAC_AT511, RES_EXP, $
          FIRST_CHAN_PLOT, LAST_CHAN_PLOT, CALLING_PROG, CTOE_LABEL, $
          USED_REBIN, REBIN_CHAN, CDET

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT

; Set up arrays for 'BIN FOR SIGNIFICANCE' 
;by binning the data in both directions, i.e.: by energy and time.

NCHAN = NUM_CHAN(CDET)

COUNT_SPAN, COMBINED_TIMES, TIME_SPAN, TIME_IND 
T_BEG = TRANSPOSE(COMBINED_TIMES(0, TIME_IND))     
T_END = TRANSPOSE(COMBINED_TIMES(1, TIME_IND))
NUM_TIMES = LONG(N_ELEMENTS(TIME_IND))
TEMP_FIT = FLTARR(NUM_TIMES, NCHAN)
IF WGSP_NO_MFIT THEN BEGIN
   TEMP_FIT = TRANSPOSE(BACK_EVAL_MODEL(T_BEG, T_END))
ENDIF ELSE BEGIN
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MODEL', NUM_TIMES, $
                     T_BEG, T_END, TEMP_FIT)
   TEMP_FIT = TRANSPOSE(TEMP_FIT(*, 0: NCHAN - 1))
ENDELSE

; Add up bins by time:
DEL_TIME = T_END - T_BEG
TOT_TIME = TOTAL(DEL_TIME)
BACK_FIT = (TEMP_FIT # DEL_TIME) / TOT_TIME

IF WGSP_NO_MFIT THEN BEGIN
   TEMP_ERRS = TRANSPOSE(BACK_EVAL_MSIG(T_BEG, T_END))
ENDIF ELSE BEGIN
   TEMP_ERRS = FLTARR(NUM_TIMES, MAX_CHAN)
   AFUNC = FLTARR(5, NUM_TIMES)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MSIG', NUM_TIMES, $
               T_BEG, T_END, AFUNC, TEMP_ERRS)
   TEMP_ERRS = TRANSPOSE(TEMP_ERRS(*, 0: NCHAN - 1))
ENDELSE
BACK_ERRS = SQRT((TEMP_ERRS * TEMP_ERRS) # (DEL_TIME * DEL_TIME)) / TOT_TIME

; Now by energy:
T_BEG = TRANSPOSE(TIMES(0, *))
T_END = TRANSPOSE(TIMES(1, *))
NUM_TIMES = LONG(N_ELEMENTS(T_BEG))
COUNT_SPAN, THRESHOLDS, ENERGY_SPAN, ENRG_IND 
IF WGSP_NO_MFIT THEN BEGIN
   TEMP_FIT = BACK_EVAL_MODEL(T_BEG, T_END)
   TEMP_FIT = TRANSPOSE(TEMP_FIT(*, ENRG_IND))
ENDIF ELSE BEGIN
   TEMP_FIT = FLTARR(NUM_TIMES, MAX_CHAN)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MODEL', NUM_TIMES, $
                     T_BEG, T_END, TEMP_FIT)
   TEMP_FIT = TRANSPOSE(TEMP_FIT(*, ENRG_IND))
ENDELSE

IF WGSP_NO_MFIT THEN BEGIN
   TEMP_ERRS = BACK_EVAL_MSIG(T_BEG, T_END)
   TEMP_ERRS = TRANSPOSE(TEMP_ERRS(*, ENRG_IND))
ENDIF ELSE BEGIN
   TEMP_ERRS = FLTARR(NUM_TIMES, MAX_CHAN)
   AFUNC = FLTARR(5, NUM_TIMES)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_EVAL_MSIG', NUM_TIMES, $
               T_BEG, T_END, AFUNC, TEMP_ERRS)
   TEMP_ERRS = TRANSPOSE(TEMP_ERRS(*, ENRG_IND))
ENDELSE

SUM_BACK = FLTARR(NUM_TIMES)
SUM_ERRS = FLTARR(NUM_TIMES)
DEL_E = THRESHOLDS(1, ENRG_IND) - THRESHOLDS(0, ENRG_IND)
TOT_E = TOTAL(DEL_E)
FOR I = 0, NUM_TIMES - 1 DO BEGIN
   SUM_BACK(I) = TOTAL(TEMP_FIT(*, I))
   SUM_ERRS(I) = SQRT(TOTAL(TEMP_ERRS(*, I) * TEMP_ERRS(*, I)))
   ENDFOR

; Normalize the result to get the proper units:
IF (CV_RATE_MODE EQ 'E') THEN BEGIN       ; Divide by bin width to get c/s-keV
   SUM_BACK = SUM_BACK / TOT_E
   SUM_ERRS = SUM_ERRS / TOT_E
   ENDIF

END ; PRO BIN_BACK
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO READ_LOOKUPS, NAME, SAVE_NAME, ENERGY_SCALE, RATE_SCALE, BAD_FLAG

COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON ERROR_C, SPECTRA_ERRORS, R_SPECTRA_ERR, SPECTRA_E_ERR, SPECTRA_T_ERR, $
                SPECTRUM_ERR, HISTORY_ERR

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON WINDOWS_C, SPECTRUM_WINDOW, HISTORY_WINDOW, PLOT_WINDOW, $
                  PROCESSED_HISTORY, PROCESSED_SPECTRUM, LAYOUT_WINDOW

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON CURRENT_MENU, CURRENT_VALUES

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
                   WGSP_FITS, WGSP_LOOKUP, WGSP_DRM


BAD_FLAG = 0 ; No error to start

         BREAK_FILE, NAME, LDEV, LDIR, LNAME, LEXT, LVERSION
         IF LDIR EQ '' THEN BEGIN
            OPENR, FL, WGSP_SCRATCH + STRLOWCASE(LNAME + LEXT), $
                   ERROR = STATUS, /GET_LUN
         ENDIF ELSE BEGIN
            OPENR, FL, NAME, ERROR = STATUS, /GET_LUN
         ENDELSE

         MESSAGE = 'Restoring saved lookup parameters.'
         INFORM, MESSAGE
         MESSAGE = ''

         SAVE_DEF_EL = DEFAULT_EL
         SAVE_DEF_TL = DEFAULT_TL
         SAVE_NRG_SPAN = ENERGY_SPAN
         SAVE_NRG_LUP = ENERGY_LOOKUP
         IF STATUS NE 0 THEN BEGIN
            MESSAGE = 'Could not open ' + NAME + '.  Ready.'
            BAD_FLAG = 1
            GOTO, BAD_TABLE
            ENDIF $
         ELSE BEGIN
            MESSAGE = 'Error reading the lookup tables file.  Ready.'
            ON_IOERROR, BAD_TABLE
            N = 0
            READF, FL, N
            IF N EQ 0 THEN $
               TABLE = INTARR(1) $
            ELSE $
               TABLE = INTARR(N)
            READF, FL, TABLE
            NT = SIZE(THRESHOLDS)
            IF N EQ 0 THEN BEGIN
               N = NT(2) + 1
               TABLE = INDGEN(N)
               ENDIF
            TABLE = TABLE(SORT(TABLE))
            DIFF = MIN(TABLE(1:*) - TABLE)
            IF TABLE(0) NE 0 THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Energy table does not begin with ' + $
                         'channel zero.  Ready.'
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE IF TABLE(N - 1) NE NT(2) THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Energy table does not end with ' + $
                         'the last threshold in the FITS file.  Ready.'
               PRINT, TABLE
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE IF DIFF EQ 0 THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Energy table contains a channel ' + $
                         'more than once.  Ready.'
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE DEFAULT_EL = TABLE

            READF, FL, N
            IF N NE 0 THEN BEGIN
               TABLE = FLTARR(N)
               READF, FL, TABLE
               ENERGY_SPAN = FLTARR((N/2),2)
               ENERGY_SPAN(0: (N/2) - 1, 0) = TABLE(0: (N/2) - 1)
               ENERGY_SPAN(0: (N/2) - 1, 1) = TABLE((N/2): *)
               ENDIF

            ENERGY_LOOKUP = DEFAULT_EL

            READF, FL, N
            IF N EQ 0 THEN $
               TABLE = INTARR(1) $
            ELSE $
               TABLE = INTARR(N)
            READF, FL, TABLE
            NT = SIZE(TIMES)
            IF N EQ 0 THEN BEGIN
               N = NT(2) + 1
               TABLE = INDGEN(N)
               ENDIF
            TABLE = TABLE(SORT(TABLE))
            DIFF = MIN(TABLE(1:*) - TABLE)
            IF TABLE(0) NE 0 THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Time table does not begin ' + $
                         'with channel zero.  Ready.'
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE IF TABLE(N - 1) NE NT(2) THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Time table does not end ' + $
                         'with the last threshold in the FITS file.  Ready.'
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE IF DIFF EQ 0 THEN BEGIN
               MESSAGE = 'LOOKUP ERROR:  Time table contains a ' + $
                         'channel more than once.  Ready.'
               BAD_FLAG = 1
               GOTO, BAD_TABLE
               ENDIF $
            ELSE DEFAULT_TL = TABLE

            READF, FL, N
            IF N NE 0 THEN BEGIN
               TABLE = FLTARR(N)
               READF, FL, TABLE
               TIME_SPAN = FLTARR((N/2),2)
               TIME_SPAN(0: (N/2) - 1, 0) = TABLE(0: (N/2) - 1)
               TIME_SPAN(0: (N/2) - 1, 1) = TABLE((N/2): *)
               ENDIF

            DO_FIT = 0
            READF, FL, N
            N_BSPAN = N
            IF N NE 0 THEN BEGIN
               TABLE = FLTARR(N)
               READF, FL, TABLE
               BACK_SPAN = FLTARR((N/2),2)
               BACK_SPAN(0: (N/2) - 1, 0) = TABLE(0: (N/2) - 1)
               BACK_SPAN(0: (N/2) - 1, 1) = TABLE((N/2): *)
               DO_FIT = 1
               ENDIF

            IF NOT EOF(FL) THEN BEGIN      ;Read in the Plot Menu
               DUMMY = ' '
               FOR I = 1, 3 DO BEGIN
                  READF, FL, DUMMY
                  CURRENT_VALUES(I-1) = DUMMY
                  ENDFOR
               ENDIF

            IF NOT EOF(FL) THEN BEGIN      ;Read in the plot ranges
               READF, FL, TIME_RANGE
               READF, FL, HIST_RANGE
               READF, FL, ENERGY_RANGE
               READF, FL, RATE_RANGE
               ENDIF

            GOT_BACK_ORDER = 0
            IF NOT EOF(FL) THEN BEGIN      ;Read in the background model order
               IF N_BSPAN NE 0 THEN BEGIN ;If it exists...
                  READF, FL, NDEGREE
                  GOT_BACK_ORDER = 1
                  ENDIF
               ENDIF

            FREE_LUN, FL
            ON_IOERROR, NULL

            TIME_LOOKUP = DEFAULT_TL

            COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, HISTORY, $
                  HISTORY_ERR, COMBINED_TIMES
            COMBINE, SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, ENERGY_LOOKUP, $
                  SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES

            HAVE_BACK = 0           ;Assume no background until BACK_FIT runs
            IF DO_FIT THEN BEGIN    ;Background regions were read, do BACK_FIT
               BACK_CHISQ = FLTARR(MAX_CHAN)
               IF GOT_BACK_ORDER THEN $  ;Background order NDEGREE was read
                  BACK_FIT, BACK_CHISQ $
               ELSE $               ;No background order was read; get from user
                  BACK_FIT, BACK_CHISQ, /GET_ORDER
               ENDIF

            INFORM, 'Updating menu.'
            DIRTY = 0               ;Just read in lookup, fresh background model
            MESSAGE = ''

            INTEGRATE_HISTORY
            INTEGRATE_SPECTRUM
            IF HAVE_BACK THEN $
               INTEGRATE_BACKGROUND

            COMBINE, SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, ENERGY_LOOKUP, $
                  SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES
            !X.TITLE = READ_MENU(4)
            !Y.TITLE = READ_MENU(5)
            WSET, SPECTRUM_WINDOW
            ENERGY_SCALE = READ_MENU(1)
            RATE_SCALE = READ_MENU(2)

            TITLE = CREATE_LABEL(COMBINED_TIMES, TIME_SPAN, ' s')
            STD_DISPLAY, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, $
                  ENERGY_RANGE, RATE_RANGE, ENERGY_SPAN, TITLE, $
                  ENERGY_SCALE, RATE_SCALE, 2, 'SPECTRUM'
            IF ENERGY_SCALE EQ 'LINEAR' THEN BEGIN
               DISPLAY_RAINBOW, THRESHOLDS
               ENDIF

            COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, HISTORY, $
                  HISTORY_ERR, COMBINED_TIMES
            !X.TITLE = READ_MENU(3)
            !Y.TITLE = READ_MENU(5)
            WSET, HISTORY_WINDOW
;            TIME_SCALE = 'LINEAR'
;            RATE_SCALE = 'LINEAR'
            TIME_SCALE = READ_MENU(9)
            RATE_SCALE = READ_MENU(10)
            IF (HIST_RANGE(1) LT MIN(HISTORY)) OR $
               (HIST_RANGE(0) GT MAX(HISTORY)) THEN $
               HIST_RANGE = [MIN(HISTORY), MAX(HISTORY)]

            TITLE = CREATE_LABEL(COMBINED_ENERGIES, ENERGY_SPAN, ' keV')
            STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, $
                  TIME_RANGE, HIST_RANGE, TIME_SPAN, TITLE, TIME_SCALE, $
                  RATE_SCALE, 1, 'HISTORY' 
            ENDELSE
BAD_TABLE:
         IF (MESSAGE NE '') THEN BEGIN
            INFORM, MESSAGE
            ; Restore to the state prior to the crash
            DEFAULT_EL    = SAVE_DEF_EL
            DEFAULT_TL    = SAVE_DEF_TL 
            ENERGY_SPAN   = SAVE_NRG_SPAN 
            ENERGY_LOOKUP = SAVE_NRG_LUP 
         ENDIF 

END; PRO READ_LOOKUPS
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO SAVE_LOOKUPS, NAME

COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
                   WGSP_FITS, WGSP_LOOKUP, WGSP_DRM

COMMON CURRENT_MENU, CURRENT_VALUES


DEFAULT_EL = ENERGY_LOOKUP
DIFF_EL = MAX(DEFAULT_EL(1:*) - DEFAULT_EL)
DEFAULT_TL = TIME_LOOKUP
DIFF_TL = MAX(DEFAULT_TL(1:*) - DEFAULT_TL)
N_ESPAN = N_ELEMENTS(ENERGY_SPAN)
E_SPAN = FLTARR(N_ESPAN)
E_SPAN(0:(N_ESPAN / 2) - 1) = ENERGY_SPAN(*, 0)
E_SPAN((N_ESPAN / 2): *) = ENERGY_SPAN(*, 1)

N_TSPAN = N_ELEMENTS(TIME_SPAN)
T_SPAN = FLTARR(N_TSPAN)
T_SPAN(0:(N_TSPAN / 2) - 1) = TIME_SPAN(*, 0)
T_SPAN((N_TSPAN / 2): *) = TIME_SPAN(*, 1)
      
IF (HAVE_BACK) THEN BEGIN
   N_BSPAN = N_ELEMENTS(BACK_SPAN)
   B_SPAN = FLTARR(N_BSPAN)
   B_SPAN(0:(N_BSPAN / 2) - 1) = BACK_SPAN(*, 0)
   B_SPAN((N_BSPAN / 2): *) = BACK_SPAN(*, 1)
ENDIF ELSE N_BSPAN = 0

BREAK_FILE, NAME, LDEV, LDIR, LNAME, LEXT, LVERSION
IF (LDIR EQ '') THEN BEGIN
   OPENW, FL, WGSP_SCRATCH + STRLOWCASE(LNAME + LEXT), ERROR = STATUS, /GET_LUN
ENDIF ELSE BEGIN
   OPENW, FL, NAME, ERROR = STATUS, /GET_LUN
ENDELSE
         
IF (STATUS EQ 0) THEN BEGIN

   IF DIFF_EL EQ 1 THEN BEGIN
      PRINTF, FL, LONG(0)
      PRINTF, FL, DEFAULT_EL(0)
   ENDIF ELSE BEGIN
      PRINTF, FL, N_ELEMENTS(DEFAULT_EL)
      PRINTF, FL, DEFAULT_EL
   ENDELSE
   PRINTF, FL, N_ESPAN
   IF (N_ESPAN NE 0) THEN $
      PRINTF, FL, E_SPAN
   IF DIFF_TL EQ 1 THEN BEGIN
      PRINTF, FL, LONG(0)
      PRINTF, FL, DEFAULT_TL(0)
   ENDIF ELSE BEGIN
      PRINTF, FL, N_ELEMENTS(DEFAULT_TL)
      PRINTF, FL, DEFAULT_TL
   ENDELSE
   PRINTF, FL, N_TSPAN
   IF (N_TSPAN NE 0) THEN $
      PRINTF, FL, T_SPAN
   PRINTF, FL, N_BSPAN
   IF (N_BSPAN NE 0) THEN $
      PRINTF, FL, B_SPAN
   FOR I = 1, 3 DO BEGIN
       DUMMY = READ_MENU(I-1)
       PRINTF, FL, DUMMY
   ENDFOR
   PRINTF, FL, TIME_RANGE
   PRINTF, FL, HIST_RANGE
   PRINTF, FL, ENERGY_RANGE
   PRINTF, FL, RATE_RANGE
   IF (N_BSPAN NE 0) THEN $
      PRINTF, FL, NDEGREE

   FREE_LUN, FL
   DIRTY = 0                 ;Reset after saving lookups

ENDIF ELSE INFORM, 'Could not open ' + NAME + '.'



END; PRO SAVE_LOOKUPS
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO DISPLAY_RAINBOW, THRESHOLDS

N = SIZE(THRESHOLDS)
E_MIN = THRESHOLDS(0, 0)
E_MAX = THRESHOLDS(1, N(2) - 1)
E_RANGE = E_MAX - E_MIN

IF E_MIN LT !X.CRANGE(0) THEN E0 = !X.CRANGE(0) ELSE E0 = E_MIN
IF E_MAX GT !X.CRANGE(1) THEN E1 = !X.CRANGE(1) ELSE E1 = E_MAX

LO_COLOR = 64.0*(E0 - E_MIN)/E_RANGE
HI_COLOR = 64.0*(E1 - E_MIN)/E_RANGE

PIX_DATA = (!P.CLIP(2) - !P.CLIP(0))/(!X.CRANGE(1) - !X.CRANGE(0))
P0 = FIX(!P.CLIP(0) + (E0 - !X.CRANGE(0))*PIX_DATA + 0.5)
P1 = FIX(!P.CLIP(0) + (E1 - !X.CRANGE(0))*PIX_DATA + 0.5)
N = P1 - P0 + 1
ROW = 3*BYTE(LO_COLOR + HI_COLOR*INDGEN(N)/N) + 1
IMAGE = BYTARR(N, 8)
FOR I = 0, 7 DO IMAGE(*, I) = ROW
TV, IMAGE, P0, 2
END ; PRO DISPLAY_RAINBOW
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO GET_FITS_STUFF, FILENAME, INTERVALS, THRESHOLDS, $
                    SPECTRA, SPECTRA_ERRORS, RETURN_STATUS, GET_ANOTHER

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON PHOT_C, PHOT_ENERGY, PHOT_WIDTH, PHOT_OBS_RATE, PHOT_OBS_SIG, $
          MODEL_PHOT_RATE, NUM_EBINS

COMMON DRM_C, CH_OUT, E_IN, DRM, DRM_LABEL, DRM_DIM1, $
          DRM_DIM2, DRM_DIM3

COMMON CNT_C, NUM_CHAN, CHAN_ENERGY, CHAN_WIDTH, CNT_RATE_SIG, NET_CRATE, $
          OBS_CRATE, BACK_CRATE, BACK_CSIG, LIVE_TIME, $
          LOOKUP_FILE, FIT_WIN

COMMON IN_ARGS_C, TOPLABEL, NUM_DET, DET_NAMES, $
          NEW_DRM, NEW_DATA, DATA_FILE_NAME, COMMENT_DATA, $
          CHAN_OFFSET, RES_FRAC_AT511, RES_EXP, $
          FIRST_CHAN_PLOT, LAST_CHAN_PLOT, CALLING_PROG, CTOE_LABEL, $
          USED_REBIN, REBIN_CHAN, CDET

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON FITS_C, HEADER, CH, DH

COMMON ANNOTATION_C, NOTES, N_NOTES, N_ANNOTES, ANNOTES, ANN_X, ANN_Y

COMMON CURRENT_MENU, CURRENT_VALUES

COMMON TWIDS, WIDS

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH


; Assume no error to start
RETURN_STATUS = 1  

MESSAGE = ''
WHILE MESSAGE NE 'O.K.' DO BEGIN

   MESSAGE = FXOPEN_FITS(FILENAME, FILETYPE, TRIGGER, HEADER) 

   IF MESSAGE EQ 'O.K.' THEN BEGIN
      IF FILETYPE EQ 'BATSE BURST SPECTRA' THEN BEGIN
         N_NOTES = 0
         MESSAGE = FXREAD_SPECTRA(FILENAME, INTERVALS, THRESHOLDS, SPECTRA, $
                                  SPECTRA_ERRORS, CAL_NAMES, CH, DH)
         IF MESSAGE NE 'O.K.' THEN BEGIN
            INFORM, MESSAGE
            RETURN_STATUS = 0
            RETURN
         ENDIF

         MESSAGE = FXFITS_NOTES(DH, NOTES, N_NOTES)
         IF MESSAGE NE 'O.K.' THEN BEGIN
            INFORM, MESSAGE
            RETURN_STATUS = 0
            RETURN
         ENDIF

         BREAK_FILE, FILENAME, D_FDEV, D_FDIR, D_FNAME, D_FEXT, D_FVER
         D_FILENAME = D_FNAME + D_FEXT

         ZIND = WHERE(STRTRIM(DATA_FILE_NAME, 2) EQ D_FILENAME)
         IF ZIND(0) EQ -1 THEN BEGIN           ; Didn't find the file in list:
            IF (NUM_DET LT MAX_DET) AND GET_ANOTHER THEN BEGIN
               NUM_DET = NUM_DET + 1           ; Put new data in first slot...
            ENDIF
            CDET = NUM_DET - 1                 ; Set Current DET to the slot...
         ENDIF $
         ELSE BEGIN
            CDET = ZIND(0)               ; Found file; info goes in prev. slot.
         ENDELSE
         DSEL = SXPAR(DH, 'DSELECT')
         W = WHERE(BYTE(DSEL) EQ 89)    ;Parse the string for occurances of 'Y'
         DETECTORS = 7 - W              ;Detector number is the complement.
         NDETS = LONG(N_ELEMENTS(DETECTORS))

         CHAN_OFFSET(CDET) = LONG(SXPAR(DH, 'LO_CHAN'))
         NUM_CHAN(CDET) = $
             LONG(SXPAR(DH, 'UP_CHAN')) - CHAN_OFFSET(CDET) + LONG(1)
         RES_FRAC_AT511(CDET) = SXPAR(DH, 'RF_511')
         RES_EXP(CDET) = SXPAR(DH, 'R_EXP')
         LTIME_COR = SXPAR(DH, 'LTIMECOR')

         THE_DET = STRTRIM(SXPAR(DH, 'DET_MODE'), 2) + ' #' + $
                   STRTRIM(STRING(DETECTORS(0)), 2)

         FOR KK = 1, NDETS - 1 DO $
            THE_DET = THE_DET + ',' + STRTRIM(STRING(DETECTORS(KK)), 2)
                          
         ONE_DET_NAME = STRING(REPLICATE(32B, 12))
         STRPUT, ONE_DET_NAME, THE_DET
         DET_NAMES(CDET) = ONE_DET_NAME

         CTOE_LABEL = STRING(REPLICATE(32B, 70))
         STRPUT, CTOE_LABEL, CAL_NAMES

         BAD_IND = WHERE((SPECTRA_ERRORS LT 0.0), BAD_CNT)
         IF (BAD_CNT NE 0) THEN BEGIN
            ; Plot data gaps for user inspection
            PLOT_GAPS, TIMES, SPECTRA, SPECTRA_ERRORS, $
                       CHAN_OFFSET, CDET, WGSP_SCRATCH, FILENAME
            INFORM, 'Negative error rates in FITS file set to 0.0.'
            SPECTRA_ERRORS(BAD_IND) = 0.0
         ENDIF

      ENDIF $
      ELSE BEGIN ;Wrong file type: maybe it's a DRM file
         MESSAGE = 'Wrong file type: re-enter FITS filename.'
         INFORM, MESSAGE
         RETURN_STATUS = 0
         RETURN
      ENDELSE
   ENDIF $
   ELSE BEGIN
      RETURN_STATUS = 0
      INFORM, Message
      WAIT, 3
      RETURN
   ENDELSE
ENDWHILE

RETURN_STATUS = 1
INFORM, ' '


END; PRO GET_FITS_STUFF
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO REBIN_SPECTRA

COMMON RATE_C, SPECTRA, R_SPECTRA, SPECTRA_E, SPECTRA_T, $
               SPECTRUM, HISTORY, RATE_RANGE, HIST_RANGE

COMMON ERROR_C, SPECTRA_ERRORS, R_SPECTRA_ERR, SPECTRA_E_ERR, SPECTRA_T_ERR, $
                SPECTRUM_ERR, HISTORY_ERR

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON WINDOWS_C, SPECTRUM_WINDOW, HISTORY_WINDOW, PLOT_WINDOW, $
                  PROCESSED_HISTORY, PROCESSED_SPECTRUM, LAYOUT_WINDOW

; Rebin the spectra according to the COMBINED_ index arrays. Purpose: to make
;plotting quicker in PLOT_SPECTRA.
; 9/6/93: Added livetime weighting to the rebinned spectra so that the spectra
;are displayed consistently in Stacked_Spectra and MFIT.
DE = THRESHOLDS(1, *) - THRESHOLDS(0, *)
NN = SIZE(SPECTRA)
N_ENERGY_BINS = NN(1)
N_TIME_BINS = N_ELEMENTS(TIME_LOOKUP) - 1
R_SPECTRA_TEMP = FLTARR(N_ENERGY_BINS, N_TIME_BINS)
R_SPECTRA_ERR_TEMP = FLTARR(N_ENERGY_BINS, N_TIME_BINS)
   FOR J = 0, N_TIME_BINS - 1 DO BEGIN
      J0 = TIME_LOOKUP(J)
      J1 = TIME_LOOKUP(J + 1) - 1
      R = FLTARR(N_ENERGY_BINS)
      R_ERR = FLTARR(N_ENERGY_BINS)
      TOTAL_TIME = 0.0
      FOR K = J0, J1 DO BEGIN
         NIND = WHERE(SPECTRA(*, K), NEG_COUNT)         ;Don't divide by zeros!
         IF NEG_COUNT NE 0 THEN BEGIN
            LIVE_T = SPECTRA(NIND, K) / (SPECTRA_ERRORS(NIND, K) * $
                                      SPECTRA_ERRORS(NIND, K))
            DT = TOTAL(LIVE_T) / N_ELEMENTS(NIND);Make the livetime the average.
            ENDIF $
         ELSE DT = 1.0E6  ; Make the livetime large to get small rates
         TOTAL_TIME = TOTAL_TIME + DT
         R = R + SPECTRA(*, K) * DT
         ERRS = SPECTRA_ERRORS(*, K) * DT
         R_ERR = R_ERR + ERRS*ERRS
         ENDFOR
      DT_DE = TOTAL_TIME
      R_SPECTRA_TEMP(*, J) = R / DT_DE
      R_SPECTRA_ERR_TEMP(*, J) = SQRT(R_ERR) / DT_DE
      ENDFOR
N_ENERGY_BINS = N_ELEMENTS(ENERGY_LOOKUP) - 1
R_SPECTRA = FLTARR(N_ENERGY_BINS, N_TIME_BINS)
R_SPECTRA_ERR = FLTARR(N_ENERGY_BINS, N_TIME_BINS)
FOR I = 0, N_ENERGY_BINS - 1 DO BEGIN
   I0 = ENERGY_LOOKUP(I)
   I1 = ENERGY_LOOKUP(I + 1) - 1
      R = FLTARR(N_TIME_BINS)
      R_ERR = FLTARR(N_TIME_BINS)
      FOR K = I0, I1 DO BEGIN
         R = R + R_SPECTRA_TEMP(K, *)
         ERRS = R_SPECTRA_ERR_TEMP(K, *)
         R_ERR = R_ERR + ERRS*ERRS
         ENDFOR
      DT_DE = TOTAL(DE(I0:I1))
      IF DT_DE EQ 0.0 THEN BEGIN        ;Handle zero-width thresholds
         R_SPECTRA(I, *) = 0.0
         R_SPECTRA_ERR(I, *) = 0.0
         ENDIF $
      ELSE BEGIN
         R_SPECTRA(I, *) = R/DT_DE
         R_SPECTRA_ERR(I, *) = SQRT(R_ERR)/DT_DE
         ENDELSE
   ENDFOR
END ; PRO REBIN_SPECTRA
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO NEW_HIST, X, Y, C, DY

IF N_PARAMS(0) LT 3 THEN C = !P.COLOR
N = SIZE(X)
IF N(0) EQ 1 THEN BEGIN
   OPLOT, X, [Y, Y], COLOR = C 
   IF N_PARAMS(0) GE 4 THEN BEGIN
      XX = 0.5*(X(0, 0) + X(1, 0))
      XX = [XX, XX]
      YY = Y(0) + [- DY(0), DY(0)]
      OPLOT, XX, YY, COLOR = C 
      ENDIF
   ENDIF $
ELSE BEGIN
   DX = X(0, 1:*) - X(1, 0:N(2) - 2)
   GAP_TOL = MAX([5.0E-5, 0.001*MIN(ABS(X(1, *) - X(0, *)))])
   GAPS = WHERE(DX GT GAP_TOL)

   IF GAPS(0) EQ -1 THEN BEGIN
      N_GAPS = 0
      GAPS = [-1, N(2) - 1]
      ENDIF $
   ELSE BEGIN
      N_GAPS = N_ELEMENTS(GAPS)
      GAPS = [-1, GAPS, N(2) - 1]
      ENDELSE

   FOR I = 0, N_GAPS DO BEGIN
      GOOD_X = [TRANSPOSE(X(0, GAPS(I) + 1:GAPS(I + 1))), X(1, GAPS(I + 1))]
      GOOD_Y = Y(GAPS(I) + 1:GAPS(I + 1))
      NX = N_ELEMENTS(GOOD_X)
      NY = N_ELEMENTS(GOOD_Y)
      XX = FLTARR(2*NX)
      YY = FLTARR(2*NY)
      EVEN = 2*INDGEN(NX)
      XX(EVEN) = GOOD_X
      XX(EVEN + 1) = GOOD_X
      EVEN = 2*INDGEN(NY)
      YY(EVEN) = GOOD_Y
      YY(EVEN + 1) = GOOD_Y
      K = 2*NX - 2
      OPLOT, XX(1:K), YY, COLOR = C 
      ENDFOR

   IF N_PARAMS(0) GE 4 THEN BEGIN
      X_RANGE = 0.005*(!X.CRANGE(1) - !X.CRANGE(0))
      DX = X(1, *) - X(0, *)
      FOR I = 0, N(2) - 1 DO BEGIN
         IF DX(I) GT X_RANGE THEN BEGIN
            XX = 0.5*(X(0, I) + X(1, I))
            XX = [XX, XX]
            YY = Y(I) + [- DY(I), DY(I)]
            OPLOT, XX, YY, COLOR = C 
            ENDIF
         ENDFOR
      ENDIF
   ENDELSE
END; PRO NEW_HIST
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Makes a standard plot of the burst time history or spectrum.
;  The axis ranges are always plotted as EXACT, and are set by X_RANGE, Y_RANGE.
;  
;  Call ROUND_AXES before STD_DISPLAY if nice axis ranges are desired.
;  DTYPE = 'HISTORY' or 'SPECTRUM'

PRO STD_DISPLAY, STD_SPECTRUM, STD_COMBINED_ERRS, STD_COMBINED_THRESHOLDS, $
                 X_RANGE, Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, SHOW_BACK, $
                 DTYPE

@wgsp_cb.pro


DTYPE        = STRUPCASE(DTYPE)
!P.CHARTHICK = GLOBAL_CHARTHICK
!P.THICK     = GLOBAL_LINETHICK
!X.THICK     = GLOBAL_LINETHICK
!Y.THICK     = GLOBAL_LINETHICK

XR      = X_RANGE
YR      = Y_RANGE
X_TYPE  = LONG(X_SCALE EQ 'LOG')
Y_TYPE  = LONG(Y_SCALE EQ 'LOG')

; STD_DISPLAY always plots exact
;X_STYLE = 0                     ; Round the values for linear axes
;Y_STYLE = 16

; RSM, 10 MAR, 1995: Let the minumum log axis range be set from the data
;IF (X_SCALE EQ 'LOG') AND (XR(0) LE 0) THEN XR(0) = XR(1)/1000
;IF (Y_SCALE EQ 'LOG') AND (YR(0) LE 0) THEN YR(0) = YR(1)/1000
IF (X_SCALE EQ 'LOG') AND (XR(0) LE 0) THEN BEGIN
   XR(0) = MIN(STD_COMBINED_THRESHOLDS(WHERE(STD_COMBINED_THRESHOLDS GT 0.0)))
ENDIF
IF (Y_SCALE EQ 'LOG') AND (YR(0) LE 0) THEN BEGIN
   YR(0) = MIN(STD_SPECTRUM(WHERE(STD_SPECTRUM GT 0.0)))
ENDIF

; Force the value to be exact
X_STYLE = 1                  
Y_STYLE = 17

PLOT, XR, YR, /NODATA, XTYPE = X_TYPE, YTYPE = Y_TYPE, XSTYLE = X_STYLE, $
              YSTYLE = Y_STYLE

NEW_HIST, STD_COMBINED_THRESHOLDS, STD_SPECTRUM, !P.COLOR, STD_COMBINED_ERRS

IF (N_ELEMENTS(SHOW_BACK) NE 0) THEN BEGIN   ; We display the background info
   IF (SHOW_BACK EQ 1) THEN BEGIN            ; Display the background history
      N = SIZE(STD_COMBINED_THRESHOLDS)
      BR = INTARR(2)
      BACK_ARR = INTARR(N(2))
      BACK_ARR(INDGEN(N(2))) = 0
      BACK_IND = INDGEN(N(2))
      IF (N_ELEMENTS(BACK_TIMES) GT 1) THEN BEGIN
         NBT = N_ELEMENTS(BACK_TIMES)
         COMBINED_BTIMES = FLTARR(2, NBT)
         COMBINED_BTIMES(0, 1: *) = BACK_TIMES(1: *)
         COMBINED_BTIMES(1, 0: NBT - 2) = BACK_TIMES(0: NBT - 2)
         COMBINED_BTIMES(0, 0) = BACK_TIMES(0) - 100.
         COMBINED_BTIMES(1, NBT - 1) = BACK_TIMES(NBT - 1) + 100.
         ENDIF

      BACK_REGIONS = SIZE(BACK_SPAN)
      BACK_I = INTARR(2)
      IF HAVE_BACK THEN BEGIN
         FOR HH = 0, BACK_REGIONS(1) - 1 DO BEGIN
            TR = NEAREST_EDGE(STD_COMBINED_THRESHOLDS, BACK_SPAN(HH, *), BR)
            TR(0) = TR(0) + 0.0001
            TR(1) = TR(1) - 0.0001
            IF N_ELEMENTS(BACK_TIMES) GT 1 THEN BEGIN
               TR = NEAREST_EDGE(COMBINED_BTIMES, TR, BACK_I)
               MININ = MAX([0, BACK_I(0) - 1])
               MAXIN = MIN([N_ELEMENTS(BACK_TIMES) - 1, BACK_I(1) + 1])
               OPLOT, BACK_TIMES(MININ: MAXIN), BACK_HISTORY(MININ: MAXIN)
               ENDIF
            I0 = BR(0)
            I1 = BR(1)
            FOR KK = I0, I1 DO BACK_ARR(KK) = 1
            ENDFOR
         BACK_IND = BACK_IND(WHERE(BACK_ARR))
                   
         BACK_HIST = STD_SPECTRUM(BACK_IND)
         BACK_THRES = STD_COMBINED_THRESHOLDS(*, BACK_IND)
         LINE_SAVE = !P.LINESTYLE
         !P.LINESTYLE = 1
         NEW_HIST, BACK_THRES, BACK_HIST, 0
         IF N_ELEMENTS(BACK_TIMES) GT 1 THEN $
            OPLOT, BACK_TIMES, BACK_HISTORY
         !P.LINESTYLE = LINE_SAVE
         ENDIF                  ; HAVE_BACK
      ENDIF $                   ; SHOW_BACK EQ 1
   ELSE BEGIN                   ; Display background spectrum
      IF HAVE_BACK THEN BEGIN
         COMBINE, BACK_FIT, BACK_ERRS, THRESHOLDS, ENERGY_LOOKUP, $
                  BACK_DISP, BERR_DISP, BACK_THRESH
         LINE_SAVE = !P.LINESTYLE
         !P.LINESTYLE = 1
         NEW_HIST, BACK_THRESH, BACK_DISP, !P.COLOR, BERR_DISP
         !P.LINESTYLE = LINE_SAVE
         ENDIF                  ; HAVE_BACK
      ENDELSE
   ENDIF

YR = !Y.CRANGE
IF Y_SCALE EQ 'LOG' THEN YR = 10^YR
N_REGIONS = SIZE(SPAN)

IF (STRUPCASE(!D.NAME) EQ 'X')   OR $          ; X-windows
   (STRUPCASE(!D.NAME) EQ 'LINUX') OR $        ; Linux
   (STRUPCASE(!D.NAME) EQ 'WIN') OR $          ; Microsoft Windoze
   (STRUPCASE(!D.NAME) EQ 'WIN32') OR $        ; MS Windoze, 32 bit app
   (STRUPCASE(!D.NAME) EQ 'MAC') THEN BEGIN    ; Macintosh
   PLOT_CURSOR = 1 
ENDIF ELSE BEGIN
   IF CV_PLOT_CURSOR THEN $
      PLOT_CURSOR = 1 $
   ELSE $
      PLOT_CURSOR = 0           ; We don't plot these on hardcopy.
ENDELSE

IF PLOT_CURSOR THEN $                     
   PLOT_EDGES, STD_COMBINED_THRESHOLDS, SPAN, Y_SCALE, 0

X_WIN = !X.WINDOW
Y_WIN = !Y.WINDOW

; If ANNOTATE_SIZE is set to 0, then don't plot the labels
IF (ANNOTATE_SIZE NE 0) THEN $
 XYOUTS, X_WIN(1) -  0.02*ANNOTATE_SIZE, Y_WIN(1) - 0.04*ANNOTATE_SIZE, TITLE, $
         ALIGN = 1, /NORM, SIZE = ANNOTATE_SIZE

IF (N_ELEMENTS(ANNOTATE_BH) NE 0) AND (DTYPE EQ 'HISTORY') THEN $
 IF (ANNOTATE_SIZE NE 0) THEN $
  XYOUTS, X_WIN(1) -  0.02*ANNOTATE_SIZE, Y_WIN(1) - 0.08*ANNOTATE_SIZE, $
          ANNOTATE_BH, $
          ALIGN = 1, /NORM, SIZE = ANNOTATE_SIZE
IF (N_ELEMENTS(ANNOTATE_BS) NE 0) AND (DTYPE EQ 'SPECTRUM') THEN $
 IF (ANNOTATE_SIZE NE 0) THEN $
  XYOUTS, X_WIN(1) -  0.02*ANNOTATE_SIZE, Y_WIN(1) - 0.08*ANNOTATE_SIZE, $
          ANNOTATE_BS, $
          ALIGN = 1, /NORM, SIZE = ANNOTATE_SIZE



END;  PRO STD_DISPLAY
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO BACK_FIT, BACK_CHISQ, GET_ORDER = GET_ORDER, CANCEL_BACK
; Procedure which calculates a background model given a background selection
;which is described by BACK_SPAN. It is an array with a column for every
;region of time bins in the selection. Usually, one region before and one after
;the burst, so that the resulting model is interpolated, rather than, extra-
;polated at the times of the burst.

;OUTPUTS (in COMMON BACK_C):
;   BACK_FIT: array containing the fit background model (energy, time).
;   B_TOT_TIME: scalar livetime of total background history used in fit.

; CANCEL_BACK is a flag that is set if the user decides to abort
;  the background calculation.  

COMMON RATE_C, SPECTRA

COMMON ERROR_C, SPECTRA_ERRORS

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON TWIDS, WIDS

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT


; Assume no CANCEL to start
CANCEL_BACK = 0

; Standard method for converting selections to indices. In this case, the
;selected regions are contained in BACK_SPAN, which is different than the
;other _SPAN arrays, so we can't use COUNT_SPAN procedure.
N_SPEC = SIZE(SPECTRA)
N_CHAN = N_SPEC(1)
N_TIME = N_SPEC(2)
TR = [TRANSPOSE(TIMES(0,*)), TIMES(1, N_TIME - 1)]
BACK_ARR = INTARR(N_TIME)
BACK_ARR(INDGEN(N_TIME)) = 0
BACK_IND = INDGEN(N_TIME)
N_BACK = SIZE(BACK_SPAN)
BACK_REGN = N_BACK(1)
FOR BB = 0, BACK_REGN - 1 DO BEGIN
   I0 = SEARCH(TR, BACK_SPAN(BB, 0))
   IF I0 EQ -1 THEN I0 = 0 ELSE IF I0 EQ N_TIME THEN I0 = N_TIME - 1
   I1 = SEARCH(TR, BACK_SPAN(BB, 1))
   IF I1 EQ -1 THEN I1 = 0 ELSE IF I1 EQ N_TIME THEN I1 = N_TIME - 1
   FOR KK = I0, I1 DO BACK_ARR(KK) = 1
   ENDFOR
BACK_IND = BACK_IND(WHERE(BACK_ARR))
; Done getting BACK_IND array. We can now access elements of the TIMES array 
;in the selection by using BACK_IND.

; B_T: Background times: center of selected time intervals.
B_T = TRANSPOSE((TIMES(0, BACK_IND) + TIMES(1, BACK_IND)) / 2.)

; DB_T: Lengths of selected time intervals. TBD: Should be livetimes.
DB_T = TRANSPOSE(TIMES(1, BACK_IND) - TIMES(0, BACK_IND))

NUM_BACK_SPEC = N_ELEMENTS(BACK_IND)

IF NUM_BACK_SPEC EQ 1 THEN $          ; Constant background: last resort.
   NUM_DEG = 0 $
ELSE $                                ; Set the degree of fit of background 
   IF NUM_BACK_SPEC LT 6 THEN $       ;model to 2 less than the number of
      NUM_DEG = NUM_BACK_SPEC - 2 $   ;selected spectra. 
   ELSE $
      NUM_DEG = 4                     ; But no more than 4.
ORDER_TOP = NUM_DEG

IF BACK_REGN EQ 1 THEN BEGIN   ; To be safe, do linear model with one region.
   IF NUM_BACK_SPEC EQ 1 THEN $       ; Constant background: last resort.
      NUM_DEG = 0 $
      ; For this case alone, BACK_FIT consists of one spectrum: the model is
      ;constant over time. Same for BACK_ERRS.
   ELSE BEGIN
      IF NUM_BACK_SPEC LT 6 THEN $       
         ORDER_TOP = NUM_BACK_SPEC - 2 $   
      ELSE $
         ORDER_TOP = 4                     
      NUM_DEG = 1
      ENDELSE
   ENDIF 

IF (KEYWORD_SET(GET_ORDER)) THEN BEGIN  ; Background order not previously set
   ORDERS = [STRTRIM(STRING(INDGEN(ORDER_TOP + 1)), 2), 'CANCEL']
   NDEGREE = WIDMENU(['Order of Background Model:', ORDERS], TITLE = 0 ) - 1
   IF (NDEGREE EQ N_ELEMENTS(ORDERS)-1) THEN BEGIN
      CANCEL_BACK = 1
      RETURN
   ENDIF
ENDIF $
ELSE IF (N_ELEMENTS(NDEGREE) EQ 0) THEN BEGIN
   NDEGREE = NUM_DEG
ENDIF

MESSAGE = 'Calculating the background model.'
IF (WIDS.WORK_BASE NE 0) THEN WORK_INFORM, MESSAGE
IF (WIDS.TOP_BASE  NE 0) THEN INFORM, MESSAGE

T_BEG = TRANSPOSE(TIMES(0, BACK_IND))
T_END = TRANSPOSE(TIMES(1, BACK_IND))
T_LIVE = FLTARR(NUM_BACK_SPEC)          ; Passes livetimes to BACK_LFIT
B_Y = TRANSPOSE(SPECTRA(*, BACK_IND))   ; BACK_LFIT expects T X E array

FOR JJ = 0, NUM_BACK_SPEC - 1 DO BEGIN  ; Calculate the background livetimes:
   TEMP_SPEC = SPECTRA(*, BACK_IND(JJ))
   TEMP_ERRS = SPECTRA_ERRORS(*, BACK_IND(JJ))
   NIND = WHERE(TEMP_SPEC)                          ; Don't divide by zeros!
   IF (NIND(0) NE -1) THEN BEGIN                    ; Spectrum not all zeros.
      LIVE_T = TEMP_SPEC(NIND) / (TEMP_ERRS(NIND) * TEMP_ERRS(NIND))
      T_LIVE(JJ) = TOTAL(LIVE_T) / N_ELEMENTS(NIND) ; Find average over energy.
      ENDIF $
   ELSE BEGIN                           ; No counts: livetime = accum time.
      T_LIVE(JJ) = T_END(JJ) - T_BEG(JJ)
      ENDELSE
   ENDFOR

; B_TOT_TIME: Returned in BACK_C. Total livetime.
B_TOT_TIME = TOTAL(T_LIVE)

   ; Michael's polynomial fitting routine, which takes into account the finite
   ;width of each time bin by integrating the basis functions across the bin.
   ;Returns the model in BACK_FIT, each energy bin as a function of time.
ER = LONARR(2)
DUM_RANGE = NEAREST_EDGE(COMBINED_ENERGIES, ENERGY_SPAN(0, *), ER)
ER(0) = ENERGY_LOOKUP(ER(0))
ER(1) = ENERGY_LOOKUP(ER(1) + 1) - 1

IF WGSP_NO_MFIT THEN BEGIN
   BACK_LFIT, NDEGREE, T_BEG, T_END, B_Y, T_LIVE, ER(0), ER(1), BACK_CHISQ, $
           BACKGND_FIT
ENDIF ELSE BEGIN
   AFUNC = DBLARR(5, NUM_BACK_SPEC)
   BACKGND_FIT = FLTARR(NUM_BACK_SPEC, MAX_CHAN)
   STATUS = CALL_EXTERNAL('MFIT_SHARE', 'BACK_LFIT', $
         LONG(NDEGREE), LONG(NUM_BACK_SPEC), T_BEG, T_END, B_Y, T_LIVE, $
         ER(0), ER(1), AFUNC, BACK_CHISQ, BACKGND_FIT)
ENDELSE

HAVE_BACK = 1 ; Now have background model; can display it.

IF WIDS.WORK_BASE NE 0 THEN WORK_INFORM, ''
IF WIDS.TOP_BASE  NE 0 THEN INFORM, ''


END;  PRO BACK_FIT
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO GET_BACK, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
              X_RANGE, Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, DTYPE

; Routine whose sole purpose is to create the BACK_SPAN array. The main loop
;is repeated while the user changes the spectra to be accumulated into the 
;background model calculation. The arguments are all INPUTs, and are to be
;passed unaltered to the STD_DISPLAY routine for updating of the 'BURST
;HISTORY' plot. 
; The logical DIRTY is set to TRUE if the background model has
;been changed (or created), so that the user can later be informed that a new
;lookup file should be saved before fitting spectra. This serves as a minimal
;audit trail, since the user may always choose not to save the model.

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON LAYOUT_C, BOXES, PLOT_TITLES, HARD_MENU, PLOT_LABEL

COMMON PARAM_C, MAX_CHAN, MAX_EBINS, MAX_PLOT, MAX_TERMS, MAX_CHI_1D, $
          MAX_CHI_2D, MAX_PLOT_TYPE, MAX_PARAM, MAX_DET

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON TWIDS, WIDS

COMMON WGSP_GET_RANGE, WLO_X, WHI_X

THE_WIN = !D.WINDOW
OLD_SPAN = FLTARR(1,2)
IF HAVE_BACK THEN BEGIN       ; Plot the old background edges first
   FAKE_SPAN = [FLTARR(1, 2), BACK_SPAN]
   PLOT_EDGES, COMBINED_THRESHOLDS, FAKE_SPAN, Y_SCALE, 1
   OLD_SPAN = FAKE_SPAN
   ENDIF

WHILE 1 DO BEGIN
         BAIL_OUT = TRACK(LO_X, LO_Y, HI_X, HI_Y, X_SCALE, Y_SCALE)
         IF BAIL_OUT EQ 1 THEN GOTO, DONE
         IF BAIL_OUT EQ 2 THEN BEGIN 
            BACK_SPAN = FLTARR(1,2)
            HAVE_BACK = 0 ; Background was reset to zero: nothing to display.
            IF N_ELEMENTS(BACK_HISTORY) NE 0 THEN $
               BACK_HISTORY(*) = -100
            IF TOTAL(OLD_SPAN) NE 0.0 THEN $
               PLOT_EDGES, COMBINED_THRESHOLDS, OLD_SPAN, Y_SCALE, 1, /ERASE
            GOTO, LOOPBACK
            ENDIF
         IF BAIL_OUT EQ 3 THEN BEGIN 
            TWO_NUM = 0
            WHILE NOT TWO_NUM DO BEGIN
               SELECT_EDGE, !X.CRANGE, DTYPE
               LO_X = WLO_X & HI_X = WHI_X
               SIZE_LO = SIZE(LO_X)
               SIZE_HI = SIZE(HI_X)
               ONE_NUM = 0
               IF SIZE_LO(0) EQ 0 THEN BEGIN
                  IF (SIZE_LO(1) GT 1) AND (SIZE_LO(1) LT 6) THEN $
                     ONE_NUM = 1
                  ENDIF
               IF SIZE_HI(0) EQ 0 THEN BEGIN
                  IF (SIZE_HI(1) GT 1) AND (SIZE_HI(1) LT 6) THEN $
                     TWO_NUM = 1
                  ENDIF
               IF (ONE_NUM EQ 1) AND (TWO_NUM EQ 1) THEN $
                  TWO_NUM = 1 $
               ELSE $
                  TWO_NUM = 0
               ENDWHILE
            ENDIF

         HAVE_BACK = 1 ; We need to display the dotted lines on the plot.
         X_SPAN = [LO_X, HI_X]
         X_SPAN = X_SPAN(SORT(X_SPAN))
         X_EDGE = NEAREST_EDGE(COMBINED_THRESHOLDS, X_SPAN)
         IF N_ELEMENTS(BACK_SPAN) EQ 0 THEN $
            BACK_SPAN = TRANSPOSE(X_SPAN) $
         ELSE IF (BACK_SPAN(0,0) EQ 0.0) AND (BACK_SPAN(0,1) EQ 0.0) THEN $
            BACK_SPAN = TRANSPOSE(X_SPAN) $
         ELSE BEGIN
            N_TEMP = SIZE(BACK_SPAN)
            N_REGIONS = N_TEMP(1)
            TEMP_SPAN = BACK_SPAN
            IND_ARR = INTARR(N_REGIONS)
            IND_ARR(INDGEN(N_REGIONS)) = 1
            FOR JJ = 0, N_REGIONS - 1 DO BEGIN
               IF ((X_EDGE(0)  LE BACK_SPAN(JJ, 0)) AND $
                   (X_EDGE(1) GE BACK_SPAN(JJ, 1))) $
                  THEN BEGIN
                  TEMP_SPAN(JJ, *) = TRANSPOSE(X_SPAN)
                  IND_ARR(JJ) = 0
                  ENDIF
               ENDFOR
            FOR JJ = 0, N_REGIONS - 1 DO BEGIN
               IF (((X_EDGE(0) GT BACK_SPAN(JJ, 0)) AND $
                    (X_EDGE(0) LT BACK_SPAN(JJ, 1))) $
               OR ((X_EDGE(1) GT BACK_SPAN(JJ, 0)) AND $
                   (X_EDGE(1) LT BACK_SPAN(JJ, 1)))) $
                  THEN BEGIN
                  TEMP_SPAN(JJ, *) = TRANSPOSE(X_SPAN)
                  IND_ARR(JJ) = 0
                  ENDIF
               ENDFOR
            TEMP_SPAN = [TEMP_SPAN, TRANSPOSE(X_SPAN)]
            IND_ARR = [IND_ARR, 1]
            TEMP_IND = WHERE(IND_ARR > 0, THE_COUNT)
            IF THE_COUNT GT 0 THEN TEMP_SPAN = TEMP_SPAN(TEMP_IND, *)
            BACK_SPAN = TEMP_SPAN(SORT(TEMP_SPAN(*, 0)), *)
            ENDELSE
LOOPBACK:
       IF HAVE_BACK NE 0 THEN BEGIN
          IF STRUPCASE(DTYPE) EQ 'SPECTRUM' THEN MUNITS = ' keV'
          IF STRUPCASE(DTYPE) EQ 'HISTORY' THEN MUNITS = ' seconds'

          MESSAGE = 'Selection Interval:  [' + STRTRIM(STRING(X_EDGE(0)),2) + $
                    ', ' + STRTRIM(STRING(X_EDGE(1)),2) + ']' + MUNITS
          WORK_INFORM, MESSAGE

          MESSAGE = ''
          IF TOTAL(OLD_SPAN) NE 0.0 THEN $
             PLOT_EDGES, COMBINED_THRESHOLDS, OLD_SPAN, Y_SCALE, 1, /ERASE
          FAKE_SPAN = [FLTARR(1, 2), BACK_SPAN]
          PLOT_EDGES, COMBINED_THRESHOLDS, FAKE_SPAN, Y_SCALE, 1
          OLD_SPAN = FAKE_SPAN
          ENDIF
       ENDWHILE

DONE:
   WORK_INFORM, ''
   IF HAVE_BACK THEN $
      STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                   Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, 1, DTYPE $
   ELSE $
      STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                   Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, 0, DTYPE


   IF (BACK_SPAN(0,0) NE 0) AND (BACK_SPAN(0,1) NE 0) THEN BEGIN
      ; We now have a background selection, so perform the fit or CANCEL:
      BACK_CHISQ = FLTARR(MAX_CHAN)
      BACK_FIT, BACK_CHISQ, /GET_ORDER, CANCEL_BACK
      IF (CANCEL_BACK NE 0) THEN BEGIN
         WW_UPDATE, 'HISTORY'
         RETURN
      ENDIF

      WSET, THE_WIN

      ; Integrate the model over energy, so that it can be displayed:
      INTEGRATE_BACKGROUND

      STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                      Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, 1, DTYPE

      SHOW_BACK = 0
      IF NDEGREE GT 0 THEN $
         SHOW_BACK = WIDMENU(['Display Reduced Chi-squares?', 'Yes', 'No'], $
                          TITLE = 0)

      IF SHOW_BACK EQ 1 THEN BEGIN
         X_SAVE = !X
         Y_SAVE = !Y

         ER = INTARR(2)
         TR = NEAREST_EDGE(COMBINED_ENERGIES, ENERGY_SPAN(0, *), ER)
         ER(0) = ENERGY_LOOKUP(ER(0))
         ER(1) = ENERGY_LOOKUP(ER(1) + 1) - 1
         CHI2_RANGE = [MIN(BACK_CHISQ(ER(0):ER(1))), MAX(BACK_CHISQ(ER(0):ER(1)))]
         E_RANGE = [TR(0), TR(1)]

         PLOTTING = 0
         DO_PLOT:
         IF PLOTTING THEN BEGIN
            OPEN_PLOT, CURRENT_DEVICE
            X0 = BOXES(0)/510.0
            Y0 = BOXES(1)/660.0
            X1 = BOXES(2)/510.0
            Y1 = BOXES(3)/660.0
            P_SAVE = !P.POSITION
            !P.POSITION = [X0, Y0, X1, Y1]
            ENDIF 

         IF NOT PLOTTING THEN BEGIN
            WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
            W_HARDCOPY, 'Background Chi-squares'
         ENDIF
         PLOT_OI, E_RANGE, CHI2_RANGE, XTITLE = 'Energy (keV)', $
            YTITLE = 'Reduced !7v!X!u2!n', /NODATA, /YNOZERO
         COUNT_SPAN, THRESHOLDS, ENERGY_SPAN, EBACK_IND
         NEW_HIST, THRESHOLDS(*, EBACK_IND), BACK_CHISQ(EBACK_IND)
;         NEW_HIST, THRESHOLDS(*, ER(0):ER(1)), BACK_CHISQ(ER(0):ER(1))

         GET_HC_EVENT, HC_OPTION
         IF NOT PLOTTING THEN BEGIN
            IF HC_OPTION EQ 1 THEN BEGIN
               WORK_INFORM, 'Creating hardcopy.'
               PLOTTING = 1
               GOTO, DO_PLOT
            ENDIF 
         ENDIF $
         ELSE BEGIN
            IF NOT CV_SUPPRESS_2ND_LABELS THEN $
                   PLOT_NOTES, !P.POSITION, 'SPECTRAL HISTORY'
            CLOSE_PLOT, CURRENT_DEVICE
            !P.POSITION = P_SAVE 
         ENDELSE

         WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 1
         WSET, THE_WIN            ; Must reselect the current window
         !X = X_SAVE 
         !Y = Y_SAVE 
         ENDIF                    ; Display background reduced chi-squares

      ; Background selection is 'DIRTY': it hasn't been saved. Set the flag:
      DIRTY = 1
      ENDIF
END;  PRO GET_BACK
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO PLOT_EDGES, COMB_THRESH, ASPAN, Y_SCAL, LINE_STY, ERASE = ER

COLOR_SAVE = !P.COLOR
YR = !Y.CRANGE
XR = !X.CRANGE
IF KEYWORD_SET(ER) THEN E_COLOR = !P.BACKGROUND $
   ELSE E_COLOR = !P.COLOR
IF Y_SCAL EQ 'LOG' THEN YR = 10^YR
N_REGIONS = SIZE(ASPAN)
IF (N_REGIONS(1) - 1) GT 0 THEN BEGIN
   FOR JJ = 1, N_REGIONS(1) - 1 DO BEGIN
   EDGE = NEAREST_EDGE(COMB_THRESH, ASPAN(JJ, *))
   IF EDGE(0) NE XR(0) THEN $
      OPLOT, [EDGE(0), EDGE(0)], YR, COLOR = E_COLOR, LINESTYLE = LINE_STY
   IF EDGE(1) NE XR(1) THEN $
      OPLOT, [EDGE(1), EDGE(1)], YR, COLOR = E_COLOR, LINESTYLE = LINE_STY
   ENDFOR
   ENDIF $
ELSE BEGIN 
   EDGE = NEAREST_EDGE(COMB_THRESH, ASPAN(0, *))
   IF EDGE(0) NE XR(0) THEN $
      OPLOT, [EDGE(0), EDGE(0)], YR, COLOR = E_COLOR, LINESTYLE = LINE_STY
   IF EDGE(1) NE XR(1) THEN $
      OPLOT, [EDGE(1), EDGE(1)], YR, COLOR = E_COLOR, LINESTYLE = LINE_STY
   ENDELSE

!P.COLOR = COLOR_SAVE 

END; PRO PLOT_EDGES
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
FUNCTION CALC_SNR, SELECTOR, OBS_RATE, OBS_ERR, D_IND, EDGES, SIGNAL, NOISE

COMMON TIME_C, TIMES, TIME_LOOKUP, COMBINED_TIMES, TIME_RANGE, TIME_SPAN, $
               DEFAULT_TL, TRIGGER

COMMON ENERGY_C, THRESHOLDS, ENERGY_LOOKUP, COMBINED_ENERGIES, ENERGY_RANGE, $
                 ENERGY_SPAN, DEFAULT_EL, ENERGIES

COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

IF SELECTOR NE -1 THEN BEGIN ; Sum SNR over energy bins
   SIGNAL = SIGNAL + OBS_RATE(D_IND) - BACK_FIT(D_IND) / EDGES(D_IND)
   SIGMA_BACK = BACK_ERRS(D_IND) / EDGES(D_IND)
   SIGMA_OBS = OBS_ERR(D_IND) 
   NOISE = SQRT(NOISE * NOISE + SIGMA_OBS * SIGMA_OBS + SIGMA_BACK * SIGMA_BACK)
   SCALE = TIME_SPAN(0,1) - TIME_SPAN(0,0)  ; Average SNR for 1 sec.
   ENDIF $
ELSE BEGIN                   ; Sum SNR over time bins
   SIGNAL = SIGNAL + OBS_RATE(D_IND) - SUM_BACK(D_IND) 
   SIGMA_OBS = OBS_ERR(D_IND) 
   SIGMA_BACK = SUM_ERRS(D_IND) 
   NOISE = SQRT(NOISE * NOISE + SIGMA_OBS * SIGMA_OBS + SIGMA_BACK * SIGMA_BACK)
   ; Scale by the average energy bin width over the selected range
   SCALE = (ENERGY_SPAN(0,1) - ENERGY_SPAN(0,0)) / N_ELEMENTS(BACK_FIT)
   ENDELSE

RETURN, SIGNAL / NOISE ;/ SCALE

END; FUNCTION CALC_SNR
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO CONSTRUCT_LOOKUP, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                      SPECTRA, ERRORS, THRESHOLDS, LOOKUP, DEFAULT_LOOKUP, $
                      X_RANGE, Y_RANGE, SPAN, TITLE, X_SCALE, Y_SCALE, $
                      SPAN_CHANGED, WHICH_BACK, $
                      BUTTON_COMMAND, DTYPE, COMBINE_OPTION


COMMON BACK_C, BACK_FIT, BACK_ERRS, BACK_SPAN, B_TOT_TIME, BACK_TIMES, $
               BACK_HISTORY, DIRTY, NDEGREE, HAVE_BACK, SUM_BACK, SUM_ERRS

COMMON MENUS_C, PLOT_FIELDS, PLOT_MENU, PLOT_OPTIONS, PLOT_TYPES, ROTATION

COMMON LAYOUT_C, BOXES, PLOT_TITLES, HARD_MENU, PLOT_LABEL

COMMON IN_ARGS_C, TOPLABEL, NUM_DET, DET_NAMES, $
          NEW_DRM, NEW_DATA, DATA_FILE_NAME, COMMENT_DATA, $
          CHAN_OFFSET, RES_FRAC_AT511, RES_EXP, $
          FIRST_CHAN_PLOT, LAST_CHAN_PLOT, CALLING_PROG, CTOE_LABEL, $
          USED_REBIN, REBIN_CHAN, CDET

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON CURRENT_MENU, CURRENT_VALUES, ANNOTATE_BH, ANNOTATE_BS, $
                     AUTO_READ, JOINT_FIT, ANNOTATE_SIZE, $
                     GLOBAL_CHARSIZE, GLOBAL_TICKLEN, $
                     GLOBAL_LINETHICK, GLOBAL_CHARTHICK

COMMON TWIDS, WIDS

COMMON WGSP_GET_RANGE, WLO_X, WHI_X


CHOICE = STRUPCASE(BUTTON_COMMAND)
CASE 1 OF
   CHOICE EQ 'SELECT BACKGROUND INTERVALS': BEGIN
     CHOICE = 'SELECT'
     BACK_OPTION = 0 
     END

   CHOICE EQ 'SELECT SOURCE INTERVALS': BEGIN
     CHOICE = 'SELECT'
     BACK_OPTION = 1 
     END

   CHOICE EQ 'COMBINE INTO 1 BIN': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = '1 BIN'
     END

   CHOICE EQ 'COMBINE SELECTIONS': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = 'COMB_SELECT'
     END
 
   CHOICE EQ 'GROUP BY TWOS': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = 'BY TWOS'
     END

   CHOICE EQ 'GROUP BY...': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = 'BY...'
     END

   CHOICE EQ 'HALF ENERGY RESOLUTION': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = 'HALF'
     END

   CHOICE EQ 'FULL ENERGY RESOLUTION': BEGIN
     CHOICE = 'COMBINE'
     DISCRIMINATOR = 'FULL'
     END

   ; Else do nothing
   ELSE:
ENDCASE

CASE CHOICE OF 

  'EXIT': RETURN ; not used

  'CREATE HARDCOPY': BEGIN

     ; Change the log scale tick marks, if necessary
     IF (!X.TYPE EQ 1) THEN BEGIN                 
        IF (CHECK_DECADE(10.0^!X.CRANGE)) THEN !X.TICKS = 5
     ENDIF
     IF (!Y.TYPE EQ 1) THEN BEGIN                  
        IF (CHECK_DECADE(10.0^!Y.CRANGE)) THEN !Y.TICKS = 5
     ENDIF

     OPEN_PLOT, CURRENT_DEVICE
     X0 = 55 / 510.0  & X0 = X0 + (0.05 * (GLOBAL_CHARSIZE-1))
     Y0 = 185 / 660.0 & Y0 = Y0 + (0.05 * (GLOBAL_CHARSIZE-1))
     X1 = 439 / 510.0 & X1 = X1 - (0.05 * (GLOBAL_CHARSIZE-1))
     Y1 = 567 / 660.0 & Y1 = Y1 - (0.05 * (GLOBAL_CHARSIZE-1))
     P_SAVE = !P.POSITION
     T_SAVE = !P.TITLE
     !P.POSITION = [X0, Y0, X1, Y1]
     T_LEN = STRLEN(TITLE) - 1
     !P.TITLE = READ_MENU(8)

     ; STD_DISPLAY rounds if the keywords XRANGE, YRANGE are not supplied
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     IF NOT CV_SUPPRESS_2ND_LABELS THEN $
            PLOT_NOTES, [0.1, 0.28, 0.86, 0.86], 'SPECTRAL HISTORY'
     CLOSE_PLOT, CURRENT_DEVICE

     !P.POSITION = P_SAVE
     !P.TITLE = T_SAVE

     !X.TICKS = 0 & !Y.TICKS = 0
     END

  'DEFAULT LOOKUPS': BEGIN
     LOOKUP = DEFAULT_LOOKUP
     COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, COMBINED_ERRS, $
              COMBINED_THRESHOLDS
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     END
  'FULL RESOLUTION': BEGIN
     N = SIZE(THRESHOLDS)
     LOOKUP = INDGEN(N(2) + 1)
     COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, COMBINED_ERRS, $
              COMBINED_THRESHOLDS
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     END
  'COMBINE': BEGIN
     CASE DISCRIMINATOR OF
      '1 BIN'       : COMB_OPT = 0
      'BY TWOS'     : COMB_OPT = 1
      'BY...'       : COMB_OPT = 2
      'HALF'        : COMB_OPT = 3
      'FULL'        : COMB_OPT = 4
      'COMB_SELECT' : COMB_OPT = 5
     ENDCASE

     NUM_BINS = 2
     IF (COMB_OPT EQ 2) THEN BEGIN
         ; COMBINE_OPTION is passed in if GROUP BY... option is selected
         NUM_BINS = COMBINE_OPTION
     ENDIF

     IF (COMB_OPT EQ 5) THEN BEGIN
         REGIONS = SIZE(SPAN)
         IF (REGIONS(1) - 1) GT 0 THEN BEGIN
            LOWER = 1
            UPPER = REGIONS(1) - 1
         ENDIF ELSE BEGIN
            LOWER = 0
            UPPER = 0
            ENDELSE
         AR = INTARR(2)
         FOR HH = UPPER, LOWER, -1 DO BEGIN
            ER = NEAREST_EDGE(COMBINED_THRESHOLDS, SPAN(HH, *), AR)
            LOOKUP = [LOOKUP(0:AR(0)), LOOKUP(AR(1) + 1: *)]
            N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
            ENDFOR

         COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                    COMBINED_ERRS, COMBINED_THRESHOLDS
         ; Jump over WHILE loop if COMBINE SELECTION option is selected
         GOTO, COMB_DONE
     ENDIF

     ; Must reset the plot ranges, so the cursor won't get lost!
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE

     N_CHANNELS = N_ELEMENTS(LOOKUP) - 1

     C = INTARR(2)

     WHILE 1 DO BEGIN
           BAIL_OUT = TRACK(LO_X, LO_Y, HI_X, HI_Y, X_SCALE, Y_SCALE)
           IF BAIL_OUT EQ 1 THEN BEGIN
              SPAN_CHANGED = 1            ; Just to be safe...
              GOTO, DONE
           ENDIF
           IF BAIL_OUT EQ 2 THEN BEGIN    ; Reset combined bins
              LOOKUP = DEFAULT_LOOKUP
              N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
              COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                       COMBINED_ERRS, COMBINED_THRESHOLDS
              STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                           X_RANGE, Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, $
                           Y_SCALE, WHICH_BACK, DTYPE
              GOTO, INVALID
           ENDIF

           IF BAIL_OUT EQ 3 THEN BEGIN     ; Numerical Entry
              TWO_NUM = 0
              WHILE NOT TWO_NUM DO BEGIN

                    SELECT_EDGE, !X.CRANGE, DTYPE
                    LO_X = WLO_X & HI_X = WHI_X

                    SIZE_LO = SIZE(LO_X)
                    SIZE_HI = SIZE(HI_X)
                    ONE_NUM = 0
                    IF SIZE_LO(0) EQ 0 THEN BEGIN
                       IF (SIZE_LO(1) GT 1) AND (SIZE_LO(1) LT 6) THEN $
                          ONE_NUM = 1
                    ENDIF
                    IF SIZE_HI(0) EQ 0 THEN BEGIN
                       IF (SIZE_HI(1) GT 1) AND (SIZE_HI(1) LT 6) THEN $
                          TWO_NUM = 1
                    ENDIF
                    IF (ONE_NUM EQ 1) AND (TWO_NUM EQ 1) THEN $
                       TWO_NUM = 1 $
                    ELSE $
                       TWO_NUM = 0
              ENDWHILE
           ENDIF

           T = THRESHOLDS(0, LOOKUP(0: N_CHANNELS - 1))

           C(0) = SEARCH(T, LO_X)
           C(1) = SEARCH(T, HI_X)

           IF C(0) EQ C(1) THEN GOTO, INVALID
           C = C(SORT(C))
           IF C(0) EQ -1 THEN C(0) = 0

           IF COMB_OPT EQ 0 THEN BEGIN
              IF (C(0) EQ 0) AND (C(1) EQ (N_CHANNELS - 1)) THEN GOTO, INVALID
              LOOKUP = [LOOKUP(0:C(0)), LOOKUP(C(1) + 1: *)]
              N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
           ENDIF $
           ELSE IF COMB_OPT LT 3 THEN BEGIN
             LOOP_TOP = (C(1) - C(0) + 1) / NUM_BINS
             FOR LOOP = 0, (LOOP_TOP - 1) DO BEGIN
                 LOOKUP = [LOOKUP(0:C(0) + LOOP), $
                          LOOKUP(C(0) + LOOP + NUM_BINS: *)]
                 N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
             ENDFOR
           ENDIF $
           ELSE BEGIN
             N = SIZE(THRESHOLDS)
             LOOKUP = INDGEN(N(2) + 1)             ; Start clean
             N_CHANNELS = N(2) - 1
             IF COMB_OPT EQ 3 THEN FACTOR = 0.5 $  ; Half resolution binning
             ELSE FACTOR = 1.0                     ; Full resolution binning

             T = THRESHOLDS(0, LOOKUP(0: N_CHANNELS - 1))
             DEL_E = THRESHOLDS(1, *) - THRESHOLDS(0, *)
             AVE_E = (THRESHOLDS(1, *) + THRESHOLDS(0, *)) / 2.0

             C(0) = SEARCH(T, LO_X)
             C(1) = SEARCH(T, HI_X)
             C = C(SORT(C))
             IF C(0) EQ -1 THEN C(0) = 0
             TOP = C(1)
             BOTTOM = C(0)
             INDEX = 0
             DONE_BINNING = 0
             WHILE NOT DONE_BINNING DO BEGIN
                   WIDTH = DEL_E(TOP - INDEX)
                   ENRG = AVE_E(TOP - INDEX) 
                   RES_WIDTH = FACTOR * ENRG * RES_FRAC_AT511(CDET) * $
                               (ENRG / 511.0) ^ RES_EXP(CDET)
                   WHILE WIDTH LT (0.8 * RES_WIDTH) DO BEGIN ;Main binning proc.
                        INDEX = INDEX + 1
                        IF (TOP - INDEX) LE BOTTOM THEN BEGIN
                           DONE_BINNING = 1 
                           INDEX = 0
                           WIDTH = 1.0E16
                        ENDIF ELSE $
                           WIDTH = WIDTH + DEL_E(TOP - INDEX)
                  ENDWHILE
                  IF (WIDTH LT (RES_WIDTH)) AND (NOT DONE_BINNING) THEN BEGIN 
                     INDEX = INDEX + 1                      ; Add just one more
                     IF (TOP - INDEX) LE BOTTOM THEN BEGIN
                        DONE_BINNING = 1 
                        INDEX = 0
                     ENDIF ELSE BEGIN
                        WIDTH = WIDTH + DEL_E(TOP - INDEX)  
                        IF WIDTH GT (1.2 * RES_WIDTH) THEN BEGIN 
                           INDEX = INDEX - 1               ; Go back if too big
                        ENDIF
                     ENDELSE
                  ENDIF
                  IF INDEX GT 0 THEN BEGIN
                     IF BOTTOM GT (TOP - INDEX) THEN  BEGIN   ; Bin left-overs
                        INDEX = TOP - BOTTOM
                        DONE_BINNING = 1
                     ENDIF
                     LOOKUP = [LOOKUP(0:TOP - INDEX), $
                              LOOKUP(TOP: *)]
                     N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
                     TOP = TOP - INDEX
                     INDEX = 0
                  ENDIF ELSE BEGIN      ; If INDEX = 0, move on to avoid looping
                     TOP = TOP - 1
                     INDEX = 0
                  ENDELSE
             ENDWHILE
           ENDELSE

           COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                    COMBINED_ERRS, COMBINED_THRESHOLDS

           STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                        Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                        WHICH_BACK, DTYPE

           IF COMB_OPT GT 2 THEN GOTO, DONE
INVALID:
     ENDWHILE

COMB_DONE:
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE

     END
  'ZOOM OUT: FULL RANGE': BEGIN
     N = SIZE(COMBINED_THRESHOLDS)
     X_RANGE = [COMBINED_THRESHOLDS(0, 0), COMBINED_THRESHOLDS(1, N(2) - 1)]
     NIND = WHERE(SPECTRUM, COUNT)
     IF COUNT EQ 0 THEN BEGIN
        Y_RANGE = [MIN(SPECTRUM - COMBINED_ERRS), MAX(SPECTRUM + COMBINED_ERRS)]
     ENDIF ELSE BEGIN
        Y_RANGE = [MIN(SPECTRUM(NIND) - COMBINED_ERRS(NIND)), $
                   MAX(SPECTRUM + COMBINED_ERRS)]
        ENDELSE

     ROUND_AXES, X_RANGE, X_SCALE EQ 'LOG', /POWER
     ROUND_AXES, Y_RANGE, Y_SCALE EQ 'LOG', /POWER
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     END
  'ZOOM OUT: BY TWOS': BEGIN
     N = SIZE(COMBINED_THRESHOLDS)
     MAX_X_RANGE = [COMBINED_THRESHOLDS(0, 0), $
                   COMBINED_THRESHOLDS(1, N(2) - 1)]
     NIND = WHERE(SPECTRUM, COUNT)
     IF COUNT EQ 0 THEN BEGIN
        MAX_Y_RANGE = [MIN(SPECTRUM - COMBINED_ERRS), $
                       MAX(SPECTRUM + COMBINED_ERRS)]
     ENDIF ELSE BEGIN
        MAX_Y_RANGE = [MIN(SPECTRUM(NIND) - COMBINED_ERRS(NIND)), $
                       MAX(SPECTRUM + COMBINED_ERRS)]
        ENDELSE
     DEL_X = X_RANGE(1) - X_RANGE(0)
     IF (X_SCALE EQ 'LOG') THEN BEGIN
        X_RANGE(1) = X_RANGE(1) * 3.
        X_RANGE(0) = X_RANGE(0) / 3.
     ENDIF ELSE BEGIN
        X_RANGE(1) = X_RANGE(1) + DEL_X / 2.
        X_RANGE(0) = X_RANGE(0) - DEL_X / 2.
     ENDELSE
     IF (X_RANGE(1) GT MAX_X_RANGE(1)) THEN X_RANGE(1) = MAX_X_RANGE(1)
     IF (X_RANGE(0) LT MAX_X_RANGE(0)) THEN X_RANGE(0) = MAX_X_RANGE(0)

     DEL_Y = Y_RANGE(1) - Y_RANGE(0)
     IF (Y_SCALE EQ 'LOG') THEN BEGIN
        Y_RANGE(1) = Y_RANGE(1) * 3.
        Y_RANGE(0) = Y_RANGE(0) / 3.
     ENDIF ELSE BEGIN
        Y_RANGE(1) = Y_RANGE(1) + DEL_Y / 2.
        Y_RANGE(0) = Y_RANGE(0) - DEL_Y / 2.
     ENDELSE
     IF (Y_RANGE(1) GT MAX_Y_RANGE(1)) THEN Y_RANGE(1) = MAX_Y_RANGE(1)
     IF (Y_RANGE(0) LT MAX_Y_RANGE(0)) THEN Y_RANGE(0) = MAX_Y_RANGE(0)

     ROUND_AXES, X_RANGE, X_SCALE EQ 'LOG', /POWER
     ROUND_AXES, Y_RANGE, Y_SCALE EQ 'LOG', /POWER
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     END
  'ZOOM': BEGIN
     ; Change the log scale tick marks, if necessary
     IF (!X.TYPE EQ 1) THEN BEGIN                 
        IF (CHECK_DECADE(10.0^!X.CRANGE)) THEN !X.TICKS = 5
     ENDIF
     IF (!Y.TYPE EQ 1) THEN BEGIN                  
        IF (CHECK_DECADE(10.0^!Y.CRANGE)) THEN !Y.TICKS = 5
     ENDIF

     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     WHILE 1 DO BEGIN
Z_AGAIN:
           LAST_XCRANGE = !X.CRANGE
           LAST_YCRANGE = !Y.CRANGE

           ZOOM, XR, YR, BAIL_OUT
           LO_X = XR(0) & HI_X = XR(1)
           LO_Y = YR(0) & HI_Y = YR(1)

           ; Change the log scale tick marks, if necessary
           IF (!X.TYPE EQ 1) THEN BEGIN                 
              IF (CHECK_DECADE(XR)) THEN !X.TICKS = 5
           ENDIF
           IF (!Y.TYPE EQ 1) THEN BEGIN                 
              IF (CHECK_DECADE(YR)) THEN !Y.TICKS = 5
           ENDIF

           IF (BAIL_OUT EQ 1) THEN BEGIN
              ; Restore the last range, since it has just been rounded in ZOOM
              !X.CRANGE = LAST_XCRANGE
              !Y.CRANGE = LAST_YCRANGE
              !X.TICKS = 0 & !Y.TICKS = 0
              GOTO, DONE
           ENDIF

           IF (BAIL_OUT EQ 2) THEN BEGIN
              N = SIZE(COMBINED_THRESHOLDS)
              IF (!X.TYPE EQ 1) THEN BEGIN
                 TEMP_LO = COMBINED_THRESHOLDS(0, 0)
                 TEMP_LO = TEMP_LO(WHERE(TEMP_LO GT 0.0))
                 TEMP_HI = COMBINED_THRESHOLDS(1, N(2) - 1)
                 TEMP_HI = TEMP_HI(WHERE(TEMP_HI GT 0.0))
                 X_RANGE = [TEMP_LO, TEMP_HI]
              ENDIF ELSE $
                 X_RANGE = [COMBINED_THRESHOLDS(0, 0), $
                            COMBINED_THRESHOLDS(1, N(2) - 1)]
              IF (!Y.TYPE EQ 1) THEN BEGIN
                 TEMP_LO = SPECTRUM - COMBINED_ERRS
                 TEMP_LO = TEMP_LO(WHERE(TEMP_LO GT 0.0))
                 TEMP_HI = SPECTRUM + COMBINED_ERRS
                 TEMP_HI = TEMP_HI(WHERE(TEMP_HI GT 0.0))
                 Y_RANGE = [MIN(TEMP_LO), MAX(TEMP_HI)]
              ENDIF ELSE $
                 Y_RANGE = [MIN(SPECTRUM - COMBINED_ERRS), $
                            MAX(SPECTRUM + COMBINED_ERRS)]
              !X.TICKS = 0 & !Y.TICKS = 0
              STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                           X_RANGE, Y_RANGE, SPAN, PLOT_LABEL, $
                           X_SCALE, Y_SCALE, WHICH_BACK, DTYPE
           ENDIF ELSE BEGIN
              ; Handle accidental double clicks 
              Z_ERR = 1.0E-10
              IF (ABS( HI_X - LO_X ) LE Z_ERR) OR $
                 (ABS( HI_Y - LO_Y ) LE Z_ERR) THEN BEGIN
                 WORK_INFORM, 'ZOOM Error.  Resetting.'
                 N = SIZE(COMBINED_THRESHOLDS)
                 X_RANGE = [COMBINED_THRESHOLDS(0, 0), $
                            COMBINED_THRESHOLDS(1, N(2) - 1)]
                 Y_RANGE = [MIN(SPECTRUM - COMBINED_ERRS), $
                            MAX(SPECTRUM + COMBINED_ERRS)]
                 !X.TICKS = 0 & !Y.TICKS = 0
                 STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                              X_RANGE, Y_RANGE, SPAN, PLOT_LABEL, $
                              X_SCALE, Y_SCALE, WHICH_BACK, DTYPE
                 GOTO, Z_AGAIN
              ENDIF
              X_RANGE(0) = LO_X
              X_RANGE(1) = HI_X
              Y_RANGE(0) = LO_Y
              Y_RANGE(1) = HI_Y
              X_RANGE = X_RANGE(SORT(X_RANGE))
              Y_RANGE = Y_RANGE(SORT(Y_RANGE))
              STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                           X_RANGE, Y_RANGE, SPAN, PLOT_LABEL, $
                           X_SCALE, Y_SCALE, WHICH_BACK, DTYPE
           ENDELSE
     ENDWHILE
     END
  'ZOOM IN: SELECTION': BEGIN
     X_EDGE = NEAREST_EDGE(COMBINED_THRESHOLDS, SPAN(0, *))
     X_RANGE = X_EDGE
     N = SIZE(COMBINED_THRESHOLDS)
     T = [TRANSPOSE(COMBINED_THRESHOLDS(0, *)),COMBINED_THRESHOLDS(1, N(2) - 1)]
     J0 = SEARCH(T, X_RANGE(0))
     J1 = SEARCH(T, X_RANGE(1)) ;- 1
     IF J1 GT N_ELEMENTS(SPECTRUM) - 1 THEN J1 = J1 - 1
     Y_RANGE = [MIN(SPECTRUM(J0:J1) - COMBINED_ERRS(J0:J1)), $
               MAX(SPECTRUM(J0:J1) + COMBINED_ERRS(J0:J1))]

     ROUND_AXES, X_RANGE, X_SCALE EQ 'LOG', /POWER
     ROUND_AXES, Y_RANGE, Y_SCALE EQ 'LOG', /POWER
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE
     END
  'SELECT': BEGIN
     STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                  WHICH_BACK, DTYPE

     IF BACK_OPTION EQ 0 THEN BEGIN
        GET_BACK, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                  Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, DTYPE
        GOTO, DONE
     ENDIF

     OLD_SPAN = SPAN
     WHILE 1 DO BEGIN
           BAIL_OUT = TRACK(LO_X, LO_Y, HI_X, HI_Y, X_SCALE, Y_SCALE)
           IF BAIL_OUT EQ 1 THEN BEGIN
              SPAN_CHANGED = 1

              STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, $
                           X_RANGE, Y_RANGE, SPAN, PLOT_LABEL, $
                           X_SCALE, Y_SCALE, WHICH_BACK, DTYPE
              GOTO, DONE
           ENDIF
           IF BAIL_OUT EQ 2 THEN BEGIN 
              N = SIZE(COMBINED_THRESHOLDS)
              NEW_SPAN = [COMBINED_THRESHOLDS(0, 0), $
                          COMBINED_THRESHOLDS(1, N(2) - 1)]
              SPAN = TRANSPOSE(NEW_SPAN)
              X_EDGE = NEAREST_EDGE(COMBINED_THRESHOLDS, SPAN)
              GOTO, LOOPBACK
           ENDIF
           IF BAIL_OUT EQ 3 THEN BEGIN 
              TWO_NUM = 0
              WHILE NOT TWO_NUM DO BEGIN

                    SELECT_EDGE, !X.CRANGE, DTYPE
                    LO_X = WLO_X & HI_X = WHI_X

                    SIZE_LO = SIZE(LO_X)
                    SIZE_HI = SIZE(HI_X)
                    ONE_NUM = 0
                    IF SIZE_LO(0) EQ 0 THEN BEGIN
                       IF (SIZE_LO(1) GT 1) AND (SIZE_LO(1) LT 6) THEN $  
                          ONE_NUM = 1
                    ENDIF
                    IF SIZE_HI(0) EQ 0 THEN BEGIN
                       IF (SIZE_HI(1) GT 1) AND (SIZE_HI(1) LT 6) THEN $
                          TWO_NUM = 1
                    ENDIF
                    IF (ONE_NUM EQ 1) AND (TWO_NUM EQ 1) THEN $
                       TWO_NUM = 1 $
                    ELSE $
                    TWO_NUM = 0
              ENDWHILE
           ENDIF

           ; 8/16/96 RP Test selection for null content!

           X_SPAN = [LO_X, HI_X]
           X_SPAN = X_SPAN(SORT(X_SPAN))
              X_SPAN = TRANSPOSE(X_SPAN)
           TEST_SPAN = [X_SPAN, X_SPAN]
              X_SPAN = TRANSPOSE(X_SPAN)
           COUNT_SPAN, COMBINED_THRESHOLDS, TEST_SPAN, TEST_IND
           IF TEST_IND(0) EQ -1 THEN BEGIN
              X_EDGE = NEAREST_EDGE(COMBINED_THRESHOLDS, SPAN)
              GOTO, LOOPBACK
              ENDIF

           Y_SPAN = [LO_Y, HI_Y]
           Y_SPAN = Y_SPAN(SORT(Y_SPAN))
           N_TEMP = SIZE(SPAN)
           N_REGIONS = N_TEMP(1) - 1
           X_EDGE = NEAREST_EDGE(COMBINED_THRESHOLDS, X_SPAN)
           IF N_REGIONS GT 0 THEN BEGIN
              TEMP_SPAN = SPAN(1:*, *)
              IND_ARR = INTARR(N_REGIONS)
              IND_ARR(INDGEN(N_REGIONS)) = 1
              FOR JJ = 1, N_REGIONS DO BEGIN
                  IF ((X_EDGE(0) LE SPAN(JJ, 0)) AND $
                     (X_EDGE(1) GE SPAN(JJ, 1))) THEN BEGIN
                     TEMP_SPAN(JJ - 1, *) = TRANSPOSE(X_SPAN)
                     IND_ARR(JJ - 1) = 0
                  ENDIF
              ENDFOR
              FOR JJ = 1, N_REGIONS DO BEGIN
                  IF (((X_EDGE(0) GT SPAN(JJ, 0)) AND $
                     (X_EDGE(0) LT SPAN(JJ, 1))) $
                  OR ((X_EDGE(1) GT SPAN(JJ, 0)) AND $
                     (X_EDGE(1) LT SPAN(JJ, 1)))) THEN BEGIN
                     TEMP_SPAN(JJ - 1, *) = TRANSPOSE(X_SPAN)
                     IND_ARR(JJ - 1) = 0
                  ENDIF
              ENDFOR
              TEMP_SPAN = [TEMP_SPAN, TRANSPOSE(X_SPAN)]
              IND_ARR = [IND_ARR, 1]
              TEMP_IND = WHERE(IND_ARR > 0, THE_COUNT)
              IF THE_COUNT GT 0 THEN TEMP_SPAN = TEMP_SPAN(TEMP_IND, *)
              TEMP_SPAN = TEMP_SPAN(SORT(TEMP_SPAN(*, 0)), *)
              T_SPAN = TRANSPOSE([MIN(TEMP_SPAN(*, 0)), $
                                MAX(TEMP_SPAN(*, 1))])
              SPAN = [T_SPAN, TEMP_SPAN]
           ENDIF ELSE BEGIN $
              X_SPAN = TRANSPOSE(X_SPAN)
              SPAN = [X_SPAN, X_SPAN]
              X_SPAN = TRANSPOSE(X_SPAN)
           ENDELSE
LOOPBACK:
           IF STRUPCASE(DTYPE) EQ 'SPECTRUM' THEN MUNITS = ' keV'
           IF STRUPCASE(DTYPE) EQ 'HISTORY' THEN MUNITS = ' seconds'

           MESSAGE = 'Selection Interval:  [' + STRTRIM(STRING(X_EDGE(0)),2) + $
                     ', ' + STRTRIM(STRING(X_EDGE(1)),2) + ']' + MUNITS + '.'
           WORK_INFORM, MESSAGE
           MESSAGE = ''
           PLOT_EDGES, COMBINED_THRESHOLDS, OLD_SPAN, Y_SCALE, 0, /ERASE
           PLOT_EDGES, COMBINED_THRESHOLDS, SPAN, Y_SCALE, 0
           OLD_SPAN = SPAN
     ENDWHILE
  WORK_INFORM, ' ' 
  END
  'REFINE': BEGIN
     WHILE 1 DO BEGIN
           ; Plot the existing rebinned spectrum.
           STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                        Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                        WHICH_BACK, DTYPE
           YPOS = !D.Y_SIZE - 15
           XYOUTS, 10, YPOS, 'Click on bin to refine, left margin to EXIT', $
                   /DEVICE, CHARSIZE = 1.5
           X_LO = !X.CRANGE(0)
           IF X_SCALE EQ 'LOG' THEN X_LO = 10^X_LO
NEXT_I:
           CURSOR, X, Y, /DOWN, /DATA
           IF X LT X_LO THEN BEGIN
              XYOUTS, 10, YPOS, 'Click on bin to refine, left margin to EXIT', $
                   /DEVICE, CHARSIZE = 1.5, COLOR = !P.BACKGROUND
              GOTO, DONE
           ENDIF
           N = SIZE(COMBINED_THRESHOLDS)
           T = [TRANSPOSE(COMBINED_THRESHOLDS(0, *)), $
                COMBINED_THRESHOLDS(1, N(2) - 1)]
           I = SEARCH(T, X)
           IF (I EQ -1) OR (I EQ (N_ELEMENTS(T) - 1)) THEN GOTO, NEXT_I
           J0 = LOOKUP(I)
           J1 = LOOKUP(I + 1)
           N = J1 - J0 - 1
           IF N GT 0 THEN BEGIN
              LOOKUP = [LOOKUP(0:I), J0 + INDGEN(N) + 1, LOOKUP(I + 1:*)]
              COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                       COMBINED_ERRS, COMBINED_THRESHOLDS
           ENDIF
     ENDWHILE
     END
  'REFINE BY HALF': BEGIN
     WHILE 1 DO BEGIN
           ; Plot the existing rebinned spectrum.
           STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                        Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                        WHICH_BACK, DTYPE
           YPOS = !D.Y_SIZE - 15
;           XYOUTS, 10, YPOS, 'Click on bin to refine', /DEVICE, CHARSIZE=1.5
           X_LO = !X.CRANGE(0)
           IF X_SCALE EQ 'LOG' THEN X_LO = 10^X_LO
NEXT_II:
           CURSOR, X, Y, /DOWN, /DATA
           IF X LT X_LO THEN BEGIN
;              XYOUTS, 10, YPOS, 'Click on bin to refine', /DEVICE, $
;                      CHARSIZE=1.5, COLOR = !P.BACKGROUND
              GOTO, DONE
           ENDIF
           N = SIZE(COMBINED_THRESHOLDS)
           T = [TRANSPOSE(COMBINED_THRESHOLDS(0, *)), $
               COMBINED_THRESHOLDS(1, N(2) - 1)]
           II = SEARCH(T, X)
           IF (II EQ -1) OR (II EQ (N_ELEMENTS(T) - 1)) THEN GOTO, NEXT_II
           J0 = LOOKUP(II)
           J1 = LOOKUP(II + 1)
           N = J1 - J0 - 1
           IF N GT 0 THEN BEGIN
              LOOKUP = [LOOKUP(0:II), J0 + N/2 + 1, LOOKUP(II + 1:*)]
              COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                       COMBINED_ERRS, COMBINED_THRESHOLDS
           ENDIF
     ENDWHILE
     END
  'BIN FOR SIGNIFICANCE': BEGIN
     RESULT = STRPOS(TITLE, 'ENERGY')
     IF HAVE_BACK THEN BEGIN   ;Otherwise, can't background subtract.
        VALID_NUM = 0
        WHILE NOT VALID_NUM DO BEGIN
              WIDGET_CONTROL, WIDS.WORK_BASE, SHOW=0
              READ, 'Input the desired SNR per bin: ', SNR
              WIDGET_CONTROL, WIDS.WORK_BASE, SHOW=1
              SNR_SIZE = SIZE(SNR)
              IF SNR_SIZE(0) EQ 0 THEN BEGIN
                 IF (SNR_SIZE(1) GT 1) AND (SNR_SIZE(1) LT 6) THEN $
                    VALID_NUM = 1
              ENDIF
        ENDWHILE
        C = INTARR(2)
        N = SIZE(THRESHOLDS)
        LOOKUP = INDGEN(N(2) + 1)             ; Start clean
        N_CHANNELS = N(2) - 1
        COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                 COMBINED_ERRS, COMBINED_THRESHOLDS

        T = THRESHOLDS(0, LOOKUP(0: N_CHANNELS - 1))
        DEL_E = TRANSPOSE(THRESHOLDS(1, *) - THRESHOLDS(0, *))

        C(0) = SEARCH(T, SPAN(0, 0))          ; We use the total spanned range
        C(1) = SEARCH(T, SPAN(0, 1))          ;rather than each region
        C = C(SORT(C))
        IF C(0) EQ -1 THEN C(0) = 0
        TOP = C(1)
        BOTTOM = C(0)
        INDEX = 0
        DONE_BINNING = 0
        WHILE NOT DONE_BINNING DO BEGIN
              SP_IND = LOOKUP(BOTTOM + INDEX)
              SIGNAL = 0.0
              NOISE = 0.0
              BIN_SNR = CALC_SNR(RESULT, SPECTRUM, COMBINED_ERRS, SP_IND, $
                        DEL_E, SIGNAL, NOISE)
              WHILE (BIN_SNR LT (0.8 * SNR)) AND (NOT DONE_BINNING) DO BEGIN   
                    ; Main binning proc.
                    INDEX = INDEX + 1
                    SP_IND = LOOKUP(BOTTOM + INDEX)
                    IF (BOTTOM + INDEX + 1) GE TOP THEN BEGIN
                       DONE_BINNING = 1 
                    ENDIF ELSE BEGIN                         ; Add in next bin
                       BIN_SNR = CALC_SNR(RESULT, SPECTRUM, COMBINED_ERRS, $
                                          SP_IND, DEL_E, SIGNAL, NOISE)
                    ENDELSE
              ENDWHILE
              IF (BIN_SNR LT SNR) AND (NOT DONE_BINNING) THEN BEGIN 
                 INDEX = INDEX + 1                        ; Add just one more
                 SP_IND = LOOKUP(BOTTOM + INDEX)
                 IF (BOTTOM + INDEX + 1) GE TOP THEN BEGIN
                     DONE_BINNING = 1 
                 ENDIF ELSE BEGIN
                     BIN_SNR = CALC_SNR(RESULT, SPECTRUM, COMBINED_ERRS, $
                                        SP_IND, DEL_E, SIGNAL, NOISE)
                     IF BIN_SNR GT (1.2 * SNR) THEN BEGIN 
                        INDEX = INDEX - 1                  ; Go back if too big
                     ENDIF
                 ENDELSE
              ENDIF
              IF INDEX GT 0 THEN BEGIN
                 IF TOP LE (BOTTOM + INDEX + 1) THEN  BEGIN   ; Bin left-overs
                    INDEX = TOP - BOTTOM
                    DONE_BINNING = 1
                 ENDIF
                 LOOKUP = [LOOKUP(0: BOTTOM), LOOKUP(BOTTOM + INDEX + 1: *)]
                 N_CHANNELS = N_ELEMENTS(LOOKUP) - 1
                 BOTTOM = BOTTOM + 1
                 TOP = TOP - INDEX 
                 INDEX = 0
              ENDIF ELSE BEGIN         ; If INDEX = 0, move on to avoid looping
                 BOTTOM = BOTTOM + 1
                 IF BOTTOM EQ TOP THEN $
                    DONE_BINNING = 1  ; Make sure we have an exit
                 INDEX = 0
              ENDELSE
        ENDWHILE

        COMBINE, SPECTRA, ERRORS, THRESHOLDS, LOOKUP, SPECTRUM, $
                 COMBINED_ERRS, COMBINED_THRESHOLDS

        STD_DISPLAY, SPECTRUM, COMBINED_ERRS, COMBINED_THRESHOLDS, X_RANGE, $
                     Y_RANGE, SPAN, PLOT_LABEL, X_SCALE, Y_SCALE, $
                     WHICH_BACK, DTYPE
     ENDIF $   ; HAVE_MODEL
     ELSE BEGIN
        WORK_INFORM, 'Must have background model for this option.'
        WAIT, 2
     ENDELSE
     ; TBD: reinstall previous 'Bin for significance' routine.
  END

ENDCASE

DONE:
!X.TICKS = 0 & !Y.TICKS = 0


END ; PRO CONSTRUCT LOOKUP
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO OPEN_PLOT, CURRENT_DEVICE

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH

COMMON CURRENT_MENU, CURRENT_VALUES


CURRENT_DEVICE = !D.NAME

EPS = CV_ENCAPSULATED_PS
PRINT_LANG = STRTRIM(CV_PRINTER_LANG, 2)

SET_PLOT, PRINT_LANG

BREAK_FILE, CURRENT_VALUES(6), FDEV, FDIR, FNAME, FEXT
HARDCOPY_FILE = FNAME 

IF (EPS EQ 0) THEN BEGIN
   PRINT_FILE = WGSP_SCRATCH + STRLOWCASE(HARDCOPY_FILE + '.' + PRINT_LANG) 
   DEVICE, FILENAME = PRINT_FILE, /INCHES, ENCAPSULATED = 0, $
                      XOFFSET = 0.5, YOFFSET = 0, XSIZE = 8, YSIZE = 11
ENDIF ELSE BEGIN
   PRINT_FILE = WGSP_SCRATCH + STRLOWCASE(HARDCOPY_FILE + '.EPS' ) 
   DEVICE, FILENAME = PRINT_FILE, ENCAPSULATED = 1, $
                      XOFFSET = 0.5, YOFFSET = 0, XSIZE = 8, YSIZE = 11
ENDELSE


END ; PRO OPEN_PLOT
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO CLOSE_PLOT, OLD_DEVICE

COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

COMMON SOURCE_LOC, WGSP_SOURCE, WGSP_PRINTER, WGSP_SCRATCH, $
       WGSP_FITS, WGSP_LOOKUP, WGSP_DRM, WGSP_NO_MFIT, $
       WGSP_SCALE, WGSP_PRINT_COMMAND, WGSP_CONFIRM, WGSP_HELP, $
       WGSP_WORK, MFIT_CONTROL_FILENAME, MFIT_FUNCINFO_FILENAME, $
       WGSP_SEPARATE_BASE, WGSP_BROWSER, $
       WGSP_COLOR_LO, WGSP_COLOR_HI, WGSP_COLOR

COMMON CURRENT_MENU, CURRENT_VALUES


; WGSP_PRINT_COMMAND is the full print command
DEVICE, /CLOSE_FILE
FILE_STATUS = STRUPCASE(STRTRIM(CV_HCOPY_FILE_STATUS, 2))
EPS = CV_ENCAPSULATED_PS

BREAK_FILE, CURRENT_VALUES(6), FDEV, FDIR, FNAME, FEXT
HARDCOPY_FILE = FNAME + '.' + STRTRIM(CV_PRINTER_LANG, 2)

IF (FILE_STATUS NE 'KEEP') AND (EPS EQ 0) THEN BEGIN
   PRINT_FILE = WGSP_SCRATCH + STRLOWCASE(HARDCOPY_FILE) 
   SP_STRING = WGSP_PRINT_COMMAND + ' ' + PRINT_FILE 
   SPAWN, SP_STRING, RESULT  ;, /NOWAIT
ENDIF ELSE BEGIN
   IF (EPS EQ 0) THEN EXT = STRTRIM(CV_PRINTER_LANG, 2) ELSE EXT = 'EPS'
   PRINT_FILE = WGSP_SCRATCH + STRLOWCASE(FNAME + '.' + EXT) 
   PRINT, 'Hardcopy file written: ' + PRINT_FILE
ENDELSE

SET_PLOT, OLD_DEVICE
IF (WGSP_COLOR EQ 1) THEN BEGIN
   !P.BACKGROUND = WGSP_COLOR_HI
   !P.COLOR = WGSP_COLOR_LO
ENDIF ELSE BEGIN
   !P.BACKGROUND = WGSP_COLOR_LO
   !P.COLOR = WGSP_COLOR_HI
ENDELSE


END ; PRO CLOSE_PLOT
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
PRO INITIALIZE_PLOTS, NOPLOT = NOPLOT

@wgsp_cb.pro


; Start out by initializing control variables
GET_CONTROL 

ANNOTES = STRARR(20)
ANN_X = FLTARR(20)
ANN_Y = FLTARR(20)
N_ANNOTES = 0

FLAT_INDEX = 0
               
HARD_MENU = ['Choose to plot:', 'Burst History', 'Burst Spectrum', $
             'Spectral History', 'Spectral Results', 'Processed History', $
             'EXIT']

DIRTY = 0                       ;This tells us if background has been changed.
HAVE_BACK = 0                   ;We don't have a background model yet.
BACK_TIMES = FLTARR(1)          ;Don't want to plot background if not there.

BOXES = INTARR(4)
BOXES(*) = [55, 185, 439, 567]  ;Make sure this matches those in 'LAYOUT'!

PLOT_TITLES = STRARR(5)

ENERGIES = (THRESHOLDS(0, *) + THRESHOLDS(1, *))/2

N = SIZE(SPECTRA)
DEFAULT_EL = INDGEN(N(1) + 1)
DEFAULT_TL = INDGEN(N(2) + 1)

ENERGY_LOOKUP = DEFAULT_EL
TIME_LOOKUP = DEFAULT_TL

N = SIZE(SPECTRA)
TIME_RANGE = [TIMES(0, 0), TIMES(1, N(2) - 1)]
ENERGY_RANGE = [THRESHOLDS(0, 0), THRESHOLDS(1, N(1) - 1)]

TIME_SPAN = FLTARR(1, 2)
ENERGY_SPAN = FLTARR(1, 2)
BACK_SPAN = FLTARR(1, 2)
TIME_SPAN(0, *) = TIME_RANGE
ENERGY_SPAN(0, *) = ENERGY_RANGE
BACK_SPAN(0, *) = 0.0

BACK_FIT = FLTARR(N(1), 4)
BACK_ERRS = FLTARR(N(1))

N = N_ELEMENTS(TIME_LOOKUP) - 1
COMBINED_TIMES = [TIMES(0, TIME_LOOKUP(0:N - 1)), $
                  TIMES(1, TIME_LOOKUP(1:N) - 1)]

N = N_ELEMENTS(ENERGY_LOOKUP) - 1
COMBINED_ENERGIES = [THRESHOLDS(0, ENERGY_LOOKUP(0:N - 1)), $
                     THRESHOLDS(1, ENERGY_LOOKUP(1:N) - 1)]

INTEGRATE_SPECTRUM
INTEGRATE_HISTORY

COMBINE, SPECTRA_T, SPECTRA_T_ERR, THRESHOLDS, ENERGY_LOOKUP, SPECTRUM, $
         SPECTRUM_ERR, COMBINED_ENERGIES
NIND = WHERE(SPECTRUM, COUNT)
IF COUNT EQ 0 THEN BEGIN
   RATE_RANGE = [MIN(SPECTRUM - SPECTRUM_ERR), MAX(SPECTRUM + SPECTRUM_ERR)]
ENDIF ELSE BEGIN
   RATE_RANGE = [MIN(SPECTRUM(NIND) - SPECTRUM_ERR(NIND)), $
                 MAX(SPECTRUM + SPECTRUM_ERR)]
   ENDELSE
WSET, SPECTRUM_WINDOW
PSYM_SAVE = !P.PSYM
!P.PSYM = 0

!X.TITLE = READ_MENU(4)
!Y.TITLE = READ_MENU(5)
ENERGY_SCALE = READ_MENU(1)
RATE_SCALE = READ_MENU(2)
TITLE = CREATE_LABEL(TIMES, TIME_SPAN, ' s')

ROUND_AXES, ENERGY_RANGE, ENERGY_SCALE EQ 'LOG', /POWER
ROUND_AXES, RATE_RANGE, RATE_SCALE EQ 'LOG', /POWER

IF N_ELEMENTS(NOPLOT) EQ 0 THEN $
STD_DISPLAY, SPECTRUM, SPECTRUM_ERR, COMBINED_ENERGIES, ENERGY_RANGE, $
             RATE_RANGE, ENERGY_SPAN, TITLE, ENERGY_SCALE, RATE_SCALE, $
             0, 'SPECTRUM'

IF ENERGY_SCALE EQ 'LINEAR' THEN BEGIN
   DISPLAY_RAINBOW, THRESHOLDS
   ENDIF

COMBINE, SPECTRA_E, SPECTRA_E_ERR, TIMES, TIME_LOOKUP, HISTORY, HISTORY_ERR, $
         COMBINED_TIMES
HIST_RANGE = [MIN(HISTORY), MAX(HISTORY)]

WSET, HISTORY_WINDOW
!X.TITLE = READ_MENU(3)
!Y.TITLE = READ_MENU(5)

IF (CV_RATE_MODE EQ 'R') THEN BEGIN       
   !Y.TITLE = STRMID(!Y.TITLE, 0, STRPOS(!Y.TITLE, ' keV'))
   ENDIF
TITLE = CREATE_LABEL(THRESHOLDS, ENERGY_SPAN, ' keV')

ROUND_AXES, TIME_RANGE, 0
ROUND_AXES, HIST_RANGE, 0

IF N_ELEMENTS(NOPLOT) EQ 0 THEN $
STD_DISPLAY, HISTORY, HISTORY_ERR, COMBINED_TIMES, TIME_RANGE, $
;             HIST_RANGE, TIME_SPAN, TITLE, 'LINEAR', 'LINEAR', 0, 'HISTORY'
             HIST_RANGE, TIME_SPAN, TITLE, $
             READ_MENU(9), READ_MENU(10), 0, 'HISTORY'

!P.PSYM = PSYM_SAVE



END; INITIALIZE_PLOTS
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *


; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Creates a plot to denote missing data segments
PRO PLOT_GAPS, TIMES, SPECTRA, SPECTRA_ERRORS, $
    CHANNEL_OFFSET, CURRENT_DETECTOR, DATA_DIR, CURRENT_DATA_FILE


DEVICE, GET_SCREEN_SIZE = CS
NUM_POINTS = N_ELEMENTS(TIMES(0, *))
NUM_CHANNELS = N_ELEMENTS(SPECTRA(*, 0))
TIME_MID = (TIMES(1, *) - TIMES(0, *)) + TIMES(0, *)
CHAN_OFFSET = CHANNEL_OFFSET(CURRENT_DETECTOR)

FOR I = 0, NUM_CHANNELS - 1 DO BEGIN

    INDEX = WHERE(SPECTRA_ERRORS(I, *) LT 0.0, COUNT)
    IF (I EQ 0) THEN BEGIN
       WINDOW, /FREE, TITLE = 'Missing Data', $
               XSIZE = CS(0)*0.75, YSIZE = CS(1)*0.75
       PLOT, /NODATA, TIMES(0, *), INDGEN(NUM_CHANNELS), $
             TITLE = '!5', $
             YRANGE = [CHAN_OFFSET - 1, (CHAN_OFFSET + NUM_CHANNELS) + 1], $
             YSTYLE = 1, $
             YTITLE = 'Energy Channel', $
             XTITLE = 'Seconds Since Burst Trigger'
    ENDIF
 
    OPLOT, !X.CRANGE, [I, I] + CHAN_OFFSET, LINESTYLE = 1
    IF (COUNT NE 0) THEN BEGIN
       OPLOT, TIME_MID(INDEX), $
              FLTARR(N_ELEMENTS(INDEX)) + I + CHAN_OFFSET, PSYM = 1
    ENDIF

ENDFOR

CHOICE = WIDMENU(['WARNING: Missing data points in FITS file', $
                'List Times', 'Write Times to File', $
                'Continue'], TITLE = 0)

WRITE_FLAG = 0
IF (CHOICE EQ 1) THEN BEGIN
        DT = ''
        FOR I = 0, NUM_CHANNELS - 1 DO BEGIN

            INDEX = WHERE(SPECTRA_ERRORS(I, *) LT 0.0, COUNT)
            IF (COUNT NE 0) THEN BEGIN
               DT = [DT, 'Energy Channel ' + STRTRIM(I + CHAN_OFFSET, 2)]
               FOR J=0, COUNT - 1 DO BEGIN
                   DT = [DT, '   ' + STRTRIM(TIMES(0, INDEX(J)), 2) + $
                             '   ' + STRTRIM(TIMES(1, INDEX(J)), 2)]
               ENDFOR
            ENDIF

        ENDFOR
        W_TEXT, DT
        JUNK = WIDMENU(['Write Times to File', 'Continue'])
        IF (JUNK EQ 0) THEN WRITE_FLAG = 1
ENDIF

IF ((CHOICE EQ 2) OR (WRITE_FLAG EQ 1)) THEN BEGIN

        BREAK_FILE, CURRENT_DATA_FILE, DDEV, DDIR, DNAME, DEXT, DVERSION
        OUTFILE = DATA_DIR + STRLOWCASE(DNAME + '.GAPS')
        OPENW, FL, OUTFILE, /GET_LUN
        FOR I = 0, NUM_CHANNELS - 1 DO BEGIN

            INDEX = WHERE(SPECTRA_ERRORS(I, *) LT 0.0, COUNT)
            IF (COUNT NE 0) THEN BEGIN
               PRINTF, FL, 'Energy Channel ' + STRTRIM(I + CHAN_OFFSET, 2)
               FOR J=0, COUNT - 1 DO BEGIN
                   PRINTF, FL, '   ', TIMES(0, INDEX(J)), TIMES(1, INDEX(J))
               ENDFOR
            ENDIF

        ENDFOR
        INFORM, 'Gap times written to file ' + OUTFILE
        FREE_LUN, FL
ENDIF

WDELETE



END
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
