; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
;+
; NAME:
;	WID_DIALOG
;
; PURPOSE:
;	A dialog box for IDL versions earlier than 4.0
;
; CATEGORY:
;	WIDGETS
;
; CALLING SEQUENCE:
;	RESULT = WID_DIALOG(TITLE [, /NO_CANCEL_BUTTON])
;
; INPUTS:
;       TITLE:   the title
;
; KEYWORD PARAMETERS:
;       NO_CANCEL_BUTTON: Turn off cancel button
;
; OUTPUTS:
;       RESULT = 'OK' or 'CANCEL'
;
; COMMON BLOCKS:
;	NONE
;
; SIDE EFFECTS:
;       Creates a button widget
;
; RESTRICTIONS:
;       NONE
;
; PROCEDURE:
;       Function call
;
; EXAMPLES:
;       QUIT confirmation:
;       STATUS = WID_DIALOG('Really QUIT?')
;
;       Error notification message:
;       STATUS = WID_DIALOG('ERROR Opening File to Read', /NO_CANCEL)
;
; MODIFICATION HISTORY:
;	Written, mallozzi@gibson.msfc.nasa.gov, Feb 1996.
;-
FUNCTION WID_DIALOG, TITLE, NO_CANCEL_BUTTON = NO_CANCEL_BUTTON



; Parent
MENU_BASE = WIDGET_BASE(TITLE=' ', /COLUMN, MAP = 0, XSIZE = 200)

 ; Add the title
 T_BASE = WIDGET_LABEL(MENU_BASE, VALUE = TITLE)

 FILLER = WIDGET_LABEL(MENU_BASE, VALUE = ' ')

 ; Selection buttons
 SUB_BASE = WIDGET_BASE(MENU_BASE, /COLUMN)

 M_BUT_OK = WIDGET_BUTTON(SUB_BASE, VALUE = 'OK')
 IF (N_ELEMENTS(NO_CANCEL_BUTTON) EQ 0) THEN BEGIN
    M_BUT_C = WIDGET_BUTTON(SUB_BASE, VALUE = 'Cancel')
 ENDIF

; Make it so
WIDGET_CONTROL, MENU_BASE, /REALIZE 

; Place the menu in center of screen
DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
WIDGET_CONTROL, MENU_BASE, TLB_GET_SIZE = MENU_SIZE
MENU_CENTER = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
     	       (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
WIDGET_CONTROL, MENU_BASE, $
                TLB_SET_XOFFSET = MENU_CENTER(0), $
                TLB_SET_YOFFSET = MENU_CENTER(1)
WIDGET_CONTROL, MENU_BASE, MAP = 1

; Place the cursor 
;WIDGET_CONTROL, M_BUT_OK, /INPUT_FOCUS

; Get the event, without using XMANAGER
EVENT = WIDGET_EVENT(MENU_BASE)

; Process the event
TYPE = TAG_NAMES(EVENT, /STRUCTURE)
CASE TYPE OF

     ; The button widget events
     'WIDGET_BUTTON': BEGIN
                      WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE
                      WIDGET_CONTROL, EVENT.TOP, /DESTROY

                      CASE STRUPCASE(VALUE(0)) OF

                           'OK': BEGIN
                        	 RETURN, 'OK'
                        	 END
                           'CANCEL': BEGIN
                        	 RETURN, 'CANCEL'
                        	 END
                      ENDCASE
                     END


ENDCASE ; for TYPE
   


END 
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; NAME:
;	CHECK_DECADE
;
; PURPOSE:
;       Checks input 2-element vector and checks if span is less than
;       a single decade.  Used to adjust IDL's incorrect axis labeling
;       for logarithmic plots.
;
; CATEGORY:
;	Plotting
;
; CALLING SEQUENCE:
;	RESULT = CHECK_DECADE(INARR)
;
; INPUTS:
;	INARR: 2-element vector that contains the axis endpoints
; 
; INPUT PARAMETERS:
;       NONE.
; 
; KEYWORD PARAMETERS:
;       NONE.
;
; OUTPUTS:
;       RESULT = 1 if span is less than 1 decade
;       RESULT = 0 if span is greater than or equal to 1 decade
;       RESULT = -1 on erorr
;
; COMMON BLOCKS:
;	NONE.
;
; SIDE EFFECTS:
;	NONE.
;
; RESTRICTIONS:
;	NONE.
;
; PROCEDURE:
;       FUNCTION CALL
;
; MODIFICATION HISTORY:
;	Written, mallozzi@gibson.msfc.nasa.gov, May, 1995.
;
FUNCTION CHECK_DECADE, INARR


IF (N_ELEMENTS(INARR) NE 2) THEN BEGIN
   PRINT, 'CHECK_DECADE: Argument must be a 2-element vector.'
   RETURN, -1
ENDIF

INARR  = INARR(SORT(INARR))
BEG_PT = FLOAT(INARR(0))
END_PT = FLOAT(INARR(1))

IF ((BEG_PT EQ 0) OR (END_PT EQ 0)) THEN BEGIN
   PRINT, 'CHECK_DECADE: Axis point cannot be zero for log scale.'
   RETURN, -1
ENDIF

IF (END_PT / BEG_PT LT 10.0) THEN ONE_DECADE = 1 ELSE ONE_DECADE = 0


RETURN, ONE_DECADE
END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; The version of HEADFITS in the FITS library does not handle 
;  IO errors.  This version has been modified for the WINGSPAN/MFIT
;  package to trap any IO errors
;
function HEADFITS, filename, EXTEN = exten
;+
; NAME:
;	HEADFITS
; CALLING SEQUENCE:
;	Result = headfits( filename ,[ EXTEN = ])
;
; PURPOSE:
;	Read a FITS file header record      
;
; INPUTS:
;	FILENAME = String containing the name of the FITS file to be read.
;
; OPTIONAL INPUT KEYWORD:
;	EXTEN  = integer scalar, specifying which FITS extension to read.
;		For example, to read the header of the first extension set
;		EXTEN = 1.   Default is to read the primary FITS header 
;		(EXTEN = 0).
;
; OUTPUTS:
;	Result of function = FITS header, string array
;
; EXAMPLE:
;	Read the FITS header of a file 'test.fits' into a string variable, h
;
;	IDL>  h = headfits( 'test.fits')
;
; MODIFICATION HISTORY:
;	adapted by Frank Varosi from READFITS by Jim Wofford, January, 24 1989
;	Keyword EXTEN added, K.Venkatakrishna, May 1992
;	Make sure first 8 characters are 'SIMPLE'  W. Landsman October 1993
;       Added On_ioerror statement, R. Mallozzi, Apr, 1995
;-
 On_error,2
 On_ioerror, io_err

 if N_params() LT 1 then begin
     print,'Syntax - header = headfits( filename, [ EXTEN = ])
     return, -1
 end
 
; Open file and read header information
       	openr,unit,filename, /GET_LUN, /BLOCK
	file = fstat(unit)
	y = indgen(36*8)
	y2 = y - 8*(y/8) + 80*(y/8)
        offset = 0
        extn = 0
START: 	r = 0
        hdr = assoc(unit, bytarr(80,36), offset)
; Read header one record at a time

	nbytesleft = file.size - offset
	
        if nbytesleft LT 2880 then $
		message,' No such extension, End of file reached'

LOOP:
	x = hdr(r)
	nbytesleft = nbytesleft - 2880
	name = string( x(y2) )		;Get first 8 char of each line
        if (r EQ 0) and (extn EQ 0) then $
 		if strmid(name,0,8) NE 'SIMPLE  ' then begin 
		free_lun, unit              ;Added Mar 94
		message, $
	   'ERROR - FITS header missing required "SIMPLE" in first 8 chars'
		endif
	
        pos = strpos( name, 'END     ' )
        if r EQ 0 then header = string(x) else header = [header,string(x)]
	if (pos lt 0) then begin
		r = r + 1
		goto, LOOP 
	endif 
    	lastline = 36*r + pos / 8
	header = header(0:lastline)
;                                        IF extension, get the size of the
;                                        data. Find no of records to skip 
        If keyword_set(EXTEN) then begin
        bitpix = sxpar( header, 'BITPIX')
        naxis = sxpar( header, 'NAXIS')
        Nax = sxpar( header, 'NAXIS*' )                  ; Read NAXES
        nbytes = nax(0) * abs( bitpix )/ 8
        if naxis GT 1 then for i = 2, naxis do nbytes = nbytes*nax(i-1) $
                      else nbytes = 0
        nrec = nbytes /2880
        if nbytes GT nrec*2880L then nrec = long( nrec + 1 ) else $
        nrec = long(nrec)                
        point_lun, -unit, pointlun
        pointlun = pointlun + nrec*2880L
        point_lun,unit,pointlun
        offset = pointlun
        extn = extn + 1
        if (extn LE EXTEN) then goto, START 
        endif
	free_lun, unit

IO_ERR:
if (N_ELEMENTS(header) EQ 0) then header = 'IO_ERROR'
return, header

end
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION SEARCH, X, TARGET

IF TARGET LT X(0) THEN RETURN, -1

FIRST = 0
LAST = N_ELEMENTS(X) - 1

SEARCHING:
   BULLSEYE = FIRST + ((LAST - FIRST) + 1)/2
   IF FIRST EQ LAST THEN RETURN, BULLSEYE

   IF TARGET LT X(BULLSEYE) THEN LAST = BULLSEYE - 1 $
   ELSE IF BULLSEYE EQ LAST THEN RETURN, BULLSEYE $
   ELSE FIRST = BULLSEYE
   GOTO, SEARCHING

RETURN, BULLSEYE
END; FUNCTION SEARCH
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION AVERAGE, X, F, LOOKUP
N = N_ELEMENTS(LOOKUP) - 1
A = FLTARR(N)
FOR I = 0, N - 1 DO BEGIN
   I0 = LOOKUP(I)
   I1 = LOOKUP(I + 1) - 1
   T = TOTAL(F(I0:I1))
   IF T GT 0 THEN A(I) = TOTAL(X(I0:I1)*F(I0:I1))/T $
             ELSE A(I) = TOTAL(X(I0:I1))/(I1 - I0 + 1)
   ENDFOR



RETURN, A
END ; FUNCTION AVERAGE
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION NEAREST_EDGE, THRESHOLDS, RANGE, INDICES

N = SIZE(THRESHOLDS)

; Only one bin selected
IF (N(0) EQ 1) THEN BEGIN
   INDICES = [0, 1]
   RETURN, THRESHOLDS
ENDIF

ER = [TRANSPOSE(THRESHOLDS(0, *)), THRESHOLDS(1, N(2) - 1)]
I0 = SEARCH(ER, RANGE(0))
IF I0 EQ -1 THEN I0 = 0 ELSE IF I0 EQ N(2) THEN I0 = N(2) - 1
IF (THRESHOLDS(1, I0) LT RANGE(0)) AND (I0 LT N(2) - 1) THEN I0 = I0 + 1
I1 = SEARCH(ER, RANGE(1))
IF I1 EQ -1 THEN I1 = 0 ELSE IF I1 EQ N(2) THEN I1 = N(2) - 1
IF KEYWORD_SET(INDICES) THEN INDICES = [I0, I1]



RETURN, [THRESHOLDS(0, I0), THRESHOLDS(1, I1)]
END; FUNCTION NEAREST_EDGE
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION GET_SINGLE_EDGE, THRESHOLDS, TARGET, INDEX

; Find the NEAREST bin edge to the input value 
; NEAREST_EDGE finds the range that brackets RANGE

JUNK0 = MIN( ABS(THRESHOLDS(0,*) - TARGET), INDEX0 )
JUNK1 = MIN( ABS(THRESHOLDS(1,*) - TARGET), INDEX1 )
JUNK  = MIN([JUNK0, JUNK1], TEMP_INDEX)
    
IF (TEMP_INDEX EQ 0) THEN BEGIN
   IFLAG = 0
   INDEX = INDEX0 
ENDIF ELSE BEGIN
   IFLAG = 1
   INDEX = INDEX1
ENDELSE
    
RETURN, THRESHOLDS(IFLAG, INDEX)
END; FUNCTION GET_SINGLE_EDGE
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION CREATE_LABEL, COMBINED, SPAN, PLACE_STR

EB = NEAREST_EDGE(COMBINED, SPAN(0, *), ARANGE)

   IF LONG(ABS(EB(0))) GT 100 THEN $
     E0 = STRTRIM(STRING(FORMAT='(D15.2)',EB(0)), 2) $
   ELSE $
     E0 = STRTRIM(STRING(FORMAT='(D15.3)',EB(0)), 2)

   IF LONG(ABS(EB(1))) GT 10000 THEN $
     E1 = STRTRIM(STRING(FORMAT='(D15.0)',EB(1)), 2) $
   ELSE IF LONG(ABS(EB(1))) GT 100 THEN $
     E1 = STRTRIM(STRING(FORMAT='(D15.2)',EB(1)), 2) $
   ELSE $
     E1 = STRTRIM(STRING(FORMAT='(D15.3)',EB(1)), 2)

THE_TITLE = E0 + ': ' + E1 + PLACE_STR



RETURN, THE_TITLE
END; FUNCTION CREATE_LABEL
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
;+
; NAME:
;	WIDMENU  (CUSTOMIZED FOR WINGSPAN)
;
; PURPOSE:
;	Widget version of WMENU, plus optional SCROLL bar
;
; CATEGORY:
;	WIDGETS
;
; CALLING SEQUENCE:
;	RESULT = WIDMENU(SELECTIONS, [TITLE = , INITIAL = , /SCROLL, WIDTH = ])
;
; INPUTS:
;       SELECTIONS: STRARR of the menu selections
;
; KEYWORD PARAMETERS:
;       TITLE   : The title
;       INITIAL : The inital cursor selection
;       SCROLL  : Adds a scroll bar
;       WIDTH   : Width of the widget in pixels (height is set automatically
;                 based on the number of buttons)
;       UL      : Place the menu in the upper left corner
;       LL      : Place the menu in the lower left corner
;       UR      : Place the menu in the upper right corner
;       LR      : Place the menu in the lower right corner
;
; OUTPUTS:
;       RESULT = index of the menu item (including TITLE, if supplied)
;
; COMMON BLOCKS:
;	NONE
;
; SIDE EFFECTS:
;       Creates a widget on the current device
;
; RESTRICTIONS:
;	TITLE can only be a single line on the widget
;
; DEPENDENCIES:
;       NONE
;
; PROCEDURE:
;       Function call; mimics WMENU 
;
; EXAMPLES:
;       Error notification message:
;       INDEX = WIDMENU(['ERROR Opening File to Read', 'OK'], $
;                         title = 0, initial = 1)
;
;       QUIT confirmation:
;       INDEX = WIDMENU(['Do you really want to QUIT?', 'Yes', 'No'], $
;                         title = 0, initial = 1)
;
; MODIFICATION HISTORY:
;	Written, <mallozzi@gibson.msfc.nasa.gov>, Dec 1994.
;       RSM, July 1995, placed menu in center of screen (thanks M. Bell)
;       RSM, July 1995, added check to see if IDL widgets are supported; if
;            not, WMENU is used 
;       RSM, Nov 1995, fixed initial cursor placement
;       RSM, Mar 1996, added UL, LL, UR, LR keywords
;-
FUNCTION WIDMENU, SELECTIONS, TITLE = TITLE, SCROLL = SCROLL, $
                  INITIAL = INITIAL, WIDTH = WIDTH, $
                  UL = UL, LL = LL, UR = UR, LR = LR

COMMON TWIDS, WIDS


IF (WIDS.TOP_BASE NE 0) THEN $
   WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 0
IF (WIDS.WORK_BASE NE 0) THEN $
   WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 0
IF (WIDS.MFIT_BASE NE 0) THEN $
   WIDGET_CONTROL, WIDS.MFIT_BASE, SENSITIVE = 0

NUM_BUTTONS  = N_ELEMENTS(SELECTIONS)

HAVE_TITLE   = N_ELEMENTS(TITLE) NE 0
HAVE_INITIAL = N_ELEMENTS(INITIAL) NE 0
HAVE_WIDTH   = N_ELEMENTS(WIDTH) NE 0
HAVE_SCROLL  = N_ELEMENTS(SCROLL) NE 0

IF (HAVE_TITLE) THEN BEGIN
   IF (TITLE GT NUM_BUTTONS - 1) THEN TITLE = NUM_BUTTONS - 1
   THE_TITLE = SELECTIONS(TITLE)
   LOCAL_SELECTIONS = SELECTIONS(WHERE(INDGEN(NUM_BUTTONS) NE TITLE))
ENDIF ELSE BEGIN
   THE_TITLE = ''
   LOCAL_SELECTIONS = SELECTIONS
ENDELSE

; Widgets available?
IF ((!D.FLAGS AND 65536) NE 0) THEN HAVE_WIDGETS = 1 ELSE HAVE_WIDGETS = 0

IF (HAVE_WIDGETS) THEN BEGIN

   ; Parent
   IF (HAVE_WIDTH) THEN BEGIN
      MENU_BASE = WIDGET_BASE(TITLE = ' ', /COLUMN, XSIZE = WIDTH, MAP = 0)
   ENDIF ELSE BEGIN
      MENU_BASE = WIDGET_BASE(TITLE = ' ', /COLUMN, MAP = 0)
   ENDELSE

   ; Add the title, if it's supplied
   IF (HAVE_TITLE) THEN BEGIN
      T_BASE = WIDGET_LABEL(MENU_BASE, VALUE = THE_TITLE)
   ENDIF

   ; Selection buttons
   IF (HAVE_SCROLL) THEN BEGIN
      B_BASE = WIDGET_BASE(MENU_BASE, /COLUMN, /FRAME, /SCROLL)
   ENDIF ELSE BEGIN
      B_BASE = WIDGET_BASE(MENU_BASE, /COLUMN, /FRAME)
   ENDELSE

   M_BUT = LONARR(NUM_BUTTONS)
   FOR I=0, N_ELEMENTS(LOCAL_SELECTIONS)-1 DO BEGIN
       M_BUT(I) = WIDGET_BUTTON(B_BASE, VALUE = LOCAL_SELECTIONS(I), UVALUE = I)
   ENDFOR

   ; Make it so
   WIDGET_CONTROL, MENU_BASE, /REALIZE 


   ; Place the menu
   DEVICE, GET_SCREEN_SIZE = CURRENT_SCREEN
   WIDGET_CONTROL, MENU_BASE, TLB_GET_SIZE = MENU_SIZE

   IF (N_ELEMENTS(UL) NE 0) THEN BEGIN
      MENU_PT = [0, 0] 
   ENDIF ELSE $
   IF (N_ELEMENTS(LL) NE 0) THEN BEGIN
      MENU_PT = [0, CURRENT_SCREEN(1) - MENU_SIZE(1)] 
   ENDIF ELSE $
   IF (N_ELEMENTS(UR) NE 0) THEN BEGIN
      MENU_PT = [CURRENT_SCREEN(0) - MENU_SIZE(0), 0] 
   ENDIF ELSE $
   IF (N_ELEMENTS(LR) NE 0) THEN BEGIN
      MENU_PT = [CURRENT_SCREEN(0) - MENU_SIZE(0), $ 
                 CURRENT_SCREEN(1) - MENU_SIZE(1)] 
   ENDIF ELSE BEGIN
      MENU_PT = [(CURRENT_SCREEN(0) / 2.0) - (MENU_SIZE(0) / 2.0), $ 
                 (CURRENT_SCREEN(1) / 2.0) - (MENU_SIZE(1) / 2.0)] 
   ENDELSE

   WIDGET_CONTROL, MENU_BASE, $
                   TLB_SET_XOFFSET = MENU_PT(0), $
                   TLB_SET_YOFFSET = MENU_PT(1)
   WIDGET_CONTROL, MENU_BASE, MAP = 1
 
   ; Place the cursor (4 cases)
   DONT_PLACE = 0
   CASE 1 OF
        HAVE_TITLE AND HAVE_INITIAL: BEGIN
             ; Is INITIAL valid?
             INDEX = WHERE(INDGEN(NUM_BUTTONS) NE TITLE)
             BUTTON_INDEX = WHERE(INDEX EQ INITIAL, INITIAL_VALID)
             IF (INITIAL_VALID) THEN BEGIN
                IF (TITLE LT INITIAL) THEN BEGIN
                   CURSOR_POSITION = INITIAL - 1
                ENDIF ELSE BEGIN
                   CURSOR_POSITION = INITIAL
                ENDELSE
             ENDIF ELSE BEGIN
                ; INITIAL not valid, just use the first available button
                CURSOR_POSITION = 0
             ENDELSE
             END

        NOT(HAVE_TITLE) AND HAVE_INITIAL: BEGIN
             ; Is INITIAL valid?
             INDEX = INDGEN(NUM_BUTTONS)
             BUTTON_INDEX = WHERE(INDEX EQ INITIAL, INITIAL_VALID)
             IF (INITIAL_VALID) THEN BEGIN
                CURSOR_POSITION = INITIAL
             ENDIF ELSE BEGIN
                ; INITIAL not valid, just use the first available button
                CURSOR_POSITION = 0
             ENDELSE
             END

;        HAVE_TITLE AND NOT(HAVE_INITIAL): BEGIN
;             ; Just use the first available button
;             CURSOR_POSITION = 0
;             END
;
;        ELSE: BEGIN
;             ; Just use the first available button
;             CURSOR_POSITION = 0
;             END
        ; RSM, 1996 March, if INITIAL is not specified, don't place 
        ;  cursor at all
        ELSE: BEGIN
             DONT_PLACE = 1
             END
   ENDCASE

   IF (DONT_PLACE EQ 0) THEN BEGIN
      WIDGET_CONTROL, M_BUT(CURSOR_POSITION), /INPUT_FOCUS
   ENDIF

   ; Get the event, without using XMANAGER
   EVENT = WIDGET_EVENT(MENU_BASE)

   ; Process the event
   TYPE = TAG_NAMES(EVENT, /STRUCTURE)
   CASE TYPE OF

    ; The button widget events
    'WIDGET_BUTTON': BEGIN
            WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, $
                            GET_UVALUE = UVALUE
                   
            FOR I=0, N_ELEMENTS(LOCAL_SELECTIONS)-1 DO BEGIN
                IF (UVALUE EQ I) THEN BEGIN
                   WIDGET_CONTROL, EVENT.TOP, /DESTROY
                   IF (HAVE_TITLE) THEN BEGIN
                      IF (I GE TITLE) THEN BEGIN
                         ADD_I = HAVE_TITLE 
                      ENDIF ELSE ADD_I = 0
                   ENDIF ELSE ADD_I = 0
                   IF (WIDS.TOP_BASE NE 0) THEN $
                      WIDGET_CONTROL, WIDS.TOP_BASE, SENSITIVE = 1
                   IF (WIDS.WORK_BASE NE 0) THEN $
                      WIDGET_CONTROL, WIDS.WORK_BASE, SENSITIVE = 1
                   IF (WIDS.MFIT_BASE NE 0) THEN $
                       WIDGET_CONTROL, WIDS.MFIT_BASE, SENSITIVE = 1
                   RETURN, I + ADD_I
                ENDIF
            ENDFOR
            END ; WIDGET_BUTTON events

   ENDCASE ; for TYPE

ENDIF ELSE BEGIN ; HAVE_WIDGETS = 0

   ; Place the cursor (4 cases)
   CASE 1 OF
        HAVE_TITLE AND HAVE_INITIAL: BEGIN
             ; Is INITIAL valid?
             INDEX = WHERE(INDGEN(NUM_BUTTONS) NE TITLE)
             BUTTON_INDEX = WHERE(INDEX EQ INITIAL, INITIAL_VALID)
             IF (INITIAL_VALID) THEN BEGIN
                   CURSOR_POSITION = INITIAL
             ENDIF ELSE BEGIN
                ; INITIAL not valid, just use the first available button
                CURSOR_POSITION = 0
             ENDELSE
             END

        NOT(HAVE_TITLE) AND HAVE_INITIAL: BEGIN
             ; Is INITIAL valid?
             INDEX = WHERE(INDGEN(NUM_BUTTONS) NE TITLE)
             BUTTON_INDEX = WHERE(INDEX EQ INITIAL, INITIAL_VALID)
             IF (INITIAL_VALID) THEN BEGIN
                IF (TITLE LT INITIAL) THEN BEGIN
                   CURSOR_POSITION = INITIAL - 1
                ENDIF ELSE BEGIN
                   CURSOR_POSITION = INITIAL
                ENDELSE
             ENDIF ELSE BEGIN
                ; INITIAL not valid, just use the first available button
                CURSOR_POSITION = 0
             ENDELSE
             END

        HAVE_TITLE AND NOT(HAVE_INITIAL): BEGIN
             ; Just use the first available button
             CURSOR_POSITION = 0
             END

        ELSE: BEGIN
             ; Just use the first available button
             CURSOR_POSITION = 0
             END
   ENDCASE

   IF (HAVE_TITLE) THEN BEGIN
      I = WMENU(SELECTIONS, TITLE = TITLE, INITIAL = CURSOR_POSITION)
   ENDIF ELSE BEGIN
      I = WMENU(SELECTIONS, INITIAL = CURSOR_POSITION)
   ENDELSE

   RETURN, I

ENDELSE ; HAVE_WIDGETS



END ; FUNCTION WIDMENU
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Functions to perform menu tasks 

FUNCTION READ_MENU, INDEX
  COMMON CURRENT_MENU, CURRENT_VALUES
  VALUE = CURRENT_VALUES(INDEX)
  RETURN, VALUE
END

PRO INFORM, MESSAGE, NO_STATUS = NO_STATUS
  COMMON TWIDS, WIDS
  MESSAGE = MESSAGE(0)
  IF MESSAGE EQ '' OR MESSAGE EQ ' ' THEN BEGIN
     IVALUE = '' 
  ENDIF ELSE BEGIN
     IF N_ELEMENTS(NO_STATUS) EQ 0 THEN IVALUE = 'STATUS:  ' + MESSAGE $
     ELSE IVALUE = MESSAGE
  ENDELSE
  WIDGET_CONTROL, WIDS.M_STATUS, SET_VALUE = IVALUE
END

PRO WORK_INFORM, MESSAGE, NO_STATUS = NO_STATUS
  COMMON TWIDS, WIDS
  MESSAGE = MESSAGE(0)
  IF MESSAGE EQ '' OR MESSAGE EQ ' ' THEN BEGIN
     IVALUE = '' 
  ENDIF ELSE BEGIN
     IF N_ELEMENTS(NO_STATUS) EQ 0 THEN IVALUE = 'STATUS:  ' + MESSAGE $
     ELSE IVALUE = MESSAGE
  ENDELSE
  STATUS = WIDGET_INFO(WIDS.MW_STATUS, /VALID_ID)
  IF (STATUS NE 0) THEN BEGIN 
     WIDGET_CONTROL, WIDS.MW_STATUS, SET_VALUE = IVALUE
  ENDIF
END

PRO MFIT_INFORM, MESSAGE, NO_STATUS = NO_STATUS
  COMMON TWIDS, WIDS
  MESSAGE = MESSAGE(0)
  IF MESSAGE EQ '' OR MESSAGE EQ ' ' THEN BEGIN
     IVALUE = '' 
  ENDIF ELSE BEGIN
     IF N_ELEMENTS(NO_STATUS) EQ 0 THEN IVALUE = 'STATUS:  ' + MESSAGE $
     ELSE IVALUE = MESSAGE
  ENDELSE
  STATUS = WIDGET_INFO(WIDS.MM_STATUS, /VALID_ID)
  IF (STATUS NE 0) THEN BEGIN 
     WIDGET_CONTROL, WIDS.MM_STATUS, SET_VALUE = IVALUE
  ENDIF
END

PRO HC_INFORM, MESSAGE, NO_STATUS = NO_STATUS
  COMMON TWIDS, WIDS
  MESSAGE = MESSAGE(0)
  IF MESSAGE EQ '' OR MESSAGE EQ ' ' THEN BEGIN
     IVALUE = '' 
  ENDIF ELSE BEGIN
     IF N_ELEMENTS(NO_STATUS) EQ 0 THEN IVALUE = 'STATUS:  ' + MESSAGE $
     ELSE IVALUE = MESSAGE
  ENDELSE
  STATUS = WIDGET_INFO(WIDS.HC_STATUS, /VALID_ID)
  IF (STATUS NE 0) THEN BEGIN 
     WIDGET_CONTROL, WIDS.HC_STATUS, SET_VALUE = IVALUE
  ENDIF
END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; WINGSPAN PLOT WIDGET
PRO GET_HC_EVENT, MAKE_HC, ZOOM_HC

COMMON TWIDS, WIDS


; Assume no hardcopy, no zoom
MAKE_HC = 0
ZOOM_HC = 0

EVENT = WIDGET_EVENT(WIDS.HC_BASE)

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
; The button widget events
WIDGET_CONTROL, EVENT.ID, GET_VALUE=VALUE, GET_UVALUE=UVALUE
CASE 1 OF 

     VALUE EQ 'DONE': BEGIN          
           WIDGET_CONTROL, EVENT.TOP, /DESTROY
           END
     VALUE EQ 'Create Hardcopy': BEGIN           
           MAKE_HC = 1
           END
     VALUE EQ 'Zoom': BEGIN          
           ZOOM_HC = 1
           END

ENDCASE ; for VALUE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO W_HARDCOPY, MTITLE, WNOTES, $
    BIG = BIG, NOINQUIRE = NOINQUIRE, ZOOM = ZOOM, NODONE = NODONE
; WNOTES is a string array that can optionally be added 
; Set NOINQUIRE to omit the option to create a hardcopy
; Set NODONE to omit the DONE button
; Set ZOOM to add a zoom button

COMMON TWIDS, WIDS
COMMON THE_SCREEN, CURRENT_SCREEN
COMMON WSP_CONTROL_C, $
       CV_DEBUG, CV_TALKY, CV_EXP_USER, CV_CONTOUR_CHOICE, $
       CV_WRITE_CONTOUR_CHI_STEPS, CV_AUTO_COMMENT, $
       CV_FIRST_BIN_CALC, CV_LAST_BIN_CALC, $
       CV_BATCH_FIT_SUMMARY, CV_GAUSS_OFFSET, $
       CV_START_GAUSS_E, CV_END_GAUSS_E, CV_GAUSS_STEP_SIZE, $
       CV_GAUSS_VWIDTH, CV_FIX_UNDETERMINED, CV_AXES_SAME, $
  
       CV_PRINTER_LANG, CV_PLOT_MODEL_EXTRAP, $
       CV_PLOT_EACH_TERM, CV_ABS_MODEL, CV_PLOT_ONLY_DATA, $
       CV_SUPPRESS_2ND_LABELS, CV_MULTI_DET_PLOT_MODE, $
       CV_GRAPH_LIMIT_MODE, CV_HCOPY_FILE_STATUS, $
       CV_ENCAPSULATED_PS, CV_SPEC_STYLE, $

       CV_RATE_MODE, CV_TRACK_MOUSE, CV_PLOT_UNDETERMINED, $
       CV_PLOT_CURSOR, CV_IDL_RETAIN, CV_TITLE_STYLE, $
       CV_SPARE_1, CV_SPARE_2, CV_SPARE_3, $

       CV_KEEP_ASCII_OPEN, CV_ASCII_OUT_FILE, $
       CV_STARTUP_TEMPLATE_FILE, CV_FULL_BATCH_OUT, $
       CV_SUPPRESS_BATCH_OUT, CV_BATCH_OUT_FILE, CV_ALWAYS_WA, $
       CV_ALWAYS_WB, $

       CV_REL_CONVERG, CV_ABS_CONVERG, CV_MAX_TRIES

DEVICE, RETAIN = CV_IDL_RETAIN

; Parent
IF N_ELEMENTS(MTITLE) NE 0 THEN MTITL = MTITLE ELSE MTITL = ''
WIDS.HC_BASE = WIDGET_BASE(TITLE = MTITL, /COLUMN)
            
 IF NOT KEYWORD_SET(NODONE) THEN BEGIN
    HC_BUT1 = WIDGET_BUTTON(WIDS.HC_BASE, VALUE = 'DONE')
 ENDIF

 IF NOT KEYWORD_SET(NOINQUIRE) THEN BEGIN
    HC_BUT2   = WIDGET_BUTTON(WIDS.HC_BASE, VALUE = 'Create Hardcopy')
 ENDIF
                                 
 IF KEYWORD_SET(ZOOM) THEN BEGIN
    HC_BUT3 = WIDGET_BUTTON(WIDS.HC_BASE, VALUE = 'Zoom')
 ENDIF

 ; Place to add any notes
 IF N_ELEMENTS(WNOTES) NE 0 THEN BEGIN
    NBASE  = WIDGET_BASE(WIDS.HC_BASE, /COLUMN)
    FOR i=0, N_ELEMENTS(WNOTES)-1 DO BEGIN
        WIDS.HC_STATUS = WIDGET_TEXT(NBASE, VALUE = WNOTES(i))
    ENDFOR
 ENDIF

 ; The draw widget
 IF KEYWORD_SET(BIG) THEN BEGIN
    XFACTOR = 0.75
    YFACTOR = 0.75
 ENDIF ELSE BEGIN
    XFACTOR = 0.50
    YFACTOR = 0.50
 ENDELSE
 WIDS.HC_DRAW = WIDGET_DRAW(WIDS.HC_BASE, XSIZE = CURRENT_SCREEN(0)*XFACTOR, $
                            YSIZE = CURRENT_SCREEN(1)*YFACTOR, /FRAME)

; Realize the BASE widget and the children
WIDGET_CONTROL, WIDS.HC_BASE, /REALIZE, SHOW = 1 

; Place the cursor on DONE button, if it exists
IF NOT KEYWORD_SET(NODONE) THEN BEGIN
   WIDGET_CONTROL, HC_BUT1, /INPUT_FOCUS
ENDIF  

; Get the draw widget window number
WIDGET_CONTROL, GET_VALUE = WPLOT_WINDOW, WIDS.HC_DRAW
WSET, WPLOT_WINDOW



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Slider widget 
FUNCTION W_SLIDER, XR, LABEL=LABEL
; XR is a two element array giving the range for slider
; SVALUE is the returned value


S_BASE = WIDGET_BASE(/COLUMN)

IF N_ELEMENTS(LABEL) NE 0 THEN SLABEL=LABEL ELSE SLABEL=''
STEXT = WIDGET_LABEL(S_BASE, VALUE='Select ' + SLABEL)

MIDDLE  = ((XR(1) - XR(0)) / 2.0) + XR(0)
WSLIDER = WIDGET_SLIDER(S_BASE, TITLE='', $
                        MIN=XR(0), MAX=XR(1), $
                        UVALUE='WSLIDER', VALUE=MIDDLE)

WIDGET_CONTROL, S_BASE, /REALIZE

EVENT = WIDGET_EVENT(S_BASE)

TYPE = TAG_NAMES(EVENT, /STRUCTURE)
WIDGET_CONTROL, EVENT.ID, GET_VALUE=VALUE, GET_UVALUE=UVALUE

CASE UVALUE OF 
  'WSLIDER': BEGIN
      SVALUE = VALUE
      WIDGET_CONTROL, EVENT.TOP, /DESTROY
      END
ENDCASE


RETURN, SVALUE
END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; WINGSPAN LABELS WIDGET
;  Used under the main widget to manipulate plot labels
 
; This is the event handler 
PRO LABEL_BASE_EVENT, EVENT

@wgsp_cb.pro


WIDGET_CONTROL, EVENT.TOP, GET_UVALUE = LABEL_IDS

WIDGET_CONTROL, LABEL_IDS.SET_LABEL, SET_VALUE = ''


TYPE = TAG_NAMES(Event, /STRUCTURE)
CASE TYPE OF

 ; The text widget events
 'WIDGET_TEXT_CH': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, GET_UVALUE = UVALUE
                   
    CASE UVALUE OF 
         'TIME_LABEL': BEGIN
           TIME_LABEL = VALUE
           CURRENT_VALUES(3) = TIME_LABEL
           WM_UPDATE
           END
         'ENERGY_LABEL': BEGIN
           ENERGY_LABEL = VALUE
           CURRENT_VALUES(4) = ENERGY_LABEL
           WM_UPDATE
           END
         'RATE_LABEL': BEGIN
           RATE_LABEL = VALUE
           CURRENT_VALUES(5) = RATE_LABEL
           WM_UPDATE
           END
         'PLOT_TITLE': BEGIN
           PLOT_TITLE = VALUE
           CURRENT_VALUES(8) = PLOT_TITLE
           END
         'ANNOTATE_BH': BEGIN
           ANNOTATE_BH = VALUE
           WM_UPDATE
           END
         'ANNOTATE_BS': BEGIN
           ANNOTATE_BS = VALUE
           WM_UPDATE
           END
         'ANNOTATE_SIZE': BEGIN
           ANNOTATE_SIZE = FLOAT(VALUE)
           ANNOTATE_SIZE = ANNOTATE_SIZE(0) 
           WM_UPDATE
           END
         'GLOBAL_CHARACTER_SIZE': BEGIN
           VALUE = VALUE(0)
           !P.CHARSIZE = FLOAT(VALUE)
           GLOBAL_CHARSIZE = FLOAT(VALUE)
           WM_UPDATE
           END
         'GLOBAL_CHARACTER_THICK': BEGIN
           VALUE = VALUE(0)
           !P.CHARTHICK = FLOAT(VALUE)
           GLOBAL_CHARTHICK = FLOAT(VALUE)
           WM_UPDATE
           END
         'GLOBAL_TICK_LENGTH': BEGIN
           VALUE = VALUE(0)
           !X.TICKLEN = FLOAT(VALUE)
           !Y.TICKLEN = FLOAT(VALUE)
           GLOBAL_TICKLEN = FLOAT(VALUE)
           ; Set the gridstyle, if needed
           IF (GLOBAL_TICKLEN GE 1.0) THEN BEGIN
              !X.GRIDSTYLE = 1 & !Y.GRIDSTYLE = 1
           ENDIF ELSE BEGIN
              !X.GRIDSTYLE = 0 & !Y.GRIDSTYLE = 0
           ENDELSE
           WM_UPDATE
           END
         'GLOBAL_LINE_THICK': BEGIN
           VALUE = VALUE(0)
           !P.THICK = FLOAT(VALUE)
           !X.THICK = FLOAT(VALUE)
           !Y.THICK = FLOAT(VALUE)
           GLOBAL_LINETHICK = FLOAT(VALUE)
           WM_UPDATE
           END
    ENDCASE ; for UVALUE
    END

 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE

    CASE STRUPCASE(VALUE) OF 
         'DONE': WIDGET_CONTROL, EVENT.TOP, /DESTROY

         'SAVE SET 1': BEGIN
             OPENW, FL, WGSP_SCRATCH + $
                    STRLOWCASE('WGSP_PLOT_PARAMS.SET1'), /GET_LUN, ERROR = EC
             IF (EC NE 0) THEN BEGIN
                W = WIDMENU([!ERR_STRING, 'OK'], TITLE = 0)
                RETURN
             ENDIF
             PRINTF, FL, CURRENT_VALUES(3)
             PRINTF, FL, CURRENT_VALUES(4)
             PRINTF, FL, CURRENT_VALUES(5)
             PRINTF, FL, CURRENT_VALUES(8)
             PRINTF, FL, ANNOTATE_BH
             PRINTF, FL, ANNOTATE_BS
             PRINTF, FL, ANNOTATE_SIZE
             PRINTF, FL, GLOBAL_CHARSIZE
             PRINTF, FL, GLOBAL_CHARTHICK
             PRINTF, FL, GLOBAL_TICKLEN
             PRINTF, FL, GLOBAL_LINETHICK
             FREE_LUN, FL

             WIDGET_CONTROL, LABEL_IDS.SET_LABEL, SET_VALUE = 'Saved Set 1'
             END

         'SAVE SET 2': BEGIN
             OPENW, FL, WGSP_SCRATCH + $
                    STRLOWCASE('WGSP_PLOT_PARAMS.SET2'), /GET_LUN, ERROR = EC
             IF (EC NE 0) THEN BEGIN
                W = WIDMENU([!ERR_STRING, 'OK'], TITLE = 0)
                RETURN
             ENDIF
             PRINTF, FL, CURRENT_VALUES(3)
             PRINTF, FL, CURRENT_VALUES(4)
             PRINTF, FL, CURRENT_VALUES(5)
             PRINTF, FL, CURRENT_VALUES(8)
             PRINTF, FL, ANNOTATE_BH
             PRINTF, FL, ANNOTATE_BS
             PRINTF, FL, ANNOTATE_SIZE
             PRINTF, FL, GLOBAL_CHARSIZE
             PRINTF, FL, GLOBAL_CHARTHICK
             PRINTF, FL, GLOBAL_TICKLEN
             PRINTF, FL, GLOBAL_LINETHICK
             FREE_LUN, FL

             WIDGET_CONTROL, LABEL_IDS.SET_LABEL, SET_VALUE = 'Saved Set 2'
             END

         'RESTORE SET 1': BEGIN
             OPENR, FL, WGSP_SCRATCH + $
                    STRLOWCASE('WGSP_PLOT_PARAMS.SET1'), /GET_LUN, ERROR = EC
             IF (EC NE 0) THEN BEGIN
                W = WIDMENU([!ERR_STRING, 'OK'], TITLE = 0)
                RETURN
             ENDIF
             TEMP = ''
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_TIME, SET_VALUE = TEMP
              CURRENT_VALUES(3) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ENERGY, SET_VALUE = TEMP
              CURRENT_VALUES(4) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_RATE, SET_VALUE = TEMP
              CURRENT_VALUES(5) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_PLOT_TITLE, SET_VALUE = TEMP
              CURRENT_VALUES(8) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_BH, SET_VALUE = TEMP
              ANNOTATE_BH = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_BS, SET_VALUE = TEMP
              ANNOTATE_BS = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_SIZE, SET_VALUE = TEMP
              ANNOTATE_SIZE = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_CHARSIZE, SET_VALUE = TEMP
              GLOBAL_CHARSIZE = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_CHARTHICK, SET_VALUE = TEMP
              GLOBAL_CHARTHICK = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_TICKL, SET_VALUE = TEMP
              GLOBAL_TICKLEN = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_LINETHICK, SET_VALUE = TEMP
              GLOBAL_LINETHICK = FLOAT(TEMP)
             FREE_LUN, FL             

             !P.CHARSIZE  = GLOBAL_CHARSIZE
             !P.CHARTHICK = GLOBAL_CHARTHICK
             !X.TICKLEN   = GLOBAL_TICKLEN
             !Y.TICKLEN   = GLOBAL_TICKLEN
             !P.THICK     = GLOBAL_LINETHICK
             !X.THICK     = GLOBAL_LINETHICK
             !Y.THICK     = GLOBAL_LINETHICK

             WM_UPDATE
             WIDGET_CONTROL, LABEL_IDS.SET_LABEL, SET_VALUE = 'Restored Set 1'
             END

         'RESTORE SET 2': BEGIN
             OPENR, FL, WGSP_SCRATCH + $
                        STRLOWCASE('WGSP_PLOT_PARAMS.SET2'), /GET_LUN, ERROR = EC
             IF (EC NE 0) THEN BEGIN
                W = WIDMENU([!ERR_STRING, 'OK'], TITLE = 0)
                RETURN
             ENDIF
             TEMP = ''
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_TIME, SET_VALUE = TEMP
              CURRENT_VALUES(3) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ENERGY, SET_VALUE = TEMP
              CURRENT_VALUES(4) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_RATE, SET_VALUE = TEMP
              CURRENT_VALUES(5) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_PLOT_TITLE, SET_VALUE = TEMP
              CURRENT_VALUES(8) = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_BH, SET_VALUE = TEMP
              ANNOTATE_BH = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_BS, SET_VALUE = TEMP
              ANNOTATE_BS = TEMP
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_ANN_SIZE, SET_VALUE = TEMP
              ANNOTATE_SIZE = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_CHARSIZE, SET_VALUE = TEMP
              GLOBAL_CHARSIZE = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_CHARTHICK, SET_VALUE = TEMP
              GLOBAL_CHARTHICK = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_TICKL, SET_VALUE = TEMP
              GLOBAL_TICKLEN = FLOAT(TEMP)
             READF, FL, TEMP
              WIDGET_CONTROL, LABEL_IDS.L_GLOBAL_LINETHICK, SET_VALUE = TEMP
              GLOBAL_LINETHICK = FLOAT(TEMP)
             FREE_LUN, FL

             !P.CHARSIZE  = GLOBAL_CHARSIZE
             !P.CHARTHICK = GLOBAL_CHARTHICK
             !X.TICKLEN   = GLOBAL_TICKLEN
             !Y.TICKLEN   = GLOBAL_TICKLEN
             !P.THICK     = GLOBAL_LINETHICK
             !X.THICK     = GLOBAL_LINETHICK
             !Y.THICK     = GLOBAL_LINETHICK

             WM_UPDATE
             WIDGET_CONTROL, LABEL_IDS.SET_LABEL, SET_VALUE = 'Restored Set 2'
             END
         ELSE:
    ENDCASE
    END

 ; The radio button widget events
 '': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE
    CASE UVALUE OF
     'NBUTTONS': BEGIN
      CASE EVENT.VALUE OF
       0: WGSP_SUPPRESS_NOTES = 0
       1: WGSP_SUPPRESS_NOTES = 1
      ENDCASE
      END
    ENDCASE
    END

ENDCASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO W_LABEL

@wgsp_cb.pro


DEVICE, RETAIN = CV_IDL_RETAIN

IF (XREGISTERED('W_LABEL')) THEN BEGIN
 ; Because of COMMON BLOCKS, only one instance of the widget is allowed
 INFORM, 'W_LABEL already running.'
 RETURN
ENDIF

LABEL_IDS = {L_TIME: 0L, L_ENERGY: 0L, L_RATE: 0L, $
             L_PLOT_TITLE: 0L, L_ANN_BH: 0L, L_ANN_BS: 0L, $
             L_ANN_SIZE: 0L, L_GLOBAL_CHARSIZE: 0L, L_GLOBAL_CHARTHICK: 0L, $
             L_GLOBAL_TICKL: 0L, L_GLOBAL_LINETHICK: 0L, $
             SET_LABEL: 0L}

; Parent
WTITLE = 'PLOT LABELS / OPTIONS'
LABEL_BASE = WIDGET_BASE(TITLE = WTITLE, /COLUMN)

; DONE button
DONEB = WIDGET_BUTTON(LABEL_BASE, VALUE = 'DONE', /FRAME)

THE_LABELS = [ 'Time Label', $
               'Energy Label', $
               'Rate Label', $
               'Plot Title', $
               'Annotate Burst History', $
               'Annotate Burst Spectrum', $
               'Annotate Size', $
               'Global Character Size', $
               'Global Character Thickness', $
               'Global Tick Length', $
               'Global Line Thickness', $
               'Suppress Plot Notes']


; The labels
INDEX   = 0
L_BASE  = WIDGET_BASE(LABEL_BASE, /FRAME, /COLUMN)
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_TIME = WIDGET_TEXT(DUMMY, VALUE = CURRENT_VALUES(3), $
                               UVALUE = 'TIME_LABEL', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_ENERGY = WIDGET_TEXT(DUMMY, VALUE = CURRENT_VALUES(4), $
                                 UVALUE = 'ENERGY_LABEL', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_RATE  = WIDGET_TEXT(DUMMY, VALUE = CURRENT_VALUES(5), $
                                UVALUE = 'RATE_LABEL', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX)) 
LABEL_IDS.L_PLOT_TITLE = WIDGET_TEXT(DUMMY, VALUE = CURRENT_VALUES(8), $
                                     UVALUE = 'PLOT_TITLE', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
IF (N_ELEMENTS(ANNOTATE_BH) EQ 0) THEN $
   BH_VALUE = ' ' ELSE BH_VALUE = ANNOTATE_BH
LABEL_IDS.L_ANN_BH = WIDGET_TEXT(DUMMY, VALUE = BH_VALUE, $
                                 UVALUE = 'ANNOTATE_BH', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
IF (N_ELEMENTS(ANNOTATE_BS) EQ 0) THEN $
   BS_VALUE = '' ELSE BS_VALUE = ANNOTATE_BS
LABEL_IDS.L_ANN_BS = WIDGET_TEXT(DUMMY, VALUE = BS_VALUE, $
                                 UVALUE = 'ANNOTATE_BS', YSIZE = 1, /EDIT)

INDEX    = INDEX + 1
DUMMY    = WIDGET_BASE(L_BASE, /ROW) 
L_LABEL  = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
AS_VALUE = ANNOTATE_SIZE
LABEL_IDS.L_ANN_SIZE = WIDGET_TEXT(DUMMY, $
                        VALUE = STRTRIM(STRING(AS_VALUE, F='(F5.2)'), 2), $
                        UVALUE = 'ANNOTATE_SIZE', YSIZE = 1, /EDIT)

; Some global parameters
INDEX     = INDEX + 1
DUMMY_BOX = WIDGET_BASE(LABEL_BASE, /COLUMN, /FRAME)
DUMMY     = WIDGET_BASE(DUMMY_BOX, /ROW) 
L_LABEL   = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_GLOBAL_CHARSIZE = WIDGET_TEXT(DUMMY, $
                    VALUE = STRTRIM(STRING(GLOBAL_CHARSIZE, F='(F5.2)'), 2), $
                    UVALUE = 'GLOBAL_CHARACTER_SIZE', YSIZE = 1, /EDIT)
                     
INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(DUMMY_BOX, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_GLOBAL_CHARTHICK = WIDGET_TEXT(DUMMY, $
                     VALUE = STRTRIM(STRING(GLOBAL_CHARTHICK, F='(F5.2)'), 2), $
                     UVALUE = 'GLOBAL_CHARACTER_THICK', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(DUMMY_BOX, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_GLOBAL_TICKL = WIDGET_TEXT(DUMMY, $
                     VALUE = STRTRIM(STRING(GLOBAL_TICKLEN, F='(F5.2)'), 2), $
                     UVALUE = 'GLOBAL_TICK_LENGTH', YSIZE = 1, /EDIT)

INDEX   = INDEX + 1
DUMMY   = WIDGET_BASE(DUMMY_BOX, /ROW) 
L_LABEL = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
LABEL_IDS.L_GLOBAL_LINETHICK = WIDGET_TEXT(DUMMY, $
                     VALUE = STRTRIM(STRING(GLOBAL_LINETHICK, F='(F5.2)'), 2), $
                     UVALUE = 'GLOBAL_LINE_THICK', YSIZE = 1, /EDIT)

INDEX     = INDEX + 1
DUMMY_BOX = WIDGET_BASE(LABEL_BASE, /COLUMN, /FRAME)
DUMMY     = WIDGET_BASE(DUMMY_BOX, /ROW) 
L_LABEL   = WIDGET_LABEL(DUMMY, VALUE = THE_LABELS(INDEX))
BUTTON_LABELS = ['No', 'Yes']
L_SUPPRESS_NOTES = CW_BGROUP(DUMMY, BUTTON_LABELS, $
                     /ROW, /EXCLUSIVE, /NO_RELEASE, UVALUE = 'NBUTTONS')
WIDGET_CONTROL, L_SUPPRESS_NOTES, SET_VALUE = WGSP_SUPPRESS_NOTES


ROW_BASE = WIDGET_BASE(LABEL_BASE, /ROW)

LBASE    = WIDGET_BASE(ROW_BASE, /COLUMN)
BUTTON   = WIDGET_BUTTON(LBASE, VALUE = 'Save Set 1')
BUTTON   = WIDGET_BUTTON(LBASE, VALUE = 'Save Set 2')

RBASE    = WIDGET_BASE(ROW_BASE, /COLUMN)
BUTTON   = WIDGET_BUTTON(RBASE, VALUE = 'Restore Set 1')
BUTTON   = WIDGET_BUTTON(RBASE, VALUE = 'Restore Set 2')

RBASE    = WIDGET_BASE(ROW_BASE, /COLUMN)
LABEL_IDS.SET_LABEL = WIDGET_LABEL(RBASE, VALUE = '')


; To pass the widget IDs
WIDGET_CONTROL, LABEL_BASE, SET_UVALUE = LABEL_IDS

; Realize the BASE widget and the children
WIDGET_CONTROL, LABEL_BASE, /REALIZE, SHOW = 1 


; Register with XMANAGER
XMANAGER, 'LABEL_BASE', LABEL_BASE, /MODAL




END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; WINGSPAN TEXT WIDGET
;  Used to display text
 
; This is the event handler 
PRO W_TEXT_BASE_EVENT, EVENT


TYPE = TAG_NAMES(Event, /STRUCTURE)
CASE TYPE OF

 ; The button widget events
 'WIDGET_BUTTON': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE=VALUE

    CASE VALUE OF 
         'DONE': WIDGET_CONTROL, EVENT.TOP, /DESTROY
         ELSE:
    ENDCASE
    END

ENDCASE



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO W_TEXT, TEXT, TITLE=TITLE


DEVICE, RETAIN = 2

; Parent
IF N_ELEMENTS(TITLE) EQ 0 THEN WTITLE = '' ELSE WTITLE = TITLE
W_TEXT_BASE = WIDGET_BASE(TITLE=WTITLE, /COLUMN, $
		SPACE = 20, XPAD = 20, YPAD = 20)
            
 ; DONE button
 DUMMY = WIDGET_BUTTON(W_TEXT_BASE, VALUE = 'DONE', /FRAME)

 ; Text
 DUMMY = WIDGET_TEXT(W_TEXT_BASE, VALUE = TEXT, XSIZE = 80, YSIZE = 24, /SCROLL)



; Realize the BASE widget and the children
WIDGET_CONTROL, W_TEXT_BASE, /REALIZE, SHOW = 1 


; Register with XMANAGER
XMANAGER, 'W_TEXT_BASE', W_TEXT_BASE, /MODAL



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
pro break_file, file, disk_log, dir, filnam, ext, fversion, node
;
;+
;NAME:
;	break_file
;PURPOSE:
;	Given a file name, break the filename into the parts
;	of disk/logical, the directory, the filename, the
;	extension, and the file version (for VMS)
;INPUT:
;	file	- The file name
;OUTPUT:
;	disk_log- The disk or logical (looks for a ":")
;		  This is generally only valid on VMS machines
;	dir	- The directory
;	filnam	- The filename (excluding the ".")
;	ext	- The filename extension (including the ".")
;	fversion- The file version (only VMS)
;	node	- The Node name (only VMS)
;RESTRICTIONS:
;	VMS:
;		Assumes that : always precedes []
;	ULTRIX:
;		Right now it has trouble with the ultrix option of use
;		of "." or ".."
;HISTORY:
;	Written 1988 by M.Morrison
;	   Aug-91 (MDM) Changed to handle Unix filename convensions
;	28-Feb-92 (MDM) * Adjusted to handle arrays
;	11-Mar-92 (MDM) - Perform a STRTRIM(x,2) on input string before
;			  doing the "break-up"
;	 1-Dec-92 (MDM) - Moved code to do filename, extension and version
;			  number for both VMS and Unix (previously it
;			  did not do version number code for Unix)
;       29-Jan-93 (DMZ/MDM) - checked for node in file name
;       11-Nov-83 (DMZ) - checked for ".]["  and "[000000" in VMS concealed directory names
;       5-Apr-95  (RSM) - didn't work for Windows.  Easiest fix was to put old
;                         code in an IF block, and then add the Windows code:
;                            IF (!VERSION.OS NE 'WINDOWS') OR $
;                               (!VERSION.OS NE 'WIN32') OR THEN BEGIN
;                               ...old code...
;                            ENDIF ELSE BEGIN
;                               ...new Windows code...
;                            ENDELSE
;       12-Jul-96  (RSM) - Good old IDL changed the !VERSION.OS string for
;                          Window95 to 'Win32'.  Added this to the IF block
;-

IF ((STRUPCASE(!VERSION.OS) NE 'WINDOWS') AND $
    (STRUPCASE(!VERSION.OS) NE 'WIN32')) THEN BEGIN

   qvms = 1
   ;dummy is where filename has /
   dummy = where(strpos(file, '/') ne -1, count)		

   ;if there is a /, then count ne 0, and therefore 
   ; it is not VMS (lots of negatives there)
   if (count ne 0) then qvms = 0		

   n = n_elements(file)
   node     = strarr(n)
   disk_log = strarr(n)
   dir      = strarr(n)
   filnam   = strarr(n)
   ext      = strarr(n)
   fversion = strarr(n)

   for ifil=0,n-1 do begin
       file0 = file(ifil)
       file0 = strtrim(file0, 2)		;MDM added 11-Mar-92
       len=strlen(file0)
    
       ;-- node name present    ;DMZ added Jan'93
       ;  (if so then strip it off now and then add it back later)
       dcolon=strpos(file0,'::')
       if dcolon gt -1 then begin
       	  node(ifil)=strmid(file0,0,dcolon+2)
          file0=strmid(file0,dcolon+2,1000)
       endif
    
       if (qvms) then begin
          p=strpos(file0,':')
	  if (p ne 1) then disk_log(ifil)=strmid(file0,0,p+1)	;includes :
	  len=len-p+1
	  file0=strmid(file0, p+1, len)

          ;-- check for .][ in dir    ;-- DMZ added Nov'93

          if strpos(file0,'.][') ne -1 then file0=str_replace(file0,'.][','.')

          p=strpos(file0,']')
          if (p ne -1) then dir(ifil)=strmid(file0,0,p+1)         ;includes ]

          ;-- check for .000000 in dir  ;-- DMZ added Nov'93

          temp=dir(ifil)
          if strpos(temp,'.000000') ne -1 then dir(ifil)=str_replace(temp,'.000000','')
          len=len-p+1
          file0=strmid(file0, p+1, len)


       end else begin
          p = 0
	  while (strpos(file0,'/', p+1) ne -1) do p = strpos(file0,'/',p+1)	;find last /
	  dir(ifil) = strmid(file0, 0, p+1)
	  file0 = strmid(file0, p+1, len-(p+1))
       end

       p=strpos(file0,'.')
       if (p eq -1) then begin
	  filnam(ifil) = strmid(file0,0,len) 
	  p=len
       end else filnam(ifil) = strmid(file0,0,p)		 ;not include .
       len=len-p
       file0=strmid(file0, p, len)
    
       p=strpos(file0,';')
       if (p eq -1) then begin
	  ext(ifil) = strmid(file0,0,len) 
	  p=len
       ;includes . but not ;
       end else ext(ifil) = strmid(file0,0,p)			
       len=len-p
       file0=strmid(file0, p, len)
    
       fversion(ifil) = ''
       if (len ne 0) then fversion(ifil) = file0

       ;-- now prefix disk name with node name
       if node(ifil) ne '' then disk_log(ifil)=node(ifil)+disk_log(ifil)
   end

   if (n eq 1) then begin		;turn output into scalars
      disk_log = disk_log(0)
      dir      = dir(0)
      filnam   = filnam(0)
      ext      = ext(0)
      fversion = fversion(0)
      node     = node(0)
   end

ENDIF ELSE BEGIN

   n = n_elements(file)
   node     = strarr(n)
   disk_log = strarr(n)
   dir      = strarr(n)
   filnam   = strarr(n)
   ext      = strarr(n)
   fversion = strarr(n)

   for ifil=0,n-1 do begin
       file0 = file(ifil)
       file0 = STRTRIM(file0, 2)		;MDM added 11-Mar-92
       len = STRLEN(file0)

       COLON = STRPOS(FILE0, ':')
       IF (COLON NE -1) THEN BEGIN
          DISK_LOG(IFIL) = STRMID(FILE0, 0, COLON + 1)
       ENDIF ELSE BEGIN
          COLON = 0
          DISK_LOG(IFIL) = ''
       ENDELSE

       DIR_COUNTER = -1
       FOR J=LEN-1, 0, -1 DO BEGIN
           TEMP_CHAR = STRMID(FILE0, J, 1)
           IF (TEMP_CHAR EQ '\') THEN BEGIN
              DIR_COUNTER = J
              GOTO, FOUND_DIR
           ENDIF
       ENDFOR

FOUND_DIR:

       IF (COLON EQ 0) THEN BEGIN
          DIR(IFIL)    = STRMID(FILE0, COLON, DIR_COUNTER + 1)
          FILNAM(IFIL) = STRMID(FILE0, DIR_COUNTER + 1, 999)
       ENDIF ELSE BEGIN
          DIR(IFIL)    = STRMID(FILE0, COLON + 1, DIR_COUNTER - 1)
          FILNAM(IFIL) = STRMID(FILE0, DIR_COUNTER + 1, 999)
       ENDELSE

       ; Handle wildcard file specifications
       STAR = STRPOS(FILE0, '*')
       IF (STAR NE -1) THEN BEGIN
          DISK_LOG(IFIL) = ''
          DIR(IFIL)      = ''
          FILNAM(IFIL)   = FILE0
       ENDIF

       DOT = STRPOS(FILNAM(IFIL), '.')
       IF (DOT NE -1) THEN BEGIN
          EXT(IFIL)    = STRMID(FILNAM(IFIL), DOT, 999)
          FILNAM(IFIL) = STRMID(FILNAM(IFIL), 0, DOT)
       ENDIF ELSE BEGIN
          EXT(IFIL) = ''
       ENDELSE

   endfor

   if (n eq 1) then begin		;turn output into scalars
      disk_log = disk_log(0)
      dir      = dir(0)
      filnam   = filnam(0)
      ext      = ext(0)
      fversion = fversion(0)
      node     = node(0)
   end

ENDELSE ; Windows, Win32


end
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; Copyright (c) 1991-1993, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.
PRO WM_XDispFile_evt, event

WIDGET_CONTROL, GET_UVALUE = retval, event.id

IF(retval EQ "EXIT") THEN WIDGET_CONTROL, event.top, /DESTROY

END


PRO WM_XDisplayFile, FILENAME, TITLE = TITLE, GROUP = GROUP, WIDTH = WIDTH, $
		HEIGHT = HEIGHT, TEXT = TEXT, FONT = FONT, MODAL = MODAL
;+
; NAME: 
;	WM_XDISPLAYFILE
;
; PURPOSE:
;	Display an ASCII text file using widgets and the widget manager.
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;	XDISPLAYFILE, Filename
;
; INPUTS:
;     Filename:	A scalar string that contains the filename of the file
;		to display.  The filename can include a path to that file.
;
; KEYWORD PARAMETERS:
;	FONT:   The name of the font to use.  If omitted use the default
;		font.
;	GROUP:	The widget ID of the group leader of the widget.  If this 
;		keyword is specified, the death of the group leader results in
;		the death of XDISPLAYFILE.
;
;	HEIGHT:	The number of text lines that the widget should display at one
;		time.  If this keyword is not specified, 24 lines is the 
;		default.
;
;	TEXT:	A string or string array to be displayed in the widget
;		instead of the contents of a file.  This keyword supercedes
;		the FILENAME input parameter.
;
;	TITLE:	A string to use as the widget title rather than the file name 
;		or "XDisplayFile".
;
;	WIDTH:	The number of characters wide the widget should be.  If this
;		keyword is not specified, 80 characters is the default.
;
;       MODAL: Create the widget as a MODAL widget
;
; OUTPUTS:
;	No explicit outputs.  A file viewing widget is created.
;
; SIDE EFFECTS:
;	Triggers the XMANAGER if it is not already in use.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Open a file and create a widget to display its contents.
;
; MODIFICATION HISTORY:
;	Written By Steve Richards, December 1990
;	Graceful error recovery, DMS, Feb, 1992.
;
;       Increased max number of lines to 2000 and changed DONE button
;       title to "Done", R. Mallozzi, Sept, 1994
;
;       Added the MODAL keyword, to invoke a modal widget, 
;       R. Mallozzi, July, 1996
;-

IF(NOT(KEYWORD_SET(TITLE))) THEN TITLE = FILENAME	;use the defaults if
IF(NOT(KEYWORD_SET(HEIGHT))) THEN HEIGHT = 24		;the keywords were not
IF(NOT(KEYWORD_SET(WIDTH))) THEN WIDTH = 80		;passed in

IF(NOT(KEYWORD_SET(TEXT)))THEN BEGIN
  OPENR, unit, FILENAME, /GET_LUN, ERROR=i		;open the file and then
  if i lt 0 then begin		;OK?
	a = [ !err_string, ' Can not display ' + filename]  ;No
  endif else begin
	  a = strarr(2000)				;Maximum # of lines
	  i = 0
	  c = ''
	  while not eof(unit) do begin
		readf,unit,c
		a(i) = c
		i = i + 1
		endwhile
	  a = a(0:i-1)
	  FREE_LUN, unit				;free the file unit.
  endelse
ENDIF ELSE a = TEXT

filebase = WIDGET_BASE(TITLE = TITLE, $			;create the base
		/COLUMN, $
		SPACE = 20, $
		XPAD = 20, $
		YPAD = 20)

filequit = WIDGET_BUTTON(filebase, $			;create a Done Button
;		VALUE = "Done with " + TITLE, $
		VALUE = "Done", $
		UVALUE = "EXIT")

IF n_elements(font) gt 0 then $
 filetext = WIDGET_TEXT(filebase, $			;create a text widget
		XSIZE = WIDTH, $			;to display the file's
		YSIZE = HEIGHT, $			;contents
		/SCROLL, FONT = font, $
		VALUE = a) $
ELSE filetext = WIDGET_TEXT(filebase, $			;create a text widget
		XSIZE = WIDTH, $			;to display the file's
		YSIZE = HEIGHT, $			;contents
		/SCROLL, $
		VALUE = a)


WIDGET_CONTROL, filebase, /REALIZE			;instantiate the widget

IF (KEYWORD_SET(MODAL)) THEN BEGIN
   Xmanager, "WM_XDisplayFile", $			;register it with the
		filebase, $				;widget manager
		GROUP_LEADER = GROUP, $
		EVENT_HANDLER = "WM_XDispFile_evt", $
                /MODAL
ENDIF ELSE BEGIN 
   Xmanager, "WM_XDisplayFile", $			;register it with the
		filebase, $				;widget manager
		GROUP_LEADER = GROUP, $
		EVENT_HANDLER = "WM_XDispFile_evt" 
ENDELSE


END  ;--------------------- procedure XDisplayFile ----------------------------
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
; $Id: xloadct.pro,v 1.4 1993/11/16 23:52:25 dave Exp $

; Copyright (c) 1991-1993, Research Systems, Inc.  All rights reserved.
;       Unauthorized reproduction prohibited.
PRO XLCT_PSAVE			;Save/Restore our plotting state.
;  Swaps our state with the current state each time its called.

COMMON xloadct_com, r0, g0, b0, tfun, state, filename, cps, psave, pnt, $
	top, bot, silent, chop, lock, g_lbl, vbot, vtop, g_slider, $
	gamma, color, use_values, ncolors

tmp = { xlct_psave, win: !d.window, x: !x.s, y: !y.s }

if psave.win ne -1 then wset, psave.win
!x.s = psave.x
!y.s = psave.y
psave = tmp
end

PRO xlct_draw_cps, i, c
COMMON colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
COMMON xloadct_com, r0, g0, b0, tfun, state, filename, cps, psave, pnt, $
	top, bot, silent, chop, lock, g_lbl, vbot, vtop, g_slider, $
	gamma, color, use_values, ncolors


tc = color
if n_elements(c) gt 0 then begin
	tc = c
	if c ne 0 then color = c
	endif

if i(0) eq -1 then j = indgen(n_elements(cps)) else j = i

plots, cps(j), tfun(j), /noclip, color = tc
plots, cps(j), tfun(j), /noclip, psym=6, color = tc
end

PRO xlct_transfer
COMMON xloadct_com, r0, g0, b0, tfun, state, filename, cps, psave, pnt, $
	top, bot, silent, chop, lock, g_lbl, vbot, vtop, g_slider, $
	gamma, color, use_values, ncolors


COMMON colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr

l = lonarr(ncolors)		;Subscripts
m = n_elements(cps)
for i=0, m-2 do begin
	n = cps(i+1)-cps(i)		;Interval
	b = (tfun(i+1)-tfun(i))/float(n)
	l(cps(i)) = findgen(n) * b + tfun(i)
	endfor
l(ncolors-1) = tfun(m-1)		;Last point
if use_values then begin
  r_curr = l(r_orig)
  g_curr = l(g_orig)
  b_curr = l(b_orig)
endif else begin
  r_curr = r_orig(l)
  g_curr = g_orig(l)
  b_curr = b_orig(l)
endelse

tvlct, r_curr, g_curr, b_curr
end

PRO xloadct_event, event
COMMON colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
COMMON xloadct_com, r0, g0, b0, tfun, state, filename, cps, psave, pnt, $
	top, bot, silent, chop, lock, g_lbl, vbot, vtop, g_slider, $
	gamma, color, use_values, ncolors


IF event.id eq state.draw THEN BEGIN	;** PROCESS DRAWABLE EVENTS **
	if event.press ne 0 then begin		;Pressed button?
		dmin = 1.0e8		;Find closest control pnt
		xlct_psave		;Remove old
		p = convert_coord(event.x, event.y, /TO_DATA, /DEVICE)
		xlct_psave		;Restore old
		x = fix(p(0))
		y = fix(p(1))
		for i=0, n_elements(cps)-1 do begin
			d = (p(0)-cps(i))^2 + (p(1)-tfun(i))^2  ; dist ^ 2
			if d lt dmin then begin
				dmin = d
				pnt = i
				endif
			endfor
		return
		endif
	if event.release ne 0 then begin	;Released button?
		pnt = -1
		return
		endif
	if pnt lt 0 then return			;Don't care here...

	xlct_psave				;Remove old
	p = convert_coord(event.x, event.y, /TO_DATA, /DEVICE)	;Coord of mouse
	n = ncolors -1		;Into range....
	m = n_elements(cps)-1
	x = fix(p(0)) > 0 < n
	if pnt eq 0 then x = 0 else $		;1st & last are fixed
	if pnt eq m then x = n else $
	x = x > (cps(pnt-1) + 1) < (cps(pnt+1)-1)  ;Others must be between

	if pnt eq 0 then xlct_draw_cps, [0, 1],0 $  ;Erase old segment
	else if pnt eq m then xlct_draw_cps, [m-1, m],0 $
	else xlct_draw_cps, [pnt-1, pnt, pnt+1],0
	cps(pnt) = x
	tfun(pnt) = fix(p(1)) > 0 < n
	xlct_transfer

	b = r_curr * .3 + g_curr * .586 + b_curr * .114 ;Ntsc colors
	c = max(abs(b-b(0)), j)  ; *** J is color index furthest from 0

	if pnt eq 0 then xlct_draw_cps, [0, 1], j $
	else if pnt eq m then xlct_draw_cps, [m-1, m], j $
	else xlct_draw_cps, [pnt-1, pnt, pnt+1], j

	xlct_psave		;Remove old
	return
	ENDIF

WIDGET_CONTROL, event.id, GET_UVALUE = eventval

abstop = NCOLORS -1

if event.id eq state.name_list then begin
	LOADCT, silent=silent, event.index, FILE=filename, NCOLORS=ncolors
	goto, set_gamma
	ENDIF

CASE eventval OF
    "TOP":    BEGIN
		WIDGET_CONTROL, top, GET_VALUE = vtop
                if lock ne 0 then begin
                        vbot = (vtop - lock) > 0 < 100
                        widget_control, bot, SET_VALUE=vbot
                        endif
                GOTO, set_gamma
              END

    "BOTTOM": BEGIN
		WIDGET_CONTROL, bot, GET_value = vbot
                if lock ne 0 then begin
                        vtop = (vbot + lock) > 0 < 100
                        widget_control, top, SET_VALUE=vtop
                        ENDIF
   set_gamma:
	if use_values then nc = 256 else nc = ncolors
	s = (nc-1)/100.
	x0 = vbot * s
	x1 = vtop * s
	if x0 ne x1 then s = (nc-1.0)/(x1 - x0) else s = 1.0
	int = -s * x0
	if gamma eq 1.0 then s = round(findgen(nc) * s + int > 0.0) $
	else s = ((findgen(nc) * (s/nc) + (int/nc) > 0.0) ^ gamma) * nc
	if chop ne 0 then begin
	    too_high = where(s ge nc, n)
	    if n gt 0 then s(too_high) = 0L
	    endif
	if use_values then begin
	    s = s < 255L
	    r_curr = s(r_orig)
	    g_curr = s(g_orig)
	    b_curr = s(b_orig)
	endif else begin
	    r_curr = r_orig(s)
	    g_curr = g_orig(s)
	    b_curr = b_orig(s)
 	endelse
	tvlct, r_curr, g_curr, b_curr
	ENDCASE

    "GAMMA": BEGIN
                WIDGET_CONTROL, g_slider, GET_VALUE = gamma
                gamma = 10^((gamma/50.) - 1)
                WIDGET_CONTROL, g_lbl, SET_VALUE = $
			STRING(gamma, format='(f6.3)')
		goto, set_gamma
             ENDCASE

    "GANG" : IF event.value eq 0 then lock = 0 else lock = vtop - vbot

    "CHOP" : BEGIN
        chop = event.value
        goto, set_gamma         ;And redraw
        ENDCASE

    "VALUES": BEGIN
	use_values = event.value
	ENDCASE

    "HELP" : XDisplayFile, FILEPATH("xloadct.txt", subdir='help'), $
                TITLE = "Xloadct Help", $
                GROUP = event.top, $
                WIDTH = 55, $
                HEIGHT = 16

    "RESTORE" : BEGIN                   ;Restore the original tables
        r_curr = (r_orig = r0)
        g_curr = (g_orig = g0)
        b_curr = (b_orig = b0)
	tvlct, r_curr, g_curr, b_curr
        ENDCASE

    "OVERWRITE" : BEGIN                 ;overwrite original tables
	r0 = (r_orig = r_curr)
	g0 = (g_orig = g_curr)
	b0 = (b_orig = b_curr)
    reset_all:
        WIDGET_CONTROL, top, SET_VALUE = 100
        WIDGET_CONTROL, bot, SET_VALUE = 0
        WIDGET_CONTROL, g_slider, SET_VALUE = 50
	vbot = 0
	vtop = 100
	gamma = 1.0
	GOTO, set_gamma
	ENDCASE

    "REVERSE" : BEGIN                   ;Reverse the table
        r_orig = reverse(r_orig)
        g_orig = reverse(g_orig)
        b_orig = reverse(b_orig)
        goto, set_gamma                 ;And redraw
        ENDCASE

    "DONE": BEGIN
        WIDGET_CONTROL, event.top, /DESTROY
        r0 = 0 & g0 = 0 & b0 = 0  ;Free common
        ENDCASE

    "NEWBASE": BEGIN
	mode = event.value
	b = ([0, 0, 1])(mode)		;Top base to map: 0 or 1.
	for i=0,1 do WIDGET_CONTROL, state.bases(i), MAP=i eq b
	if b eq 0 then begin		;table or option mode?
	   b = ([2,3,0])(mode)		;bottom base to map (mode eq 0 or 1)
	   for i=2,3 do WIDGET_CONTROL, state.bases(i), MAP=i eq b
	   endif
	if mode eq 2 then begin
	    reset_all = 1
	    xlct_psave			;Save old state
	    plot, [0, ncolors-1], [0, ncolors-1], xstyle=3, $
		ystyle=3, xmargin = [1,1], ymargin=[1,1], ticklen = -0.03, $
		/NODATA, $
		xtickname = replicate(' ', 10), ytickname = replicate(' ', 10)
	    goto, interp_cps
	    endif
	
	ENDCASE

    "TFUNR": BEGIN
     reset_tfun:
	xlct_psave
	xlct_draw_cps, -1, 0	;Erase all
	tfun = cps		;Linear ramp
	goto, interp_cps
	ENDCASE

    "REMCP": BEGIN
	n = n_elements(cps)
	if n gt 2 then begin
	  xlct_psave
	  xlct_draw_cps, -1, 0
	  igap = 0
	  for i=0, n-2 do $
		if (cps(i+1) - cps(i)) lt (cps(igap+1)-cps(igap)) then $
			igap = i
	  keep = where(indgen(n) ne (igap > 1))
	  cps = cps(keep)
	  tfun = tfun(keep)
	  goto, interp_cps
	  ENDIF
	ENDCASE
    "ADDCP": BEGIN
	xlct_psave
	xlct_draw_cps, -1, 0
	igap = 0			;Find largest gap
	for i=0, n_elements(cps)-2 do $
		if (cps(i+1) - cps(i)) gt (cps(igap+1)-cps(igap)) then $
			igap = i
	cps = [ cps(0:igap), (cps(igap)+cps(igap+1))/2, cps(igap+1:*)]
	tfun = [ tfun(0:igap), (tfun(igap)+tfun(igap+1))/2, tfun(igap+1:*)]
      interp_cps:  xlct_draw_cps, -1, color  ;Redraw new
	xlct_transfer
	xlct_psave		;Restore old points
	if n_elements(reset_all) then goto, reset_all
	ENDCASE
ENDCASE

END


;+
; NAME:
;       XLOADCT
; PURPOSE:
;       A graphical interface to the LOADCT user library procedure.
;       XLOADCT displays the current color map and provides
;       an array of buttons, one per availible predefined color
;       table. Using the mouse to press these buttons causes
;       the corresponding color map to be loaded.
; CATEGORY:
;       Widgets
; CALLING SEQUENCE:
;       XLOADCT
; INPUTS:
;       None.
; KEYWORDS:
;	FILE:	If this keyword is set, the file by the given name is used
;		instead of the file colors1.tbl in the IDL directory.  This
;		allows multiple IDL users to have their own color table file.
;       GROUP = The widget ID of the widget that calls XLoadct.  When 
;               this ID is specified, a death of the caller results in a 
;               death of XLoadct
;	NCOLORS = number of colors to use.  Use color indices from 0
;		to the smaller of !D.TABLE_SIZE-1 and NCOLORS-1.
;		Default = !D.TABLE_SIZE = all available colors.
;       PICK_ONE - Normally, XLOADCT remains running until the user
;               presses the "QUIT" button. If PICK_ONE is present and 
;               non-zero, the "QUIT" button is not displayed, and
;               XLOADCT quits after a single selection.
;       SILENT - Normally, no informational message is printed when
;               a color map is loaded. If this keyword is present and
;               zero, this message is printed.
;	USE_CURRENT: If set, use the current color tables, regardless of
;		the contents of the COMMON block COLORS.
; OUTPUTS:
;       None.
; COMMON BLOCKS:
;       None.
; SIDE EFFECTS:
;       One of the predefined color maps may be loaded.
; RESTRICTIONS:
;       This routine uses the LOADCT user library procedure to
;       do the actual work.
; MODIFICATION HISTORY:
;       24, August, 1990, Written by AB, RSI.
;       March 1, 1992  Mark Rivers added Reverse Table to options menu.
;	7/92, DMS, Added new color tables (allows more than 16).
;	9/92, ACY, Add FILE keyword.
;-

PRO XLoadct, PICK_ONE=PICK_ONE, SILENT=silent_f, GROUP = GROUP, FILE=file, $
	USE_CURRENT = use_current, NCOLORS = nc

COMMON colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
COMMON xloadct_com, r0, g0, b0, tfun, state, filename, cps, psave, pnt, $
	top, bot, silent, chop, lock, g_lbl, vbot, vtop, g_slider, $
	gamma, color, use_values, ncolors


IF(XRegistered("xloadct") NE 0) THEN return

values_button = lonarr(2)

IF KEYWORD_SET(SILENT_f) THEN silent = SILENT_F ELSE silent = 1

IF N_ELEMENTS(file) GT 0 THEN filename = file

siz = 256			;Basic width of tool
names = 0
LOADCT, GET_NAMES = names, FILE = file	;Get table names
w_height = 50                   ;Height of ramp
cur_win = !D.WINDOW
lock = 0
chop = 0
vbot = 0
vtop = 100
gamma = 1.0
use_values=0


; Bases:
;  0 = slider base  (stretch bottom, stretch top, gamma)
;  1 = transfer function drawable + buttons
;  2 = color table list
;  3 = options base  (sliders. top, stretch)

state = { bases: lonarr(4), draw: 0L, name_list: 0L }

base = WIDGET_BASE(TITLE="XLoadct", /COLUMN)


show = WIDGET_DRAW(base, YSIZE=w_height, XSIZE=siz, /FRAME, RETAIN = 2)

IF(NOT(KEYWORD_SET(pick_one))) THEN BEGIN
  junk = WIDGET_BASE(base, /ROW)
  done = WIDGET_BUTTON(junk, VALUE=' Done ', UVALUE = "DONE")
  junk1 = WIDGET_BUTTON(junk, VALUE=' Help ', UVALUE = "HELP")

  junk = CW_BGROUP(base, /ROW, /EXCLUSIVE, /NO_REL, $
	['Tables', 'Options', 'Function'], $
	UVALUE='NEWBASE', SET_VALUE=0)

  junk = widget_base(base)
  for i=0,1 do state.bases(i) = WIDGET_BASE(junk, /COLUMN)

  sbase=WIDGET_BASE(state.bases(0), /COLUMN)
  bot = WIDGET_SLIDER(sbase, TITLE = "Stretch Bottom", MINIMUM = 0, $
        MAXIMUM = 100, VALUE = 0, /DRAG, UVALUE = "BOTTOM", xsize=siz)
  top = WIDGET_SLIDER(sbase, TITLE = "Stretch Top", MINIMUM = 0, $
        MAXIMUM = 100, VALUE = 100, /DRAG, UVALUE = "TOP", xsize=siz)
  g_lbl = WIDGET_LABEL(sbase, VALUE = STRING(1.0))
  g_slider = WIDGET_slider(sbase, TITLE = "Gamma Correction", $
        MINIMUM = 0, MAXIMUM = 100, VALUE = 50, UVALUE = "GAMMA", $
        /SUPPRESS_VALUE, /DRAG, xsize=siz)
ENDIF ELSE BEGIN
  done = 0
ENDELSE

junk = WIDGET_BASE(sbase)
for i=2,3 do state.bases(i) = WIDGET_BASE(junk, /COLUMN)
DEVICE, GET_SCREEN = junk
if junk(1) le 768 then junk = 8 else junk = 16
state.name_list = WIDGET_LIST(state.bases(2), VALUE = names, ysize = junk)


;		Drawable for transfer function

junk = WIDGET_BASE(state.bases(1), /COLUMN, /FRAME)
junk1 = WIDGET_BUTTON(junk, VALUE = 'Reset Transfer Function', $
	UVALUE='TFUNR')
junk1 = WIDGET_BUTTON(junk, VALUE='Add Control Point', UVALUE='ADDCP')
junk1 = WIDGET_BUTTON(junk, VALUE='Remove Control Point', UVALUE='REMCP')

state.draw = WIDGET_DRAW(state.bases(1), xsize = siz, ysize = siz, $
	/BUTTON_EVENTS, /MOTION_EVENTS)


 opt_id = state.bases(3)
 junk = CW_BGROUP(opt_id, /ROW, LABEL_LEFT='Sliders:', /EXCLUSIVE, /NO_REL, $
		['Independent', 'Gang'], UVALUE='GANG', SET_VALUE=lock)
 junk = CW_BGROUP(opt_id, /ROW, LABEL_LEFT = 'Top:',  /EXCLUSIVE, /NO_REL, $
		['Clip', 'Chop'], SET_VALUE=chop, UVALUE='CHOP')
 junk = CW_BGROUP(opt_id, /ROW, LABEL_LEFT='Stretch:',  /EXCLUSIVE, /NO_REL, $
		['Indices', 'Intensity'], UVALUE='VALUES', $
		SET_VALUE=use_values)
 junk = WIDGET_BUTTON(opt_id, VALUE='Reverse Table', $
                UVALUE="REVERSE", /NO_REL)
 junk = WIDGET_BUTTON(opt_id, VALUE='REPLACE Original Table', $
		UVALUE = "OVERWRITE", /NO_REL)
 junk = WIDGET_BUTTON(opt_id, VALUE='RESTORE Original Table', $
                UVALUE="RESTORE", /NO_REL)

WIDGET_CONTROL, state.bases(1), MAP=0	;Tfun is not visible
WIDGET_CONTROL, state.bases(3), MAP=0	;options are not visible

WIDGET_CONTROL, base, /REALIZE
WIDGET_CONTROL, state.draw, GET_VALUE=tmp

wset, tmp			;Initial graph
ncolors = !d.table_size
if n_elements(nc) gt 0 then ncolors = ncolors < nc

plot, [0, ncolors-1], [0, ncolors-1], xstyle=3, ystyle=3, $
	xmargin = [1,1], ymargin=[1,1], ticklen = -0.03, /NODATA
psave = { xlct_psave, win: tmp, x: !x.s, y: !y.s }   ;Our initial state

			;If no common, use current colors
IF KEYWORD_SET(use_current) or N_ELEMENTS(r_orig) LE 0 THEN BEGIN
	TVLCT, r_orig, g_orig, b_orig, /GET
	r_curr = r_orig
	b_curr = b_orig
	g_curr = g_orig
	ENDIF

r0 = r_curr             ;Save original colors
g0 = g_curr
b0 = b_curr
color = ncolors-1
cps = [0, color]
tfun = cps
pnt = -1

WIDGET_CONTROL, show, GET_VALUE=show_win
WSET, show_win
TVSCL, BYTSCL(INDGEN(siz) # REPLICATE(1, w_height), top = ncolors-1)
IF(cur_win NE -1) THEN WSET, cur_win

XManager, "xloadct", base, GROUP_LEADER = GROUP


END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO SPLIT, SARR, F1, F2, F3, F4, F5, F6, F7, F8, F9, F10, F11, F12, F13, F14, $
                 F15, F16, F17, F18, F19, F20, F21, F22, F23, F24, F25

NP = N_PARAMS(1) - 1
N_SARR = N_ELEMENTS(SARR)  &  I = 0

N1 = N_ELEMENTS(F1)  &  IF N1 EQ 0 THEN N1 = 1
IF N1 LE 1 THEN F1 = STRTRIM(SARR(I)) ELSE F1 = SARR(I:I+N1-1)  &  I=I+N1
IF NP EQ 1 THEN GOTO, DONE

N2 = N_ELEMENTS(F2)  &  IF N2 EQ 0 THEN N2 = 1
IF N2 LE 1 THEN F2 = STRTRIM(SARR(I)) ELSE F2 = SARR(I:I+N2-1)  &  I=I+N2
IF NP EQ 2 THEN GOTO, DONE

N3 = N_ELEMENTS(F3)  &  IF N3 EQ 0 THEN N3 = 1
IF N3 LE 1 THEN F3 = STRTRIM(SARR(I)) ELSE F3 = SARR(I:I+N3-1)  &  I=I+N3
IF NP EQ 3 THEN GOTO, DONE

N4 = N_ELEMENTS(F4)  &  IF N4 EQ 0 THEN N4 = 1
IF N4 LE 1 THEN F4 = STRTRIM(SARR(I)) ELSE F4 = SARR(I:I+N4-1)  &  I=I+N4
IF NP EQ 4 THEN GOTO, DONE

N5 = N_ELEMENTS(F5)  &  IF N5 EQ 0 THEN N5 = 1
IF N5 LE 1 THEN F5 = STRTRIM(SARR(I)) ELSE F5 = SARR(I:I+N5-1)  &  I=I+N5
IF NP EQ 5 THEN GOTO, DONE

N6 = N_ELEMENTS(F6)  &  IF N6 EQ 0 THEN N6 = 1
IF N6 LE 1 THEN F6 = STRTRIM(SARR(I)) ELSE F6 = SARR(I:I+N6-1)  &  I=I+N6
IF NP EQ 6 THEN GOTO, DONE

N7 = N_ELEMENTS(F7)  &  IF N7 EQ 0 THEN N7 = 1
IF N7 LE 1 THEN F7 = STRTRIM(SARR(I)) ELSE F7 = SARR(I:I+N7-1)  &  I=I+N7
IF NP EQ 7 THEN GOTO, DONE

N8 = N_ELEMENTS(F8)  &  IF N8 EQ 0 THEN N8 = 1
IF N8 LE 1 THEN F8 = STRTRIM(SARR(I)) ELSE F8 = SARR(I:I+N8-1)  &  I=I+N8
IF NP EQ 8 THEN GOTO, DONE

N9 = N_ELEMENTS(F9)  &  IF N9 EQ 0 THEN N9 = 1
IF N9 LE 1 THEN F9 = STRTRIM(SARR(I)) ELSE F9 = SARR(I:I+N9-1)  &  I=I+N9
IF NP EQ 9 THEN GOTO, DONE

N10 = N_ELEMENTS(F10)  &  IF N10 EQ 0 THEN N10 = 1
IF N10 LE 1 THEN F10 = STRTRIM(SARR(I)) ELSE F10 = SARR(I:I+N10-1)  &  I=I+N10
IF NP EQ 10 THEN GOTO, DONE

N11 = N_ELEMENTS(F11)  &  IF N11 EQ 0 THEN N11 = 1
IF N11 LE 1 THEN F11 = STRTRIM(SARR(I)) ELSE F11 = SARR(I:I+N11-1)  &  I=I+N11
IF NP EQ 11 THEN GOTO, DONE

N12 = N_ELEMENTS(F12)  &  IF N12 EQ 0 THEN N12 = 1
IF N12 LE 1 THEN F12 = STRTRIM(SARR(I)) ELSE F12 = SARR(I:I+N12-1)  &  I=I+N12
IF NP EQ 12 THEN GOTO, DONE

N13 = N_ELEMENTS(F13)  &  IF N13 EQ 0 THEN N13 = 1
IF N13 LE 1 THEN F13 = STRTRIM(SARR(I)) ELSE F13 = SARR(I:I+N13-1)  &  I=I+N13
IF NP EQ 13 THEN GOTO, DONE

N14 = N_ELEMENTS(F14)  &  IF N14 EQ 0 THEN N14 = 1
IF N14 LE 1 THEN F14 = STRTRIM(SARR(I)) ELSE F14 = SARR(I:I+N14-1)  &  I=I+N14
IF NP EQ 14 THEN GOTO, DONE

N15 = N_ELEMENTS(F15)  &  IF N15 EQ 0 THEN N15 = 1
IF N15 LE 1 THEN F15 = STRTRIM(SARR(I)) ELSE F15 = SARR(I:I+N15-1)  &  I=I+N15
IF NP EQ 15 THEN GOTO, DONE

N16 = N_ELEMENTS(F16)  &  IF N16 EQ 0 THEN N16 = 1
IF N16 LE 1 THEN F16 = STRTRIM(SARR(I)) ELSE F16 = SARR(I:I+N16-1)  &  I=I+N16
IF NP EQ 16 THEN GOTO, DONE

N17 = N_ELEMENTS(F17)  &  IF N17 EQ 0 THEN N17 = 1
IF N17 LE 1 THEN F17 = STRTRIM(SARR(I)) ELSE F17 = SARR(I:I+N17-1)  &  I=I+N17
IF NP EQ 17 THEN GOTO, DONE

N18 = N_ELEMENTS(F18)  &  IF N18 EQ 0 THEN N18 = 1
IF N18 LE 1 THEN F18 = STRTRIM(SARR(I)) ELSE F18 = SARR(I:I+N18-1)  &  I=I+N18
IF NP EQ 18 THEN GOTO, DONE

N19 = N_ELEMENTS(F19)  &  IF N19 EQ 0 THEN N19 = 1
IF N19 LE 1 THEN F19 = STRTRIM(SARR(I)) ELSE F19 = SARR(I:I+N19-1)  &  I=I+N19
IF NP EQ 19 THEN GOTO, DONE

N20 = N_ELEMENTS(F20)  &  IF N20 EQ 0 THEN N20 = 1
IF N20 LE 1 THEN F20 = STRTRIM(SARR(I)) ELSE F20 = SARR(I:I+N20-1)  &  I=I+N20
IF NP EQ 20 THEN GOTO, DONE

N21 = N_ELEMENTS(F21)  &  IF N21 EQ 0 THEN N21 = 1
IF N21 LE 1 THEN F21 = STRTRIM(SARR(I)) ELSE F21 = SARR(I:I+N21-1)  &  I=I+N21
IF NP EQ 21 THEN GOTO, DONE

N22 = N_ELEMENTS(F22)  &  IF N22 EQ 0 THEN N22 = 1
IF N22 LE 1 THEN F22 = STRTRIM(SARR(I)) ELSE F22 = SARR(I:I+N22-1)  &  I=I+N22
IF NP EQ 22 THEN GOTO, DONE

N23 = N_ELEMENTS(F23)  &  IF N23 EQ 0 THEN N23 = 1
IF N23 LE 1 THEN F23 = STRTRIM(SARR(I)) ELSE F23 = SARR(I:I+N23-1)  &  I=I+N23
IF NP EQ 23 THEN GOTO, DONE

N24 = N_ELEMENTS(F24)  &  IF N24 EQ 0 THEN N24 = 1
IF N24 LE 1 THEN F24 = STRTRIM(SARR(I)) ELSE F24 = SARR(I:I+N24-1)  &  I=I+N24
IF NP EQ 24 THEN GOTO, DONE

N25 = N_ELEMENTS(F25)  &  IF N25 EQ 0 THEN N25 = 1
IF N25 LE 1 THEN F25 = STRTRIM(SARR(I)) ELSE F25 = SARR(I:I+N25-1)  &  I=I+N25
IF NP EQ 25 THEN GOTO, DONE

DONE:
RETURN
END; PROCEDURE SEPARATE
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
FUNCTION CONCATS, LEN, F1, F2, F3, F4, F5, F6, F7, F8, F9, F10, F11, F12, F13, $
                       F14, F15, F16, F17, F18, F19, F20, F21, F22, F23, F24, $
                       F25

NP = N_PARAMS(1) - 1
N1 = N_ELEMENTS(F1)  &  L1 = STRLEN(F1)
N2 = N_ELEMENTS(F2)  &  L2 = STRLEN(F2)
TOTAL = N1 + N2
IF NP EQ 2 THEN GOTO, ASSIGN

N3 = N_ELEMENTS(F3)  &  L3 = STRLEN(F3)  &  TOTAL = TOTAL + N3
IF NP EQ 3 THEN GOTO, ASSIGN

N4 = N_ELEMENTS(F4)  &  L4 = STRLEN(F4)  &  TOTAL = TOTAL + N4
IF NP EQ 4 THEN GOTO, ASSIGN

N5 = N_ELEMENTS(F5)  &  L5 = STRLEN(F5)  &  TOTAL = TOTAL + N5
IF NP EQ 5 THEN GOTO, ASSIGN

N6 = N_ELEMENTS(F6)  &  L6 = STRLEN(F6)  &  TOTAL = TOTAL + N6
IF NP EQ 6 THEN GOTO, ASSIGN

N7 = N_ELEMENTS(F7)  &  L7 = STRLEN(F7)  &  TOTAL = TOTAL + N7
IF NP EQ 7 THEN GOTO, ASSIGN

N8 = N_ELEMENTS(F8)  &  L8 = STRLEN(F8)  &  TOTAL = TOTAL + N8
IF NP EQ 8 THEN GOTO, ASSIGN

N9 = N_ELEMENTS(F9)  &  L9 = STRLEN(F9)  &  TOTAL = TOTAL + N9
IF NP EQ 9 THEN GOTO, ASSIGN

N10 = N_ELEMENTS(F10)  &  L10 = STRLEN(F10)  &  TOTAL = TOTAL + N10
IF NP EQ 10 THEN GOTO, ASSIGN

N11 = N_ELEMENTS(F11)  &  L11 = STRLEN(F11)  &  TOTAL = TOTAL + N11
IF NP EQ 11 THEN GOTO, ASSIGN

N12 = N_ELEMENTS(F12)  &  L12 = STRLEN(F12)  &  TOTAL = TOTAL + N12
IF NP EQ 12 THEN GOTO, ASSIGN

N13 = N_ELEMENTS(F13)  &  L13 = STRLEN(F13)  &  TOTAL = TOTAL + N13
IF NP EQ 13 THEN GOTO, ASSIGN

N14 = N_ELEMENTS(F14)  &  L14 = STRLEN(F14)  &  TOTAL = TOTAL + N14
IF NP EQ 14 THEN GOTO, ASSIGN

N15 = N_ELEMENTS(F15)  &  L15 = STRLEN(F15)  &  TOTAL = TOTAL + N15
IF NP EQ 15 THEN GOTO, ASSIGN

N16 = N_ELEMENTS(F16)  &  L16 = STRLEN(F16)  &  TOTAL = TOTAL + N16
IF NP EQ 16 THEN GOTO, ASSIGN

N17 = N_ELEMENTS(F17)  &  L17 = STRLEN(F17)  &  TOTAL = TOTAL + N17
IF NP EQ 17 THEN GOTO, ASSIGN

N18 = N_ELEMENTS(F18)  &  L18 = STRLEN(F18)  &  TOTAL = TOTAL + N18
IF NP EQ 18 THEN GOTO, ASSIGN

N19 = N_ELEMENTS(F19)  &  L19 = STRLEN(F19)  &  TOTAL = TOTAL + N19
IF NP EQ 19 THEN GOTO, ASSIGN

N20 = N_ELEMENTS(F20)  &  L20 = STRLEN(F20)  &  TOTAL = TOTAL + N20
IF NP EQ 20 THEN GOTO, ASSIGN

N21 = N_ELEMENTS(F21)  &  L21 = STRLEN(F21)  &  TOTAL = TOTAL + N21
IF NP EQ 21 THEN GOTO, ASSIGN

N22 = N_ELEMENTS(F22)  &  L22 = STRLEN(F22)  &  TOTAL = TOTAL + N22
IF NP EQ 22 THEN GOTO, ASSIGN

N23 = N_ELEMENTS(F23)  &  L23 = STRLEN(F23)  &  TOTAL = TOTAL + N23
IF NP EQ 23 THEN GOTO, ASSIGN

N24 = N_ELEMENTS(F24)  &  L24 = STRLEN(F24)  &  TOTAL = TOTAL + N24
IF NP EQ 24 THEN GOTO, ASSIGN

N25 = N_ELEMENTS(F25)  &  L25 = STRLEN(F25)  &  TOTAL = TOTAL + N25
IF NP EQ 25 THEN GOTO, ASSIGN

ASSIGN:  TEMP = STRARR(LEN, TOTAL)
IF N1 GT 1 THEN FOR I = 0, (N1 - 1) DO TEMP(I) = F1(I) ELSE TEMP(0) = F1
IF N2 GT 1 THEN FOR I = 0, (N2 - 1) DO TEMP(I + N1) = F2(I) ELSE TEMP(N1) = F2
IF NP EQ 2 THEN GOTO, DONE

J = N1 + N2
IF N3 GT 1 THEN FOR I = 0, (N3 - 1) DO TEMP(I + J) = F3(I) ELSE TEMP(J) = F3
IF NP EQ 3 THEN GOTO, DONE

J = J + N3
IF N4 GT 1 THEN FOR I = 0, (N4 - 1) DO TEMP(I + J) = F4(I) ELSE TEMP(J) = F4
IF NP EQ 4 THEN GOTO, DONE

J = J + N4
IF N5 GT 1 THEN FOR I = 0, (N5 - 1) DO TEMP(I + J) = F5(I) ELSE TEMP(J) = F5
IF NP EQ 5 THEN GOTO, DONE

J = J + N5
IF N6 GT 1 THEN FOR I = 0, (N6 - 1) DO TEMP(I + J) = F6(I) ELSE TEMP(J) = F6
IF NP EQ 6 THEN GOTO, DONE

J = J + N6
IF N7 GT 1 THEN FOR I = 0, (N7 - 1) DO TEMP(I + J) = F7(I) ELSE TEMP(J) = F7
IF NP EQ 7 THEN GOTO, DONE

J = J + N7
IF N8 GT 1 THEN FOR I = 0, (N8 - 1) DO TEMP(I + J) = F8(I) ELSE TEMP(J) = F8
IF NP EQ 8 THEN GOTO, DONE

J = J + N8
IF N9 GT 1 THEN FOR I = 0, (N9 - 1) DO TEMP(I + J) = F9(I) ELSE TEMP(J) = F9
IF NP EQ 9 THEN GOTO, DONE

J = J + N9
IF N10 GT 1 THEN FOR I = 0, (N10 - 1) DO TEMP(I + J) = F10(I) ELSE TEMP(J) = F10
IF NP EQ 10 THEN GOTO, DONE

J = J + N10
IF N11 GT 1 THEN FOR I = 0, (N11 - 1) DO TEMP(I + J) = F11(I) ELSE TEMP(J) = F11
IF NP EQ 11 THEN GOTO, DONE

J = J + N11
IF N12 GT 1 THEN FOR I = 0, (N12 - 1) DO TEMP(I + J) = F12(I) ELSE TEMP(J) = F12
IF NP EQ 12 THEN GOTO, DONE

J = J + N12
IF N13 GT 1 THEN FOR I = 0, (N13 - 1) DO TEMP(I + J) = F13(I) ELSE TEMP(J) = F13
IF NP EQ 13 THEN GOTO, DONE

J = J + N13
IF N14 GT 1 THEN FOR I = 0, (N14 - 1) DO TEMP(I + J) = F14(I) ELSE TEMP(J) = F14
IF NP EQ 14 THEN GOTO, DONE

J = J + N14
IF N15 GT 1 THEN FOR I = 0, (N15 - 1) DO TEMP(I + J) = F15(I) ELSE TEMP(J) = F15
IF NP EQ 15 THEN GOTO, DONE

J = J + N15
IF N16 GT 1 THEN FOR I = 0, (N16 - 1) DO TEMP(I + J) = F16(I) ELSE TEMP(J) = F16
IF NP EQ 16 THEN GOTO, DONE

J = J + N16
IF N17 GT 1 THEN FOR I = 0, (N17 - 1) DO TEMP(I + J) = F17(I) ELSE TEMP(J) = F17
IF NP EQ 17 THEN GOTO, DONE

J = J + N17
IF N18 GT 1 THEN FOR I = 0, (N18 - 1) DO TEMP(I + J) = F18(I) ELSE TEMP(J) = F18
IF NP EQ 18 THEN GOTO, DONE

J = J + N18
IF N19 GT 1 THEN FOR I = 0, (N19 - 1) DO TEMP(I + J) = F19(I) ELSE TEMP(J) = F19
IF NP EQ 19 THEN GOTO, DONE

J = J + N19
IF N20 GT 1 THEN FOR I = 0, (N20 - 1) DO TEMP(I + J) = F20(I) ELSE TEMP(J) = F20
IF NP EQ 20 THEN GOTO, DONE

J = J + N20
IF N21 GT 1 THEN FOR I = 0, (N21 - 1) DO TEMP(I + J) = F21(I) ELSE TEMP(J) = F21
IF NP EQ 21 THEN GOTO, DONE

J = J + N21
IF N22 GT 1 THEN FOR I = 0, (N22 - 1) DO TEMP(I + J) = F22(I) ELSE TEMP(J) = F22
IF NP EQ 22 THEN GOTO, DONE

J = J + N22
IF N23 GT 1 THEN FOR I = 0, (N23 - 1) DO TEMP(I + J) = F23(I) ELSE TEMP(J) = F23
IF NP EQ 23 THEN GOTO, DONE

J = J + N23
IF N24 GT 1 THEN FOR I = 0, (N24 - 1) DO TEMP(I + J) = F24(I) ELSE TEMP(J) = F24
IF NP EQ 24 THEN GOTO, DONE

J = J + N24
IF N25 GT 1 THEN FOR I = 0, (N25 - 1) DO TEMP(I + J) = F25(I) ELSE TEMP(J) = F25
IF NP EQ 25 THEN GOTO, DONE

J = J + N25
DONE:  RETURN, TEMP
END; FUNCTION CONCATS
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO SELECT_EDGE_EVENT, EVENT

@wgsp_cb.pro
COMMON SR_DATA, VALUE_ID, VAL_INDEX, $
       LNEW_VAL, RNEW_VAL, USE_THRES


TYPE = TAG_NAMES(EVENT, /STRUCTURE)
WIDGET_CONTROL, EVENT.ID, GET_UVALUE = UVALUE

; The button widget events
CASE TYPE OF
  'WIDGET_BUTTON': BEGIN
    IF ((UVALUE NE 'DONE') AND (UVALUE NE 'HELP') AND $
       (UVALUE NE 'CANCEL')) THEN BEGIN
       ; Left edge
       IF STRPOS(UVALUE, 'L') NE -1 THEN BEGIN
          LOLD_VAL = USE_THRES(0, VAL_INDEX(0))
          CASE 1 OF
               UVALUE EQ '-10L'  : VAL_INDEX(0) = VAL_INDEX(0) - 10
               UVALUE EQ '-1L'   : VAL_INDEX(0) = VAL_INDEX(0) - 1
               UVALUE EQ '+1L'   : VAL_INDEX(0) = VAL_INDEX(0) + 1
               UVALUE EQ '+10L'  : VAL_INDEX(0) = VAL_INDEX(0) + 10
               ELSE: VAL_INDEX(0) = VAL_INDEX(0) 
          ENDCASE

          ; Step through the USE_THRES array by the appropriate amount
          ; Past left edge?
          IF (VAL_INDEX(0) LT 0) THEN $
             VAL_INDEX(0) = 0
          ; Past right edge?
          IF (VAL_INDEX(0) GT N_ELEMENTS(USE_THRES(0,*))-1) THEN $
             VAL_INDEX(0) = N_ELEMENTS(USE_THRES(0,*))-1
          LNEW_VAL = USE_THRES(0, VAL_INDEX(0))
          XP     = [LNEW_VAL, LNEW_VAL]
          XP_OLD = [LOLD_VAL, LOLD_VAL]
          IF (!Y.TYPE EQ 1) THEN YP = 10.0^!Y.CRANGE ELSE YP = !Y.CRANGE
          OPLOT, XP_OLD, YP, LINESTY = 1, COLOR = !P.BACKGROUND
          OPLOT, XP, YP, LINESTY = 1

          WIDGET_CONTROL, VALUE_ID(0), $
                 SET_VALUE = STRTRIM(STRING(LNEW_VAL, F = '(F11.3)'), 2)
       ENDIF
       ; Right edge
       IF STRPOS(UVALUE, 'R') NE -1 THEN BEGIN 
          ROLD_VAL = USE_THRES(1, VAL_INDEX(1))
          CASE 1 OF
               UVALUE EQ '-10R'  : VAL_INDEX(1) = VAL_INDEX(1) - 10
               UVALUE EQ '-1R'   : VAL_INDEX(1) = VAL_INDEX(1) - 1
               UVALUE EQ '+1R'   : VAL_INDEX(1) = VAL_INDEX(1) + 1
               UVALUE EQ '+10R'  : VAL_INDEX(1) = VAL_INDEX(1) + 10
               ELSE: VAL_INDEX(1) = VAL_INDEX(1) 
          ENDCASE

          ; Step through the USE_THRES array by the appropriate amount
          IF (VAL_INDEX(1) LT 0) THEN $
             VAL_INDEX(1) = 0
          IF (VAL_INDEX(1) GT N_ELEMENTS(USE_THRES(1,*))-1) THEN $
             VAL_INDEX(1) = N_ELEMENTS(USE_THRES(1,*))-1
          RNEW_VAL = USE_THRES(1, VAL_INDEX(1))
          XP     = [RNEW_VAL, RNEW_VAL]
          XP_OLD = [ROLD_VAL, ROLD_VAL]
          IF (!Y.TYPE EQ 1) THEN YP = 10.0^!Y.CRANGE ELSE YP = !Y.CRANGE
          OPLOT, XP_OLD, YP, LINESTY = 1, COLOR = !P.BACKGROUND
          OPLOT, XP, YP, LINESTY = 1

          WIDGET_CONTROL, VALUE_ID(1), $
                 SET_VALUE = STRTRIM(STRING(RNEW_VAL, F = '(F11.3)'), 2)
       ENDIF
    ENDIF ELSE BEGIN
       CASE UVALUE OF
        'CANCEL': BEGIN
          WIDGET_CONTROL, EVENT.TOP, /DESTROY
          RETURN
          END
        'HELP': BEGIN
          WM_XDISPLAYFILE, $
             WGSP_SOURCE + STRLOWCASE('SELECT_EDGE.HLP'), TITLE = UVALUE
          END
        'DONE': BEGIN
          ; Return the bin centers, not the outer edges
          L_INNER = USE_THRES(1, VAL_INDEX(0))
          LBIN_CENT = (ABS(L_INNER - LNEW_VAL)/2.0) + LNEW_VAL

          R_INNER = USE_THRES(0, VAL_INDEX(1))
          RBIN_CENT = RNEW_VAL - (ABS(R_INNER - RNEW_VAL)/2.0)
          WLO_X = DOUBLE(LBIN_CENT)
          WHI_X = DOUBLE(RBIN_CENT)
          WIDGET_CONTROL, EVENT.TOP, /DESTROY
          END
       ENDCASE
    ENDELSE
    END ; WIDGET_BUTTON

  'WIDGET_TEXT_CH': BEGIN
    WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE

    LOLD_VAL = USE_THRES(0, VAL_INDEX(0))
    ROLD_VAL = USE_THRES(1, VAL_INDEX(1))

    ; Find the nearest bin edge to the input value
    EDGE = GET_SINGLE_EDGE(USE_THRES, VALUE(0), INDEX)

    ; Only one value of VAL_INDEX in this case, but leave it as LONARR(2)

    IF STRPOS(UVALUE, 'L') NE -1 THEN BEGIN
       VAL_INDEX(0) = INDEX
       LNEW_VAL = EDGE
       WIDGET_CONTROL, VALUE_ID(0), $
              SET_VALUE = STRTRIM(STRING(LNEW_VAL, F = '(F11.3)'), 2)
       XP     = [LNEW_VAL, LNEW_VAL]
       XP_OLD = [LOLD_VAL, LOLD_VAL]
       IF (!Y.TYPE EQ 1) THEN YP = 10.0^!Y.CRANGE ELSE YP = !Y.CRANGE
       OPLOT, XP_OLD, YP, LINESTY = 1, COLOR = !P.BACKGROUND
       OPLOT, XP, YP, LINESTY = 1
    ENDIF
    IF STRPOS(UVALUE, 'R') NE -1 THEN BEGIN
       VAL_INDEX(1) = INDEX - 1 
       RNEW_VAL = EDGE
       WIDGET_CONTROL, VALUE_ID(1), $
              SET_VALUE = STRTRIM(STRING(RNEW_VAL, F = '(F11.3)'), 2)
       XP     = [RNEW_VAL, RNEW_VAL]
       XP_OLD = [ROLD_VAL, ROLD_VAL]
       IF (!Y.TYPE EQ 1) THEN YP = 10.0^!Y.CRANGE ELSE YP = !Y.CRANGE
       OPLOT, XP_OLD, YP, LINESTY = 1, COLOR = !P.BACKGROUND
       OPLOT, XP, YP, LINESTY=1
    ENDIF

    END ; WIDGET_TEXT_CH
ENDCASE ; for TYPE

; Set it back:
;DEVICE, SET_GRAPHICS_FUNCTION = OLDG



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO SELECT_EDGE, INPUT_RANGE, DTYPE
; Allows selection of bin edges for background model, etc
; INPUT_RANGE is the independent axis range (normally !X.CRANGE)

@wgsp_cb.pro
COMMON SR_DATA, VALUE_ID, VAL_INDEX, $
       LNEW_VAL, RNEW_VAL, USE_THRES


DEVICE, RETAIN = CV_IDL_RETAIN

IF N_ELEMENTS(DTYPE) NE 0 THEN BEGIN
   CASE DTYPE OF
        'HISTORY' : USE_THRES = COMBINED_TIMES
        'SPECTRUM': USE_THRES = COMBINED_ENERGIES
        ELSE      : USE_THRES = COMBINED_TIMES    ; Default
   ENDCASE
ENDIF ELSE USE_THRES = COMBINED_TIMES             ; Default

; Handle LOG case
IF (!X.TYPE EQ 1) THEN INPUT_RANGE = 10.0^INPUT_RANGE

;  Adjust the initial selection to the nearest edges on the current plot
;LNEW_VAL = GET_SINGLE_EDGE(USE_THRES, INPUT_RANGE(0), INDEX0)
;RNEW_VAL = GET_SINGLE_EDGE(USE_THRES, INPUT_RANGE(1), INDEX1)

; The initial selection
INDICES  = LONARR(2)
NEW_VAL  = NEAREST_EDGE(USE_THRES, INPUT_RANGE, INDICES)
LNEW_VAL = NEW_VAL(0)
RNEW_VAL = NEW_VAL(1)

; Find the nearest bin edge to the initial value
VAL_INDEX = LONARR(2)
VAL_INDEX(0) = INDICES(0)
VAL_INDEX(1) = INDICES(1)

; Parent
SEDGE_BASE = WIDGET_BASE(/COLUMN, TITLE = 'Select Range')
            
D = WIDGET_BUTTON(SEDGE_BASE, VALUE = 'Done', UVALUE = 'DONE')
;C = WIDGET_BUTTON(SEDGE_BASE, VALUE = 'Cancel', UVALUE = 'CANCEL')
H = WIDGET_BUTTON(SEDGE_BASE, VALUE = 'Help', UVALUE = 'HELP')

; -----------------------------------------------------------------------------
SEDGE_BASET = WIDGET_BASE(SEDGE_BASE, /COLUMN, /FRAME)

VALUE_ID  = LONARR(2) ; IDs of the text boxes

L_LABEL_ID  = WIDGET_LABEL(SEDGE_BASET, VALUE = 'Select Left Bin')
VALUE_ID(0) = WIDGET_TEXT(SEDGE_BASET, UVALUE = 'TEXTL', $
                 VALUE = STRTRIM(STRING(LNEW_VAL, F = '(F11.3)'), 2), /EDIT)

; Define some bit map buttons

; Single left arrow
S_ARROW_LEFT = [ $
  [000B, 000B, 000B], [000B, 000B, 000B], [000B, 000B, 000B], $
  [000B, 032B, 000B], [000B, 048B, 000B], [000B, 056B, 000B], $
  [000B, 060B, 000B], [000B, 062B, 000B], [000B, 255B, 001B], $
  [128B, 255B, 001B], [192B, 255B, 001B], [224B, 255B, 001B], $
  [240B, 255B, 001B], [248B, 255B, 001B], [252B, 255B, 001B], $
  [252B, 255B, 001B], [248B, 255B, 001B], [240B, 255B, 001B], $
  [224B, 255B, 001B], [192B, 255B, 001B], [128B, 255B, 001B], $
  [000B, 255B, 001B], [000B, 062B, 000B], [000B, 060B, 000B], $
  [000B, 056B, 000B], [000B, 048B, 000B], [000B, 032B, 000B], $
  [000B, 000B, 000B], [000B, 000B, 000B], [000B, 000B, 000B]  $
  ]
; Single right arrow
S_ARROW_RIGHT = [ $
  [000B, 000B, 000B], [000B, 000B, 000B], [000B, 000B, 000B], $
  [032B, 000B, 000B], [096B, 000B, 000B], [224B, 000B, 000B], $
  [224B, 001B, 000B], [224B, 003B, 000B], [252B, 007B, 000B], $
  [252B, 015B, 000B], [252B, 031B, 000B], [252B, 063B, 000B], $
  [252B, 127B, 000B], [252B, 255B, 000B], [252B, 255B, 001B], $
  [252B, 255B, 001B], [252B, 255B, 000B], [252B, 127B, 000B], $
  [252B, 063B, 000B], [252B, 031B, 000B], [252B, 015B, 000B], $
  [252B, 007B, 000B], [224B, 003B, 000B], [224B, 001B, 000B], $
  [224B, 000B, 000B], [096B, 000B, 000B], [032B, 000B, 000B], $
  [000B, 000B, 000B], [000B, 000B, 000B], [000B, 000B, 000B]  $
  ]
; Double left arrow
D_ARROW_LEFT = [ $
  [000B, 000B, 000B, 000B, 000B], [000B, 000B, 000B, 000B, 000B], $
  [000B, 000B, 000B, 000B, 000B], [000B, 032B, 000B, 064B, 000B], $
  [000B, 048B, 000B, 096B, 000B], [000B, 056B, 000B, 112B, 000B], $
  [000B, 060B, 000B, 120B, 000B], [000B, 062B, 000B, 124B, 000B], $
  [000B, 255B, 001B, 254B, 003B], [128B, 255B, 001B, 255B, 003B], $
  [192B, 255B, 129B, 255B, 003B], [224B, 255B, 193B, 255B, 003B], $
  [240B, 255B, 225B, 255B, 003B], [248B, 255B, 241B, 255B, 003B], $
  [252B, 255B, 249B, 255B, 003B], [252B, 255B, 249B, 255B, 003B], $
  [248B, 255B, 241B, 255B, 003B], [240B, 255B, 225B, 255B, 003B], $
  [224B, 255B, 193B, 255B, 003B], [192B, 255B, 129B, 255B, 003B], $
  [128B, 255B, 001B, 255B, 003B], [000B, 255B, 001B, 254B, 003B], $
  [000B, 062B, 000B, 124B, 000B], [000B, 060B, 000B, 120B, 000B], $
  [000B, 056B, 000B, 112B, 000B], [000B, 048B, 000B, 096B, 000B], $
  [000B, 032B, 000B, 064B, 000B], [000B, 000B, 000B, 000B, 000B], $
  [000B, 000B, 000B, 000B, 000B], [000B, 000B, 000B, 000B, 000B]  $
  ]
; Double right arrow
D_ARROW_RIGHT = [ $
  [000B, 000B, 000B, 000B, 000B], [000B, 000B, 000B, 000B, 000B], $
  [000B, 000B, 000B, 000B, 000B], [032B, 000B, 064B, 000B, 000B], $
  [096B, 000B, 192B, 000B, 000B], [224B, 000B, 192B, 001B, 000B], $
  [224B, 001B, 192B, 003B, 000B], [224B, 003B, 192B, 007B, 000B], $
  [252B, 007B, 248B, 015B, 000B], [252B, 015B, 248B, 031B, 000B], $
  [252B, 031B, 248B, 063B, 000B], [252B, 063B, 248B, 127B, 000B], $
  [252B, 127B, 248B, 255B, 000B], [252B, 255B, 248B, 255B, 001B], $
  [252B, 255B, 249B, 255B, 003B], [252B, 255B, 249B, 255B, 003B], $
  [252B, 255B, 248B, 255B, 001B], [252B, 127B, 248B, 255B, 000B], $
  [252B, 063B, 248B, 127B, 000B], [252B, 031B, 248B, 063B, 000B], $
  [252B, 015B, 248B, 031B, 000B], [252B, 007B, 248B, 015B, 000B], $
  [224B, 003B, 192B, 007B, 000B], [224B, 001B, 192B, 003B, 000B], $
  [224B, 000B, 192B, 001B, 000B], [096B, 000B, 192B, 000B, 000B], $
  [032B, 000B, 064B, 000B, 000B], [000B, 000B, 000B, 000B, 000B], $
  [000B, 000B, 000B, 000B, 000B], [000B, 000B, 000B, 000B, 000B]  $
  ]

BL_BASE = WIDGET_BASE(SEDGE_BASET, /ROW)
BL1 = WIDGET_BUTTON(BL_BASE, VALUE = D_ARROW_LEFT, UVALUE = '-10L')
BL2 = WIDGET_BUTTON(BL_BASE, VALUE = S_ARROW_LEFT, UVALUE = '-1L')
BL3 = WIDGET_LABEL(BL_BASE, VALUE = '  BIN  ', UVALUE = 'BINL')
BL4 = WIDGET_BUTTON(BL_BASE, VALUE = S_ARROW_RIGHT, UVALUE = '+1L')
BL5 = WIDGET_BUTTON(BL_BASE, VALUE = D_ARROW_RIGHT, UVALUE = '+10L')
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
SEDGE_BASEB = WIDGET_BASE(SEDGE_BASE, /COLUMN, /FRAME)

R_LABEL_ID  = WIDGET_LABEL(SEDGE_BASEB, VALUE = 'Select Right Bin')
VALUE_ID(1) = WIDGET_TEXT(SEDGE_BASEB,  UVALUE = 'TEXTR', $
                 VALUE = STRTRIM(STRING(RNEW_VAL, F = '(F11.3)'), 2), /EDIT)

BR_BASE = WIDGET_BASE(SEDGE_BASEB, /ROW)
BR1 = WIDGET_BUTTON(BR_BASE, VALUE = D_ARROW_LEFT, UVALUE = '-10R')
BR2 = WIDGET_BUTTON(BR_BASE, VALUE = S_ARROW_LEFT, UVALUE = '-1R')
BR3 = WIDGET_LABEL(BR_BASE, VALUE = '  BIN  ', UVALUE = 'BINR')
BR4 = WIDGET_BUTTON(BR_BASE, VALUE = S_ARROW_RIGHT, UVALUE = '+1R')
BR5 = WIDGET_BUTTON(BR_BASE, VALUE = D_ARROW_RIGHT, UVALUE = '+10R')
; -----------------------------------------------------------------------------

; Realize the BASE widget and the children
WIDGET_CONTROL, SEDGE_BASE, /REALIZE, SHOW = 1 

XMANAGER, 'SELECT_EDGE', SEDGE_BASE, /MODAL


END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
;+
; NAME:
;	RESET_PLOT
;
; PURPOSE:
;	This procedure resets the default plot parameters. 
;
; CATEGORY:
;	Plotting.
;
; CALLING SEQUENCE:
;	RESET_PLOT [,/X ,/Y ,/P]
;
; INPUTS:
;	None.
;
; OPTIONAL INPUT PARAMETERS:
;	None.
;
; KEYWORD PARAMETERS:
;	None.
;
; OPTIONAL KEYWORD PARAMETERS:
;	If X, Y, or P are set, only those structures will be reset
;
; OUTPUTS:
;	No explicit outputs.
;
; SIDE EFFECTS:
;       Resets plot parameters.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Straightforward.
;
; MODIFICATION HISTORY:
;       RS Mallozzi, Dec, 1994
;-
PRO RESET_PLOT, X = X, Y = Y, P = P

IF (N_ELEMENTS(X) EQ 0) AND $
   (N_ELEMENTS(Y) EQ 0) AND $
   (N_ELEMENTS(P) EQ 0) THEN $
   RESET_ALL = 1 $
ELSE $
   RESET_ALL = 0


; FIRST !P
IF (N_ELEMENTS(P) NE 0) OR (RESET_ALL EQ 1) THEN BEGIN
   !P.BACKGROUND = 0
   !P.CHARSIZE   = 0.000000
   !P.CHARTHICK  = 0.000000
   !P.CLIP       = [0L, 0, 639, 511, 0, 0]
   !P.COLOR      = 255L
   !P.FONT       = -1L
   !P.LINESTYLE  = 0L
   !P.MULTI      = LONARR(5)
   !P.NOCLIP     = 0L
   !P.NOERASE    = 0L
   !P.NSUM       = 0L
   !P.POSITION   = FLTARR(4)
   !P.PSYM       = 0L
   !P.REGION     = FLTARR(4)
   !P.SUBTITLE   = ''
   !P.SYMSIZE    = 0.000000
   !P.T          = FLTARR(4, 4)
   !P.T3D        = 0L
   !P.THICK      = 0.000000
   !P.TITLE      =  ''
   !P.TICKLEN    = 0.0200000
   !P.CHANNEL    = 0L
ENDIF

; NOW !X
IF (N_ELEMENTS(X) NE 0) OR (RESET_ALL EQ 1) THEN BEGIN
   !X.TITLE      = ''
   !X.TYPE       = 0L
   !X.STYLE      = 0L
   !X.TICKS      = 0L
   !X.TICKLEN    = 0.000000
   !X.THICK      = 0.000000
   !X.RANGE      = FLTARR(2)
   !X.CRANGE     = FLTARR(2)
   !X.S          = FLTARR(2)
   !X.MARGIN     = [10.0, 3.0]
   !X.OMARGIN    = FLTARR(2)
   !X.WINDOW     = FLTARR(2)
   !X.REGION     = FLTARR(2)
   !X.CHARSIZE   = 0.000000
   !X.MINOR      = 0L
   !X.TICKV      = FLTARR(30)
   !X.TICKNAME   = STRARR(30)
   !X.GRIDSTYLE  = 0L
   !X.TICKFORMAT = ''
ENDIF

; NOW !Y   
IF (N_ELEMENTS(Y) NE 0) OR (RESET_ALL EQ 1) THEN BEGIN
   !Y.TITLE      = ''
   !Y.TYPE       = 0L
   !Y.STYLE      = 0L
   !Y.TICKS      = 0L
   !Y.TICKLEN    = 0.000000
   !Y.THICK      = 0.000000
   !Y.RANGE      = FLTARR(2)
   !Y.CRANGE     = FLTARR(2)
   !Y.S          = FLTARR(2)
   !Y.MARGIN     = [4.0, 2.0]
   !Y.OMARGIN    = FLTARR(2)
   !Y.WINDOW     = FLTARR(2)
   !Y.REGION     = FLTARR(2)
   !Y.CHARSIZE   = 0.000000
   !Y.MINOR      = 0L
   !Y.TICKV      = FLTARR(30)
   !Y.TICKNAME   = STRARR(30)
   !Y.GRIDSTYLE  = 0L
   !Y.TICKFORMAT = ''
ENDIF


END ; RESET_PLOT
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
PRO EXPAND_PLOT_EVENT, EVENT
; Expands a plot to the full screen. 

@wgsp_cb.pro
; Following COMMON internal to this routine and its event handler
COMMON TR_WINDOW, WIN_SAVE, PSAVE, XSAVE, YSAVE, ASAVE 


TYPE = TAG_NAMES(EVENT, /STRUCTURE)
WIDGET_CONTROL, EVENT.ID, GET_VALUE = VALUE, GET_UVALUE = UVALUE

; The button widget events
CASE (TYPE) OF
  'WIDGET_BUTTON': BEGIN
    CASE (VALUE) OF
         'DONE': BEGIN
                 !P.CHARSIZE = PSAVE 
                 !X.CHARSIZE = XSAVE
                 !Y.CHARSIZE = YSAVE
                 ANNOTATE_SIZE = ASAVE
                 WSET, WIN_SAVE
                 WIDGET_CONTROL, EVENT.TOP, /DESTROY
                 END
    ENDCASE ; for VALUE
    END ; WIDGET_BUTTON

ENDCASE ; for TYPE


END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
PRO EXPAND_PLOT, BUTTON_COMMAND, MFIT_PLOT_OPTION

@wgsp_cb.pro
; Following COMMON internal to this routine and its event handler
COMMON TR_WINDOW, WIN_SAVE, PSAVE, XSAVE, YSAVE, ASAVE 
; Some things need to be stored between calls, so that the 'Display Fit
;  Parameters' functions work
COMMON BATCHFIT_C, BATCH_TIMES, PARAM_HIST, PARAM_UNCER_HIST, RESID_CONTOURS, $
                   NUFNU_CONTOURS, PARAM_STORE, NUM_STORE, HAVE_APLOT, $
                   BATCH_SELECT, ENERGY_SCALE, RATE_SCALE, T_IND 

DEVICE, RETAIN = CV_IDL_RETAIN

WIN_SAVE = !D.WINDOW
PSAVE    = !P.CHARSIZE
XSAVE    = !X.CHARSIZE
YSAVE    = !Y.CHARSIZE
ASAVE    = ANNOTATE_SIZE

!P.CHARSIZE = 1.25
!X.CHARSIZE = 1.25
!Y.CHARSIZE = 1.25
ANNOTATE_SIZE = 2.0

; Parent
EX_BASE = WIDGET_BASE(/COLUMN, TITLE = ' ')
            
D = WIDGET_BUTTON(EX_BASE, VALUE = 'DONE', UVALUE = 'DONE')

DRAW = WIDGET_DRAW(EX_BASE, $
                   XSIZE = CURRENT_SCREEN(0) * 0.99, $
                   YSIZE = CURRENT_SCREEN(1) * 0.93)

; Realize the BASE widget and the children
WIDGET_CONTROL, EX_BASE, /REALIZE, SHOW = 1 

WIDGET_CONTROL, DRAW, GET_VALUE = T_WIN 
WSET, T_WIN

THE_PP = [0.10, 0.28, 0.86, 0.86]
HARD_COPY = 0
CASE MFIT_PLOT_OPTION OF
     ; Option 0 is currently not used
     ; Options 1-6 are for Count Rate, Photon Flux, etc. Handled 
     ;  under ELSE:

     7:  BEGIN
          CHI_1D_PLOT, THE_PP, 2 ;HARD_COPY
         END
     8:  BEGIN
          CHI_2D_PLOT, THE_PP, 2 ;HARD_COPY
         END
     9:  BEGIN
          FIT_GAUSS_PLOT, THE_PP, HARD_COPY
         END
     10: BEGIN
          PPS = [0.1, 0.1, 0.95, 0.95]
          DISPLAY_DRM, PPS, 0
         END
     11: BEGIN
         RESET_LIST = 0
         ; Must check that model is still valid
         IF PARAM_STORE(0) NE PARAM_LABEL_LIST(0) THEN BEGIN
            PARAM_TEMP = PARAM_LABEL_LIST
            NUM_TEMP = NUM_LIST
            PARAM_LABEL_LIST = PARAM_STORE
            NUM_LIST = NUM_STORE
            RESET_LIST = 1
         ENDIF
         B_LABEL = STRTRIM(TOPLABEL, 2)
         CASE WGSP_LAST_PLOT OF
              'PARAMETER TIME HISTORY'    : PARAM_OPTION = 1
              'ONE PARAMETER VS. ANOTHER' : PARAM_OPTION = 2
              'PARAMETER HISTOGRAM'       : PARAM_OPTION = 3
              'SIGMA RESIDUALS MAP'       : PARAM_OPTION = 4
              'MODEL NU-F-NU CONTOURS'    : PARAM_OPTION = 5
         ENDCASE
         PLOT_PARAMS, BATCH_TIMES, $
                      PARAM_HIST, PARAM_UNCER_HIST, B_LABEL, $
                      ENERGY_SCALE, RATE_SCALE, ENERGY_RANGE, RATE_RANGE, $
                      RESID_CONTOURS, NUFNU_CONTOURS, HAVE_APLOT, PARAM_OPTION
    
         IF RESET_LIST THEN BEGIN
            PARAM_LABEL_LIST = PARAM_TEMP
            NUM_LIST = NUM_TEMP
            RESET_LIST = 0
         ENDIF
         END
     12: BEGIN
          PLOT_PARAMS, BATCH_TIMES, $
                       PARAM_HIST, PARAM_UNCER_HIST, STRTRIM(TOPLABEL, 2), $
                       ENERGY_SCALE, RATE_SCALE, ENERGY_RANGE, RATE_RANGE, $
                       RESID_CONTOURS, NUFNU_CONTOURS, HAVE_APLOT, 4
         END
     13: BEGIN 
          PLOT_PARAMS, BATCH_TIMES, $
                       PARAM_HIST, PARAM_UNCER_HIST, STRTRIM(TOPLABEL, 2), $
                       ENERGY_SCALE, RATE_SCALE, ENERGY_RANGE, RATE_RANGE, $
                       RESID_CONTOURS, NUFNU_CONTOURS, HAVE_APLOT, 5
         END
     14: BEGIN
          PPS = !P.POSITION 
          !P.POSITION = [0.1, 0.1, 0.95, 0.95]
          WW_UPDATE, 'HISTORY'
          !P.POSITION = PPS
         END
     15: BEGIN
          PPS = !P.POSITION 
          !P.POSITION = [0.1, 0.1, 0.95, 0.95]
          WW_UPDATE, 'SPECTRUM'
          !P.POSITION = PPS
         END
     ELSE: INIT_MFIT, THE_PP, BUTTON_COMMAND, MFIT_PLOT_OPTION
ENDCASE


XMANAGER, 'EXPAND_PLOT', EX_BASE, /MODAL



END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
 
; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
;+
; NAME:
;       WORDARRAY
; PURPOSE:
;       Convert text string or string array to 1-d array of words.
; CATEGORY:
; CALLING SEQUENCE:
;       wordarray, instring, outlist
; INPUTS:
;       instring = string or string array to process.  in
; KEYWORD PARAMETERS:
;       Keywords:
;         IGNORE = character or array of characters.
;           These characters are removed before processing.
;           Ex: wordarray,in,out,ignore=','
;               wordarray,in,out,ignore=[',',';','(',')']
; OUTPUTS:
;       outlist = 1-d array of words in instring.      out
; COMMON BLOCKS:
; NOTES:
;       Notes: Words are assumed delimited by spaces and/or tabs
;        Non-spaces/tabs are returned as part of the words.
;        Spaces/tabs not needed at the front and end of the strings.
; MODIFICATION HISTORY:
;       R. Sterner, 29 Nov, 1989
;       BLG --- Modified June 22,1991 to include tabs as delimiters
;       R. Sterner, 11 Dec, 1992 --- fixed to handle pure white space.
;       R. Sterner, 27 Jan, 1993 --- dropped reference to array.
;
; Copyright (C) 1989, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;-
;-------------------------------------------------------------
 
	pro wordarray, in0, out, ignore=ign, help=hlp
 
	if (n_params(0) lt 2) or keyword_set(hlp) then begin
	  print,' Convert text string or string array to 1-d array of words.'
	  print,' wordarray, instring, outlist'
	  print,'   instring = string or string array to process.  in'
	  print,'   outlist = 1-d array of words in instring.      out'
	  print,' Keywords:'
	  print,'   IGNORE = character or array of characters.'
	  print,'     These characters are removed before processing.'
	  print,"     Ex: wordarray,in,out,ignore=','"
	  print,"         wordarray,in,out,ignore=[',',';','(',')']"
	  print,' Notes: Words are assumed delimited by spaces and/or tabs'
	  print,'  Non-spaces/tabs are returned as part of the words.'
	  print,'  Spaces/tabs not needed at the front and end of the strings.' 
	  return
	endif
 
	in = in0
 
	if n_elements(ign) gt 0 then begin
	  rm = ign
	  for i = 0L, n_elements(rm)-1L do in = repchr(in, rm(i))
	endif
 
	t = ' ' + in + ' '		; Force spaces on ends
	b = byte(t)				; Convert to byte array.
	w = where(b ne 0, count)		; Find non-null chars.
	if count gt 0 then b = b(w)		; Extract non-null characters.
	X = (B NE 32b) and (B NE 9b)		; non-space/tab chars.
	X = [0,X,0]				; tack 0s at ends.
	if total(x) eq 0 then begin		; All white space.
	  out = ''
	  return
	endif
 
	Y = (X-SHIFT(X,1)) EQ 1			; look for transitions.
	Z = WHERE(SHIFT(Y,-1) EQ 1)
	Y2 = (X-SHIFT(X,-1)) EQ 1
	Z2 = WHERE(SHIFT(Y2,1) EQ 1)
 
	NWDS = TOTAL(Y)				; Total words in IN.
	LOC = Z					; Word start positions.
	LEN = Z2 - Z - 1			; Word lengths.
 
	out = bytarr(max(len), nwds)		; Set up output array.
	if nwds gt 1 then begin
	  for i = 0L, nwds-1L do begin
	    out(0,i) = b(loc(i):(loc(i)+len(i)-1L))
	  endfor
	  out = string(out)
	endif else begin
	  out(0) = b(loc(0):(loc(0)+len(0)-1L))
	  out = string(out)
	endelse
 
 
	return
 
	END
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;+
; Name:
;	MULTIPLOT
; Purpose:
;	This procedure makes a matrix of plots with *SHARED AXES*, either using
;	parameters passed to multiplot or !p.multi in a non-standard way.
;	It is good for data with one or two shared axes and retains all the
;	versatility of the plot commands (e.g. all keywords and log scaling).
;	The plots are connected with the shared axes, which saves space by
;	omitting redundant ticklabels and titles.  Multiplot does this by
;	setting !p.position, !x.tickname and !y.tickname automatically.
;	A call (multiplot,/reset) restores original values.
;
;	Note: This method may be superseded by future improvements in !p.multi
;	by RSI.  For now, it's a good way to gang plots together.
; Examples:
;	multiplot,/help			; print this header.
;	; Then copy & paste, from your xterm, the following lines to test:
;
;	x = findgen(100)		;	   MULTIPLOT
;	t=exp(-(x-50)^2/300)		;	 -------------------------
;	erase				;	 |           |           |
;	u=exp(-x/30)			;	 |           |           |
;	y = sin(x)			;	 |  UL plot  |  UR plot  |
;	r = reverse(y*u)		;	 |           |           |
;	!p.multi=[0,2,2,0,0]		;	 |           |           |
;	multiplot 	 		;	y-------------------------
;	plot,x,y*u,title='MULTIPLOT'	;	l|           |           |
;	multiplot & plot,x,r 		;	a|           |           |
;	multiplot 			;	b|  LL plot  |  LR plot  |
;	plot,x,y*t,ytit='ylabels'	;	e|           |           |
;	multiplot 			;	l|           |           |
;	plot,x,y*t,xtit='xlabels'	;	s-------------------------
;	multiplot,/reset		;		        xlabels
;					 
;	wait,2 & erase			;		 TEST
;	multiplot,[1,3]			;	H------------------------
;	plot,x,y*u,title='TEST'		;	E|	plot #1		|
;	multiplot			;	I------------------------
;	plot,x,y*t,ytit='HEIGHT'	;	G|	plot #2 	|
;	multiplot			;	H------------------------
;	plot,x,r,xtit='PHASE'		;	T|	plot #3		|
;	multiplot,/reset		;	 ------------------------
;					;		 PHASE
;
;	multiplot,[1,1],/init,/verbose	; one way to return to single plot
;	% MULTIPLOT: Initialized for 1x1, plotted across then down (column major).
; Usage:
;	multiplot[pmulti][,/help][,/initialize][,/reset][,/rowmajor]
; Optional Inputs:
;	pmulti = 2-element or 5-element vector giving number of plots, e.g.,
;	  multiplot,[1,6]		; 6 plots vertically
;	  multiplot,[0,4,2,0,0]		; 4 plots along x and 2 along y
;	  multiplot,[0,4,2,0,1]		; ditto, except rowmajor (down 1st)
;	  multiplot,[4,2],/rowmajor 	; identical to previous line
; Optional Keywords:
;	help = flag to print header
;	initialize = flag to begin only---no plotting, just setup,
;	  e.g., multiplot,[4,2],/init,/verbose & multiplot & plot,x,y
;	reset = flag to reset system variables to values prior to /init
;	default = flag to restore IDL's default value for system variables
;	rowmajor = flag to number plots down column first (D=columnmajor)
;	verbose = flag to output informational messages
; Outputs:
;	!p.position = 4-element vector to place a plot
;	!x.tickname = either '' or else 30 ' ' to suppress ticknames
;	!y.tickname = either '' or else 30 ' ' to suppress ticknames
;	!p.noerase = 1
; Common blocks:
;	multiplot---to hold saved variables and plot counter.  See code.
; Side Effects:
;	Multiplot sets a number of system variables: !p.position, !p.multi,
;	!x.tickname, !y.tickname, !P.noerase---but all can be reset with
;	the call: multiplot,/reset
; Restrictions:
;	1. If you use !p.multi as the method of telling how many plots
;	are present, you have to set !p.multi at the beginning each time you
;	use multiplot or call multiplot with the /reset keyword.
;	2. There's no way to make an xtitle or ytitle span more than one plot,
;	except by adding spaces to shift it or to add it manually with xyouts.
;	3. There is no way to make plots of different sizes; each plot
;	covers the same area on the screen or paper.
; Procedure:
;	This routine makes a matrix of plots with common axes, as opposed to
;	the method of !p.multi where axes are separated to allow labels.
;	Here the plots are joined and labels are suppressed, except at the
;	left edge and the bottom.  You tell multiplot how many plots to make
;	using either !p.multi (which is then reset) or the parameter pmulti.
;	However, multiplot keeps track of the position by itself because
;	!p.multi interacts poorly with !p.position.
; Modification history:
;	write, 21-23 Mar 94, Fred Knight (knight@ll.mit.edu)
;	alter plot command that sets !x.window, etc. per suggestion of
;	  Mark Hadfield (hadfield@storm.greta.cri.nz), 7 Apr 94, FKK
;	add a /default keyword restore IDL's default values of system vars,
;	  7 Apr 94, FKK
;	modify two more sys vars !x(y).tickformat to suppress user-formatted
;	  ticknames, per suggestion of Mark Hadfield (qv), 8 Apr 94, FKK
;-
pro multiplot,help=help,pmulti $
  ,initialize=initialize,reset=reset,default=default $
  ,rowmajor=rowmajor,verbose=verbose
;
;	=====>> COMMON
;
common multiplot $
  ,nplots $	; [# of plots along x, # of plots along y]
  ,nleft $	; # of plots remaining---like the first element of !p.multi
  ,pdotmulti $	; saved value of !p.multi
  ,margins $	; calculated margins based on !p.multi or pmulti
  ,pposition $	; saved value of !p.position
  ,colmajor $	; flag for column major order
  ,noerase $	; saved value of !p.noerase
  ,xtickname $	; Original value
  ,ytickname $	; Original value
  ,xtickformat $; Original value
  ,ytickformat 	; Original value
;
;	=====>> HELP
;
;on_error,2
if keyword_set(help) then begin & doc_library,'multiplot' & return & endif
;
;	=====>> RESTORE IDL's DEFAULT VALUES (kill multiplot's influence)
;
if keyword_set(default) then begin
  !p.position = 0
  !x.tickname = ''
  !y.tickname = ''
  !x.tickformat = ''
  !y.tickformat = ''
  !p.multi = 0
  !p.noerase = 0
  nleft = 0
  nplots = [1,1]
  pdotmulti = !p.multi
  margins = 0
  pposition = !p.position
  noerase = !p.noerase
  xtickname = !x.tickname
  ytickname = !y.tickname
  xtickformat = !x.tickformat
  ytickformat = !y.tickformat
  if keyword_set(verbose) then begin
    message,/inform,'Restore IDL''s defaults for affected system variables.'
    message,/inform,'Reset multiplot''s common to IDL''s defaults.'
    endif
  return
  endif
;
;	=====>> RESTORE SAVED SYSTEM VARIABLES
;
if keyword_set(reset) then begin
  if n_elements(pposition) gt 0 then begin
    !p.position = pposition
    !x.tickname = xtickname
    !y.tickname = ytickname
    !x.tickformat = xtickformat
    !y.tickformat = ytickformat
    !p.multi = pdotmulti
    !p.noerase = noerase
    endif
  nleft = 0
  if keyword_set(verbose) then begin
    coords = '['+string(!p.position,form='(3(f4.2,","),f4.2)')+']'
    multi = '['+string(!p.multi,form='(4(i2,","),i2)')+']'
    message,/inform,'Reset.  !p.position='+coords+', !p.multi='+multi
    endif
  return
  endif
;
;	=====>> SETUP: nplots, MARGINS, & SAVED SYSTEM VARIABLES
;
if n_elements(nleft) eq 1 then init = (nleft eq 0) else init = 1
if (n_elements(pmulti) eq 2) or (n_elements(pmulti) eq 5) then init = 1
if (n_elements(!p.multi) eq 5) then begin
  if (!p.multi(1) gt 0) and (!p.multi(2) gt 0) then init = (!p.multi(0) eq 0) 
  endif
if init or keyword_set(initialize) then begin
  case n_elements(pmulti) of
  0:begin
    if n_elements(!p.multi) eq 1 then return	; NOTHING TO SET
    if n_elements(!p.multi) ne 5 then message,'Bogus !p.multi; aborting.'
    nplots = !p.multi(1:2) > 1
    if keyword_set(rowmajor) then colmajor = 0 else colmajor = !p.multi(4) eq 0
    end
  2:begin
    nplots = pmulti
    colmajor = not keyword_set(rowmajor)	; D=colmajor: left to rt 1st
    end
  5:begin
    nplots = pmulti(1:2)
    if keyword_set(rowmajor) then colmajor = 0 else colmajor = pmulti(4) eq 0
    end
  else: message,'pmulti can only have 0, 2, or 5 elements.'
  endcase
  pposition = !p.position			; save sysvar to be altered
  xtickname = !x.tickname
  ytickname = !y.tickname
  xtickformat = !x.tickformat
  ytickformat = !y.tickformat
  pdotmulti = !p.multi
  nleft = nplots(0)*nplots(1)			; total # of plots
  !p.position = 0				; reset
  !p.multi = 0
  plot,/nodata,xstyle=4,ystyle=4,!x.range,!y.range,/noerase	; set window & region
  margins = [min(!x.window)-min(!x.region) $	; in normlized coordinates
    ,min(!y.window)-min(!y.region) $
    ,max(!x.region)-max(!x.window) $
    ,max(!y.region)-max(!y.window)]
  noerase = !p.noerase
  !p.noerase = 1				; !p.multi does the same
  if keyword_set(verbose) then begin
    major = ['across then down (column major).','down then across (row major).']
    if colmajor then index = 0 else index = 1
    message,/inform,'Initialized for '+strtrim(nplots(0),2) $
    +'x'+strtrim(nplots(1),2)+', plotted '+major(index)
    endif
;  print,margins,'=margins'
  if keyword_set(initialize) then return
endif
;
;	=====>> Define the plot region without using !p.multi.
;
cols = nplots(0)			; for convenience
rows = nplots(1)
nleft = nleft - 1			; decrement plots remaining
cur = cols*rows - nleft			; current plot #: 1 to cols*rows
idx = [(1.-margins(0)-margins(2))/cols $
  ,(1.-margins(1)-margins(3))/rows]	; normalized coords per plot
if colmajor then begin			; location in matrix of plots
  col = cur mod cols
  if col eq 0 then col = cols
  row = (cur-1)/cols + 1
endif else begin			; here (1,2) is 1st col, 2nd row
  row = cur mod rows
  if row eq 0 then row = rows
  col = (cur-1)/rows + 1
endelse
pos = [(col-1)*idx(0),(rows-row)*idx(1),col*idx(0),(rows-row+1)*idx(1)] $
  + [margins(0),margins(1),margins(0),margins(1)]
;print,row,col,rows,cols,pos
;
;	=====>> Finally set the system variables; user shouldn't change them.
;
!p.position = pos
onbottom = (row eq rows) or (rows eq 1)
onleft = (col eq 1) or (cols eq 1)
if onbottom then !x.tickname = xtickname else !x.tickname = replicate(' ',30)
if onleft then !y.tickname = ytickname else !y.tickname = replicate(' ',30)
if onbottom then !x.tickformat = xtickformat else !x.tickformat = ''
if onleft then !y.tickformat = ytickformat else !y.tickformat = ''
if keyword_set(verbose) then begin
  coords = '['+string(pos,form='(3(f4.2,","),f4.2)')+']'
  plotno = 'Setup for plot ['+strtrim(col,2)+','+strtrim(row,2)+'] of ' $
    +strtrim(cols,2)+'x'+strtrim(rows,2)
  message,/inform,plotno+' at '+coords
  endif
;stop

return
end
